/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"
#include "Cdimse.h"
#include "DcmFile.h"

#include "Pdu.h"

#include <algorithm>
#include <iostream>

using std::for_each;
using std::cout;
using std::ios;
using std::find;
using std::string;


CEchoRQ::CEchoRQ(UINT16 msgID, const std::string& classUID)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_ECHO_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
}

CEchoRSP::CEchoRSP(UINT16 msgID, const std::string& classUID)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_ECHO_RSP);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
	insertElem(TAG_STATUS, ST_SUCCESS);
}

CStoreRQ::CStoreRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
	UINT16 priority)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_STORE_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_PRIORITY, priority);
	insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
	insertElem(TAG_AFF_SOP_INST_UID, VR_UI, instUID);
}

CStoreRQ::CStoreRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
	const std::string& moveAET, UINT16 moveMsgID, UINT16 priority)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_STORE_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_PRIORITY, priority);
	insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
	insertElem(TAG_AFF_SOP_INST_UID, VR_UI, instUID);
	insertElem(TAG_MOVE_ORIG_AET, VR_AE, moveAET);
	insertElem(TAG_MOVE_ORIG_MSG_ID, moveMsgID);
}

CStoreRSP::CStoreRSP(UINT16 msgID, const std::string& classUID, UINT16 stat)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_STORE_RSP);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
	insertElem(TAG_STATUS, stat);
}

CFindRQ::CFindRQ(UINT16 msgID, const std::string& classUID, UINT16 priority)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_FIND_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_PRIORITY, priority);
	insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
}

CFindRSP::CFindRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_FIND_RSP);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, dsType);
	insertElem(TAG_STATUS, stat);
}

CCancelRQ::CCancelRQ(UINT16 msgID)
{
	insertElem(TAG_CMD_FIELD, C_CANCEL_RQ);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
}

CGetRQ::CGetRQ(UINT16 msgID, const std::string& classUID, UINT16 priority)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_GET_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_PRIORITY, priority);
	insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
}

CGetRSP::CGetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_GET_RSP);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, dsType);
	insertElem(TAG_STATUS, stat);
}

void CGetRSP::setRemaining(UINT16 n)
{
	insertElem(TAG_NUM_REMAIN_SUBOP, n);
}

void CGetRSP::setCompleted(UINT16 n)
{
	insertElem(TAG_NUM_COMPL_SUBOP, n);
}

void CGetRSP::setFailed(UINT16 n)
{
	insertElem(TAG_NUM_FAIL_SUBOP, n);
}

void CGetRSP::setWarning(UINT16 n)
{
	insertElem(TAG_NUM_WARN_SUBOP, n);
}

CMoveRQ::CMoveRQ(UINT16 msgID, const std::string& classUID, const std::string& destAET,
	UINT16 priority)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_MOVE_RQ);
	insertElem(TAG_MSG_ID, msgID);
	insertElem(TAG_PRIORITY, priority);
	insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
	insertElem(TAG_MOVE_DEST, VR_AE, destAET);
}

CMoveRSP::CMoveRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType)
{
	insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
	insertElem(TAG_CMD_FIELD, C_MOVE_RSP);
	insertElem(TAG_MSG_ID_RSP, msgID);
	insertElem(TAG_DATA_SET_TYPE, dsType);
	insertElem(TAG_STATUS, stat);
}

void CMoveRSP::setRemaining(UINT16 n)
{
	insertElem(TAG_NUM_REMAIN_SUBOP, n);
}

void CMoveRSP::setCompleted(UINT16 n)
{
	insertElem(TAG_NUM_COMPL_SUBOP, n);
}

void CMoveRSP::setFailed(UINT16 n)
{
	insertElem(TAG_NUM_FAIL_SUBOP, n);
}

void CMoveRSP::setWarning(UINT16 n)
{
	insertElem(TAG_NUM_WARN_SUBOP, n);
}

bool CEchoSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
	UINT16 msgID = 0;
	if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
		return false;

	CEchoRSP rspCmd(msgID, classUID);
	return pdu.WriteCommand(rspCmd, classUID) != false;
}

bool CStoreSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
	UINT16 msgID = 0;
	if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
		return false;

	UINT16 dstype = 0;
	if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
		return false;

	if (dstype == NO_DATA_SET)
		return false;

	DataSet rqData;
	if (pdu.Read(rqData) == false)
		return false;

	UINT16 stat = ST_SUCCESS;
	if (m_pHandler)
		stat = m_pHandler->handle(pdu.GetSocketfd(), rqCmd, rqData, 0);

	CStoreRSP rspCmd(msgID, classUID, stat);
	return pdu.WriteCommand(rspCmd, classUID) != false;
}

bool CFindSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
	UINT16 msgID = 0;
	if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
		return false;

	UINT16 dstype = 0;
	if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
		return false;

	if (dstype == NO_DATA_SET)
		return false;

	DataSet rqData;
	if (pdu.Read(rqData) == false)
		return false;

	UINT16 stat = ST_SUCCESS;
	if (m_pHandler)
	{
		stat = m_pHandler->handle(pdu.GetSocketfd(), rqCmd, rqData, &m_sq);
		if (stat == ST_SUCCESS)
		{
			for (; !m_sq.empty(); m_sq.pop_back())
			{
				DataSet& rspData = *m_sq.back();
				CFindRSP rspCmd(msgID, classUID, ST_PENDING, YES_DATA_SET);
				pdu.WriteCommand(rspCmd, classUID);
				pdu.WriteDataSet(rspData, classUID);
			}
		}
	}
	CFindRSP rspCmd(msgID, classUID, stat, NO_DATA_SET);
	return pdu.WriteCommand(rspCmd, classUID) != false;
}

bool CGetSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
	UINT16 msgID = 0;
	if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
		return false;

	UINT16 dstype = 0;
	if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
		return false;

	if (dstype == NO_DATA_SET)
		return false;

	DataSet rqData;
	if (pdu.Read(rqData) == false)
		return false;

	UINT16 stat = ST_SUCCESS;
	UINT16 remaining = 0;
	UINT16 completed = 0;
	UINT16 failed = 0;
	UINT16 warning = 0;
	if (m_pHandler)
	{
		stat = m_pHandler->handle(pdu.GetSocketfd(), rqCmd, rqData, &m_sq);
		printf( "Here!" );
		if ( stat == ST_SUCCESS )
		{
			//TODO
			std::string	sHost;
			int		iPort;
			std::string	sImage;
			for ( ; !m_sq.empty( ); m_sq.pop_back( ) ) {
				DataSet& rspData = *m_sq.back();
				if ( !rspData.getSafeStr( &sHost, TAG_MOVE_DEST, VR_AE ) ) break;
				if ( !rspData.getSafeStr( &sImage, TAG_NULL, VR_UI ) ) break;
				if ( !rspData.getSafeSL( &iPort, TAG_PRIORITY, VR_SS ) ) break;
				printf( "Moving %s to [ %s : %d ]\n", sImage.c_str( ), sHost.c_str( ), iPort );
			}
		}
	}
	CGetRSP rspCmd(msgID, classUID, stat, NO_DATA_SET);
	rspCmd.setCompleted(completed);
	rspCmd.setFailed(failed);
	rspCmd.setWarning(warning);
	return pdu.WriteCommand(rspCmd, classUID) != false;
}

static const char* STORE_SOP[] = {
	UID_CR_IMAGE_STORAGE_SOP_CLASS,
	UID_CT_IMAGE_STORAGE_SOP_CLASS,
	UID_USOLD_MF_IMAGE_STORAGE_SOP_CLASS,
	UID_US_MF_IMAGE_STORAGE_SOP_CLASS,
	UID_MR_IMAGE_STORAGE_SOP_CLASS,
	UID_USOLD_IMAGE_STORAGE_SOP_CLASS,
	UID_US_IMAGE_STORAGE_SOP_CLASS,
	UID_SC_IMAGE_STORAGE_SOP_CLASS,
	UID_NM_IMAGE_STORAGE_SOP_CLASS,
	UID_XA_IMAGE_STORAGE_SOP_CLASS,
	UID_XRF_IMAGE_STORAGE_SOP_CLASS,
	UID_XA2_IMAGE_STORAGE_SOP_CLASS,
	UID_PET_IMAGE_STORAGE_SOP_CLASS,
	UID_DX_PRES_IMAGE_STORAGE_SOP_CLASS,
	UID_DX_PROC_IMAGE_STORAGE_SOP_CLASS,
	UID_MAMMO_PRES_IMAGE_STORAGE_SOP_CLASS,
	UID_MAMMO_PROC_IMAGE_STORAGE_SOP_CLASS,
	UID_BASIC_TEXT_SR_STORAGE_SOP_CLASS,
	UID_ENHANCED_SR_STORAGE_SOP_CLASS,
	UID_COMPREHENSIVE_SR_STORAGE_SOP_CLASS,
	UID_INTRAORAL_PRES_IMAGE_STORAGE_SOP_CLASS,
	UID_INTRAORAL_PROC_IMAGE_STORAGE_SOP_CLASS
};

static const int STORE_SOP_NUM = sizeof( STORE_SOP ) / sizeof( STORE_SOP[ 0 ] );

static const char* STORE_SOP_DESC[STORE_SOP_NUM+1] = {
	"Computed Radiography Image Storage",
	"CT Image Storage",
	"Ultrasound Multi-frame Image Storage (Retired)",
	"Ultrasound Multi-frame Image Storage",
	"MR Image Storage",
	"Ultrasound Image Storage (Retired)",
	"Ultrasound Image Storage",
	"Secondary Capture Image Storage",
	"Nuclear Medicine Image Storage",
	"X-Ray Angiographic Image Storage",
	"X-Ray Radiofluoroscopic Image Storage",
	"X-Ray Angiographic Bi-Plane Image Storage (Retired)",
	"Positron Emission Tomography Image Storage",
	"Digital X-Ray Image Storage - For Presentation",
	"Digital X-Ray Image Storage - For Processing",
	"Digital Mammography X-Ray Image Storage - For Presentation",
	"Digital Mammography X-Ray Image Storage - For Processing",
	"Basic Text Structured Report Storage",
	"Enhanced Structured Report Storage",
	"Comprehensive Structured Report Storage",
	"Digital Intra-oral X-Ray Image Storage - For Presentation",
	"Digital Intra-oral X-Ray Image Storage - For Processing",
	"Unkwown Image Storage"
};

struct AuxAddAbstractSyntax {
  PDU_Service& m_pdu;
  AuxAddAbstractSyntax( PDU_Service& pdu ) : m_pdu( pdu ) { }
  void operator( ) ( const char* uid ) { m_pdu.AddAbstractSyntax( uid ); }
};

bool	openAssociate( PDU_Service& m_pdu, std::string sHost, std::string sPort, std::vector<bool> &m_storeSOP ) {
	m_pdu.ClearAbstractSyntaxs();
	m_pdu.SetLocalAddress( "Dicomsvr" );
	m_pdu.SetRemoteAddress( "Remote CSTORER" );
	m_pdu.SetTimeOut( 300 );
	m_pdu.SetApplicationContext( UID_APPLICATION_CONTEXT );
	m_pdu.AddAbstractSyntax( UID_VERIFICATION_SOP_CLASS );
	for_each( STORE_SOP, STORE_SOP + STORE_SOP_NUM, AuxAddAbstractSyntax( m_pdu ) );

	if( !m_pdu.Connect( ( LPCSTR ) sHost.c_str( ), ( LPCSTR ) sPort.c_str( ) ) ) {
		// Connect failed
		printf( "Connection failed!\n" );
		return false;
	} else {
		for (int i = 0; i < STORE_SOP_NUM; ++i) {
			m_storeSOP[i] = m_pdu.GetPresentationContextID( STORE_SOP[i], UID_IMPL_VR_LE_TRANSFER_SYNTAX ) != 0;
		}
    }
	return true;
}

bool CMoveSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
	UINT16 msgID;
	if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
		return false;

	UINT16 dstype = 0;
	if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
		return false;

	if (dstype == NO_DATA_SET)
		return false;

	DataSet rqData;
	if (pdu.Read(rqData) == false)
		return false;

	UINT16 stat = ST_SUCCESS;
	UINT16 remaining = 0;
	UINT16 completed = 0;
	UINT16 failed = 0;
	UINT16 total = 0;
	UINT16 warning = 0;
	if (m_pHandler)
	{
		stat = m_pHandler->handle(pdu.GetSocketfd(), rqCmd, rqData, &m_sq);
		if ( ( stat == ST_SUCCESS ) && !m_sq.empty( ) )
		{
			//TODO
			std::string	sHost;
			std::string	sPort;
			std::string	sImage;
			std::string	sFileName;
			PDU_Service m_pdu;
			std::vector<bool> m_storeSOP( STORE_SOP_NUM+1, false );
			// First dataset contains destination.
			DataSet& rspData = *m_sq.back();
			if ( !rspData.getSafeStr( &sPort, TAG_PAT_ID, VR_UI ) ) sHost = "";
			if ( !rspData.getSafeStr( &sHost, TAG_MOVE_DEST, VR_AE ) ) sPort = "";
			m_sq.pop_back( );
			// Open associate
			total = m_sq.size( );
			if ( !openAssociate( m_pdu, sHost, sPort, m_storeSOP ) ) sHost = "";
			for ( ; !m_sq.empty( ); m_sq.pop_back( ) ) {
				/*
					Why dose the following line use 'new'?  The object could equally
					well be created on the heap, i.e.
					CMoveRSP rspCmd1(msgID,classUID,ST_PENDING,NO_DATA_SET);
				*/
				CMoveRSP *rspCmd1 = new CMoveRSP( msgID, classUID, ST_PENDING, NO_DATA_SET );
				rspCmd1->setRemaining( total - completed - failed - warning );
				rspCmd1->setCompleted( completed );
				rspCmd1->setFailed( failed );
				rspCmd1->setWarning( warning );
				if ( !pdu.WriteCommand( *rspCmd1, classUID ) ) {
					delete rspCmd1;
					break;
				}
				delete rspCmd1;
				DataSet& rspData = *m_sq.back();
				if ( !rspData.getSafeStr( &sImage, TAG_NULL, VR_UI ) ) break;
				if ( !rspData.getSafeStr( &sFileName, TAG_FILE_NAME, VR_LO ) ) break;
				if ( ( sHost.length( ) == 0 ) || ( sPort.length( ) == 0 ) || ( sImage.length( ) == 0 ) ) break;
				cout << "Moving " << sImage << " to [" << sHost << ":" << sPort << "]" << std::endl;
				// CStore a file
				DcmFile dcmf;
				dcmf.open( sImage.c_str( ), ios::binary | ios::in );
				if ( !dcmf.is_open() ) {
					printf( "Opening image failed -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}
				DataSet ds;
				if ( !dcmf.read( ds ) ) {
					printf( "Reading image failed -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}
				ds.insertElem( TAG_FILE_NAME, VR_LO, sFileName );
				std::string instUid;
				if ( !ds.getSafeStr( &instUid, TAG_SOP_INST_UID, VR_UI ) || instUid.empty( ) ) {
					printf( "Reading image failed [Missing Attribut (0008,0018) SOP Instance UID] -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}
				std::string classUid;
				if ( !ds.getSafeStr( &classUid, TAG_SOP_CLASS_UID, VR_UI ) || classUid.empty( ) ) {
					printf( "Reading image failed [Missing Attribut (0008,0016) SOP Class UID] -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}

				int index = find( STORE_SOP, STORE_SOP + STORE_SOP_NUM, classUid ) - STORE_SOP;
				if ( !m_storeSOP[ index ] )	{
					printf( "Reading image failed [Not supported %s ] -- %s\n", sImage.c_str( ), STORE_SOP_DESC[index] );
					failed ++;
					continue;
				}
				ds.insertElem( TAG_FILE_NAME, VR_LO, sFileName );
				CStoreSCU storage( classUid );
				if ( !storage.writeRQ( m_pdu, instUid, ds, 0 ) ) {
					printf( "Writing request failed -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}
				UINT16 stat;
				DataSet rsp;
				if ( !storage.readRSP( &stat, &rsp, m_pdu ) ) {
					printf( "Reading response failed -- %s\n", sImage.c_str( )  );
					failed ++;
					continue;
				}
				completed ++;
			}
			m_pdu.Close( );
			remaining = total - completed - failed - warning;
		}
	}
	CMoveRSP rspCmd(msgID, classUID, stat, NO_DATA_SET);
	rspCmd.setCompleted(completed);
	rspCmd.setFailed(failed);
	rspCmd.setWarning(warning);
	return pdu.WriteCommand(rspCmd, classUID) != false;
}

CEchoSCU::CEchoSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool CEchoSCU::writeRQ(PDU_Service& pdu)
{
	CEchoRQ rq(uniq16odd(), m_classUID);
	return pdu.WriteCommand(rq, m_classUID) != false;
}

bool CEchoSCU::readRSP(UINT16* stat_p, PDU_Service& pdu)
{
	DataSet rsp;
	return readRSP(stat_p, &rsp, pdu);
}

bool CEchoSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, PDU_Service& pdu)
{
	return pdu.Read(*rsp_p) != false &&
		rsp_p->getSafeUS(stat_p, TAG_STATUS);
}

CStoreSCU::CStoreSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool CStoreSCU::writeRQ(PDU_Service& pdu,
						const std::string& instUID, const DataSet& data, UINT16 priority)
{
	CStoreRQ rq(uniq16odd(), m_classUID, instUID, priority);
	return pdu.WriteCommand(rq, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool CStoreSCU::readRSP(UINT16* stat_p, PDU_Service& pdu)
{
	DataSet rsp;
	return readRSP(stat_p, &rsp, pdu);
}

bool CStoreSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, PDU_Service& pdu)
{
	return pdu.Read(*rsp_p) != false &&
		rsp_p->getSafeUS(stat_p, TAG_STATUS);
}

CFindSCU::CFindSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool CFindSCU::writeRQ(PDU_Service& pdu,
						const DataSet& data, UINT16 priority)
{
	CFindRQ rq(uniq16odd(), m_classUID, priority);
	return pdu.WriteCommand(rq, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool CFindSCU::readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu)
{
	DataSet rsp;
	return readRSP(stat_p, &rsp, data_p, pdu);
}

bool CFindSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu)
{
	UINT16 dstype = 0;
	return pdu.Read(*rsp_p) != false &&
		rsp_p->getSafeUS(&dstype, TAG_DATA_SET_TYPE) &&
		rsp_p->getSafeUS(stat_p, TAG_STATUS) &&
		(dstype != NO_DATA_SET ? pdu.Read(*data_p) != false : true);
}

CGetSCU::CGetSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool CGetSCU::writeRQ(PDU_Service& pdu,
						const DataSet& data, UINT16 priority)
{
	CGetRQ rq(uniq16odd(), m_classUID, priority);
	return pdu.WriteCommand(rq, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool CGetSCU::readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu)
{
	DataSet rsp;
	return readRSP(stat_p, &rsp, data_p, pdu);
}

bool CGetSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu)
{
	UINT16 dstype = 0;
	if (pdu.Read(*rsp_p) == false ||
		!rsp_p->getSafeUS(&dstype, TAG_DATA_SET_TYPE) ||
		!rsp_p->getSafeUS(stat_p, TAG_STATUS) ||
		(dstype != NO_DATA_SET && pdu.Read(*data_p) == false))
		return false;
	return true;
}

CMoveSCU::CMoveSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool CMoveSCU::writeRQ(PDU_Service& pdu, const string& destAET,
						const DataSet& data, UINT16 priority)
{
	CMoveRQ rq(uniq16odd(), m_classUID, destAET, priority);
	return pdu.WriteCommand(rq, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool CMoveSCU::readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu)
{
	DataSet rsp;
	return readRSP(stat_p, &rsp, data_p, pdu);
}

bool CMoveSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu)
{
	UINT16 dstype = 0;
	if (pdu.Read(*rsp_p) == false ||
		!rsp_p->getSafeUS(&dstype, TAG_DATA_SET_TYPE) ||
		!rsp_p->getSafeUS(stat_p, TAG_STATUS) ||
		(dstype != NO_DATA_SET && pdu.Read(*data_p) == false))
		return false;

	return true;
}

