#ifndef __CDIMSE_H__
#define __CDIMSE_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "DataSet.h"
#include "Uid.h"

class PDU_Service;

class CEchoRQ : public DataSet  
{
public:
	CEchoRQ(UINT16 msgID, const std::string& classUID);
};

class CEchoRSP : public DataSet  
{
public:
	CEchoRSP(UINT16 msgID, const std::string& classUID);
};

class CStoreRQ : public DataSet  
{
public:
	CStoreRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 priority = PR_MEDIUM);
	CStoreRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		const std::string& moveAET, UINT16 moveMsgID,
		UINT16 priority = PR_MEDIUM);
};

class CStoreRSP : public DataSet  
{
public:
	CStoreRSP(UINT16 msgID, const std::string& classUID, UINT16 stat);
};

class CFindRQ : public DataSet  
{
public:
	CFindRQ(UINT16 msgID, const std::string& classUID, UINT16 priority = PR_MEDIUM);
};

class CFindRSP : public DataSet  
{
public:
	CFindRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType);
};

class CCancelRQ : public DataSet  
{
public:
	CCancelRQ(UINT16 msgID);
};

class CGetRQ : public DataSet  
{
public:
	CGetRQ(UINT16 msgID, const std::string& classUID, UINT16 priority = PR_MEDIUM);
};

class CGetRSP : public DataSet  
{
public:
	CGetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType);
	void setRemaining(UINT16 n);
	void setCompleted(UINT16 n);
	void setFailed(UINT16 n);
	void setWarning(UINT16 n);
};

class CMoveRQ : public DataSet  
{
public:
	CMoveRQ(UINT16 msgID, const std::string& classUID, const std::string& destAET,
		UINT16 priority = PR_MEDIUM);
};

class CMoveRSP : public DataSet  
{
public:
	CMoveRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType);
	void setRemaining(UINT16 n);
	void setCompleted(UINT16 n);
	void setFailed(UINT16 n);
	void setWarning(UINT16 n);
};

class CHandler
{
public:
	virtual UINT16 handle(int sock, const DataSet& rq, const DataSet& data, Sequence* sq) = 0;
};

class CEchoSCP
{
public:
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class CStoreSCP
{
	CHandler* m_pHandler;
public:
	CStoreSCP(CHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class CFindSCP
{
	CHandler* m_pHandler;
	Sequence m_sq;
public:
	CFindSCP(CHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class CGetSCP
{
	CHandler* m_pHandler;
	Sequence m_sq;
public:
	CGetSCP(CHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class CMoveSCP
{
	CHandler* m_pHandler;
	Sequence m_sq;
public:
	CMoveSCP(CHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class CEchoSCU  
{
	const std::string m_classUID;
public:
	CEchoSCU(const std::string& classUID = UID_VERIFICATION_SOP_CLASS);
	bool writeRQ(PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, PDU_Service& pdu);
};

class CStoreSCU  
{
	const std::string m_classUID;
public:
	CStoreSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu, const std::string& instUID,
		const DataSet& data, UINT16 priority = PR_MEDIUM);
	bool readRSP(UINT16* stat_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, PDU_Service& pdu);
};

class CFindSCU  
{
	const std::string m_classUID;
public:
	CFindSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu,
		const DataSet& data, UINT16 priority = PR_MEDIUM);
	bool readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu);
};

class CGetSCU  
{
	const std::string m_classUID;
public:
	CGetSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu,
		const DataSet& data, UINT16 priority = PR_MEDIUM);
	bool readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu);
};

class CMoveSCU  
{
	const std::string m_classUID;
public:
	CMoveSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu, const std::string& destAET,
		const DataSet& data, UINT16 priority = PR_MEDIUM);
	bool readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu);
};

#endif //__CDIMSE_H__