// DataDict.cpp: implementation of the DataDict class.
//
//////////////////////////////////////////////////////////////////////

#include "base.hpp"
#include "DataDict.h"

struct DictEntry
{
	UINT32 tag;
	VR vr;
	const char* name;
};

static const DictEntry DICT_ENTRY0[] =
{
	0x00000000,VR_UL,"GroupLength",
	0x00000002,VR_UI,"AffectedSOPClassUID",
	0x00000003,VR_UI,"RequestedSOPClassUID",
	0x00000100,VR_US,"CommandField",
	0x00000110,VR_US,"MessageID",
	0x00000120,VR_US,"MessageIDBeingRespondedTo",
	0x00000600,VR_AE,"MoveDestination",
	0x00000700,VR_US,"Priority",
	0x00000800,VR_US,"DataSetType",
	0x00000900,VR_US,"Status",
	0x00000901,VR_AT,"OffendingElement",
	0x00000902,VR_LO,"ErrorComment",
	0x00000903,VR_US,"ErrorID",
	0x00001000,VR_UI,"AffectedSOPInstanceUID",
	0x00001001,VR_UI,"RequestedSOPInstanceUID",
	0x00001002,VR_US,"EventTypeID",
	0x00001005,VR_AT,"AttributeIdentifierList",
	0x00001008,VR_US,"ActionTypeID",
	0x00001020,VR_US,"NumberofRemainingSub-operations",
	0x00001021,VR_US,"NumberofCompletedSub-operations",
	0x00001022,VR_US,"NumberofFailedSub-operations",
	0x00001023,VR_US,"NumberofWarningSub-operations",
	0x00001030,VR_AE,"MoveOriginatorApplicationEntityTitle",
	0x00001031,VR_US,"MoveOriginatorMessageID",
};

static const DictEntry DICT_ENTRY2[] =
{
	0x00020000,VR_UL,"GroupLength",
	0x00020001,VR_OB,"FileMetaInformationVersion",
	0x00020002,VR_UI,"MediaStorageSOPClassUID",
	0x00020003,VR_UI,"MediaStorageSOPInstanceUID",
	0x00020010,VR_UI,"TransferSyntaxUID",
	0x00020012,VR_UI,"ImplementationClassUID",
	0x00020013,VR_SH,"ImplementationVersionName",
	0x00020016,VR_AE,"SourceApplicationEntityTitle",
	0x00020100,VR_UI,"PrivateInformationCreatorUID",
	0x00020102,VR_OB,"PrivateInformation",
};

static const DictEntry DICT_ENTRY4[] =
{
	0x00040000,VR_UL,"GroupLength",
	0x00041130,VR_CS,"File-setID",
	0x00041141,VR_CS,"File-setDescriptorFileID",
	0x00041142,VR_CS,"SpecificCharacterSetofFile-setDescriptorFile",
	0x00041200,VR_UL,"OffsetoftheFirstDirectoryRecordoftheRootDirectoryEntity",
	0x00041202,VR_UL,"OffsetoftheLastDirectoryRecordoftheRootDirectoryEntity",
	0x00041212,VR_US,"File-setConsistencyFlag",
	0x00041220,VR_SQ,"DirectoryRecordSequence",
	0x00041400,VR_UL,"OffsetoftheNextDirectoryRecord",
	0x00041410,VR_US,"RecordIn-useFlag",
	0x00041420,VR_UL,"OffsetofReferencedLower-LevelDirectoryEntity",
	0x00041430,VR_CS,"DirectoryRecordType",
	0x00041432,VR_UI,"PrivateRecordUID",
	0x00041500,VR_CS,"ReferencedFileID",
	0x00041504,VR_UL,"MRDRDirectoryRecordOffset",
	0x00041510,VR_UI,"ReferencedSOPClassUIDinFile",
	0x00041511,VR_UI,"ReferencedSOPInstanceUIDinFile",
	0x00041512,VR_UI,"ReferencedTransferSyntaxUIDinFile",
	0x00041600,VR_UL,"NumberofReferences",
};

static const DictEntry DICT_ENTRY[] =
{
	0x00080000,VR_UL,"GroupLength",
	0x00080005,VR_CS,"SpecificCharacterSet",
	0x00080008,VR_CS,"ImageType",
	0x00080012,VR_DA,"InstanceCreationDate",
	0x00080013,VR_TM,"InstanceCreationTime",
	0x00080014,VR_UI,"InstanceCreatorUID",
	0x00080016,VR_UI,"SOPClassUID",
	0x00080018,VR_UI,"SOPInstanceUID",
	0x00080020,VR_DA,"StudyDate",
	0x00080021,VR_DA,"SeriesDate",
	0x00080022,VR_DA,"AcquisitionDate",
	0x00080023,VR_DA,"Image/ContentDate",
	0x00080024,VR_DA,"OverlayDate",
	0x00080025,VR_DA,"CurveDate",
	0x00080030,VR_TM,"StudyTime",
	0x00080031,VR_TM,"SeriesTime",
	0x00080032,VR_TM,"AcquisitionTime",
	0x00080033,VR_TM,"Image/ContentTime",
	0x00080034,VR_TM,"OverlayTime",
	0x00080035,VR_TM,"CurveTime",
	0x00080050,VR_SH,"AccessionNumber",
	0x00080052,VR_CS,"Query/RetrieveLevel",
	0x00080054,VR_AE,"RetrieveAETitle",
	0x00080058,VR_UI,"FailedSOPInstanceUIDList",
	0x00080060,VR_CS,"Modality",
	0x00080061,VR_CS,"ModalitiesinStudy",
	0x00080064,VR_CS,"ConversionType",
	0x00080068,VR_CS,"PresentationIntentType",
	0x00080070,VR_LO,"Manufacturer",
	0x00080080,VR_LO,"InstitutionName",
	0x00080081,VR_ST,"InstitutionAddress",
	0x00080082,VR_SQ,"InstitutionCodeSequence",
	0x00080090,VR_PN,"ReferringPhysiciansName",
	0x00080092,VR_ST,"ReferringPhysician'sAddress",
	0x00080094,VR_SH,"ReferringPhysiciansTelephoneNumbers",
	0x00080100,VR_SH,"CodeValue",
	0x00080102,VR_SH,"CodingSchemeDesignator",
	0x00080104,VR_LO,"CodeMeaning",
	0x0008010c,VR_UI,"PrivateSchemeCreatorUID",
	0x00081010,VR_SH,"StationName",
	0x00081030,VR_LO,"StudyDescription",
	0x00081032,VR_SQ,"ProcedureCodeSequence",
	0x0008103E,VR_LO,"SeriesDescription",
	0x00081040,VR_LO,"InstitutionalDepartmentName",
	0x00081048,VR_PN,"Physician(s)ofRecord",
	0x00081050,VR_PN,"PerformingPhysiciansName",
	0x00081060,VR_PN,"NameofPhysician(s)ReadingStudy",
	0x00081070,VR_PN,"OperatorsName",
	0x00081080,VR_LO,"AdmittingDiagnosesDescription",
	0x00081084,VR_SQ,"AdmittingDiagnosisCodeSequence",
	0x00081090,VR_LO,"ManufacturersModelName",
	0x00081100,VR_SQ,"ReferencedResultsSequence",
	0x00081110,VR_SQ,"ReferencedStudySequence",
	0x00081111,VR_SQ,"ReferencedStudyComponentSequence",
	0x00081115,VR_SQ,"ReferencedSeriesSequence",
	0x00081120,VR_SQ,"ReferencedPatientSequence",
	0x00081125,VR_SQ,"ReferencedVisitSequence",
	0x00081130,VR_SQ,"ReferencedOverlaySequence",
	0x00081140,VR_SQ,"ReferencedImageSequence",
	0x00081145,VR_SQ,"ReferencedCurveSequence",
	0x00081150,VR_UI,"ReferencedSOPClassUID",
	0x00081155,VR_UI,"ReferencedSOPInstanceUID",
	0x00081160,VR_IS,"ReferencedFrameNumber",
	0x00081195,VR_UI,"TransactionUID",
	0x00081197,VR_US,"FailureReason",
	0x00081198,VR_SQ,"FailedSOPSequence",
	0x00081199,VR_SQ,"ReferencedSOPSequence",
	0x00082111,VR_ST,"DerivationDescription",
	0x00082112,VR_SQ,"SourceImageSequence",
	0x00082120,VR_SH,"StageName",
	0x00082122,VR_IS,"StageNumber",
	0x00082124,VR_IS,"NumberofStages",
	0x00082128,VR_IS,"ViewNumber",
	0x00082129,VR_IS,"NumberofEventTimers",
	0x0008212A,VR_IS,"NumberofViewsinStage",
	0x00082130,VR_DS,"EventElapsedTime(s)",
	0x00082132,VR_LO,"EventTimerName(s)",
	0x00082142,VR_IS,"StartTrim",
	0x00082143,VR_IS,"StopTrim",
	0x00082144,VR_IS,"RecommendedDisplayFrameRate",
	0x00082218,VR_SQ,"AnatomicRegionSequence",
	0x00082220,VR_SQ,"AnatomicRegionModifierSequence",
	0x00082228,VR_SQ,"PrimaryAnatomicStructureSequence",
	0x00082229,VR_SQ,"AnatomicStructure,SpaceorRegionSequence",
	0x00082230,VR_SQ,"PrimaryAnatomicStructureModifierSequence",
	0x00082240,VR_SQ,"TransducerPositionSequence",
	0x00082242,VR_SQ,"TransducerPositionModifierSequence",
	0x00082244,VR_SQ,"TransducerOrientationSequence",
	0x00082246,VR_SQ,"TransducerOrientationModifierSequence",
	0x00100000,VR_UL,"GroupLength",
	0x00100010,VR_PN,"PatientsName",
	0x00100020,VR_LO,"PatientID",
	0x00100021,VR_LO,"IssuerofPatientID",
	0x00100030,VR_DA,"Patient'sBirthDate",
	0x00100032,VR_TM,"Patient'sBirthTime",
	0x00100040,VR_CS,"Patient'sSex",
	0x00100050,VR_SQ,"Patient'sInsurancePlanCodeSequence",
	0x00101000,VR_LO,"OtherPatientIDs",
	0x00101001,VR_PN,"OtherPatientNames",
	0x00101005,VR_PN,"Patient'sBirthName",
	0x00101010,VR_AS,"Patient'sAge",
	0x00101020,VR_DS,"Patient'sSize",
	0x00101030,VR_DS,"Patient'sWeight",
	0x00101040,VR_LO,"Patient'sAddress",
	0x00101060,VR_PN,"Patient'sMother'sBirthName",
	0x00101080,VR_LO,"MilitaryRank",
	0x00101081,VR_LO,"BranchofService",
	0x00101090,VR_LO,"MedicalRecordLocator",
	0x00102000,VR_LO,"MedicalAlerts",
	0x00102110,VR_LO,"ContrastAllergies",
	0x00102150,VR_LO,"CountryofResidence",
	0x00102152,VR_LO,"RegionofResidence",
	0x00102154,VR_SH,"PatientsTelephoneNumbers",
	0x00102160,VR_SH,"EthnicGroup",
	0x00102180,VR_SH,"Occupation",
	0x001021A0,VR_CS,"SmokingStatus",
	0x001021B0,VR_LT,"AdditionalPatientHistory",
	0x001021C0,VR_US,"PregnancyStatus",
	0x001021D0,VR_DA,"LastMenstrualDate",
	0x001021F0,VR_LO,"Patient'sReligiousPreference",
	0x00104000,VR_LT,"PatientComments",
	0x00180000,VR_UL,"GroupLength",
	0x00180010,VR_LO,"Contrast/BolusAgent",
	0x00180012,VR_SQ,"Contrast/BolusAgentSequence",
	0x00180014,VR_SQ,"Contrast/BolusAdministrationRouteSequence",
	0x00180015,VR_CS,"BodyPartExamined",
	0x00180020,VR_CS,"ScanningSequence",
	0x00180021,VR_CS,"SequenceVariant",
	0x00180022,VR_CS,"ScanOptions",
	0x00180023,VR_CS,"MRAcquisitionType",
	0x00180024,VR_SH,"SequenceName",
	0x00180025,VR_CS,"AngioFlag",
	0x00180026,VR_SQ,"InterventionDrugInformationSequence",
	0x00180027,VR_TM,"InterventionDrugStopTime",
	0x00180028,VR_DS,"InterventionDrugDose",
	0x00180029,VR_SQ,"InterventionDrugCodeSequence",
	0x0018002A,VR_SQ,"AdditionalDrugSequence",
	0x00180031,VR_LO,"Radiopharmaceutical",
	0x00180034,VR_LO,"InterventionDrugName",
	0x00180035,VR_TM,"InterventionDrugStartTime",
	0x00180036,VR_SQ,"InterventionalTherapySequence",
	0x00180037,VR_CS,"TherapyType",
	0x00180038,VR_CS,"InterventionalStatus",
	0x00180039,VR_CS,"TherapyDescription",
	0x00180040,VR_IS,"CineRate",
	0x00180050,VR_DS,"SliceThickness",
	0x00180059,VR_US,"SlantAngle",  //added by Sam Shen for GMD QC
	0x00180060,VR_DS,"KVP",
	0x00180070,VR_IS,"CountsAccumulated",
	0x00180071,VR_CS,"AcquisitionTerminationCondition",
	0x00180072,VR_DS,"EffectiveSeriesDuration",
	0x00180073,VR_CS,"AcquisitionStartCondition",
	0x00180074,VR_IS,"AcquisitionStartConditionData",
	0x00180075,VR_IS,"AcquisitionTerminationConditionData",
	0x00180080,VR_DS,"RepetitionTime",
	0x00180081,VR_DS,"EchoTime",
	0x00180082,VR_DS,"InversionTime",
	0x00180083,VR_DS,"NumberofAverages",
	0x00180084,VR_DS,"ImagingFrequency",
	0x00180085,VR_SH,"ImagedNucleus",
	0x00180086,VR_IS,"EchoNumber(s)",
	0x00180087,VR_DS,"MagneticFieldStrength",
	0x00180088,VR_DS,"SpacingBetweenSlices",
	0x00180089,VR_IS,"NumberofPhaseEncodingSteps",
	0x00180090,VR_DS,"DataCollectionDiameter",
	0x00180091,VR_IS,"EchoTrainLength",
	0x00180093,VR_DS,"PercentSampling",
	0x00180094,VR_DS,"PercentPhaseFieldofView",
	0x00180095,VR_DS,"PixelBandwidth",
	0x00181000,VR_LO,"DeviceSerialNumber",
	0x00181004,VR_LO,"PlateID",
	0x00181010,VR_LO,"SecondaryCaptureDeviceID",
	0x00181011,VR_LO,"HardcopyCreationDeviceID",
	0x00181012,VR_DA,"DateofSecondaryCapture",
	0x00181014,VR_TM,"TimeofSecondaryCapture",
	0x00181016,VR_LO,"SecondaryCaptureDeviceManufacturer",
	0x00181017,VR_LO,"HardcopyDeviceManufacturer",
	0x00181018,VR_LO,"SecondaryCaptureDeviceManufacturersModelName",
	0x00181019,VR_LO,"SecondaryCaptureDeviceSoftwareVersion(s)",
	0x0018101A,VR_LO,"HardcopyDeviceSoftwareVersion",
	0x0018101B,VR_LO,"HardcopyDeviceManfuacturer'sModelName",
	0x00181020,VR_LO,"SoftwareVersion(s)",
	0x00181022,VR_SH,"VideoImageFormatAcquired",
	0x00181023,VR_LO,"DigitalImageFormatAcquired",
	0x00181030,VR_LO,"ProtocolName",
	0x00181040,VR_LO,"Contrast/BolusRoute",
	0x00181041,VR_DS,"Contrast/BolusVolume",
	0x00181042,VR_TM,"Contrast/BolusStartTime",
	0x00181043,VR_TM,"Contrast/BolusStopTime",
	0x00181044,VR_DS,"Contrast/BolusTotalDose",
	0x00181045,VR_IS,"SyringeCounts",
	0x00181046,VR_DS,"ContrastFlowRate(s)",
	0x00181047,VR_DS,"ContrastFlowDuration(s)",
	0x00181048,VR_CS,"Contrast/BolusIngredient",
	0x00181049,VR_DS,"Contrast/BolusIngredientConcentration",
	0x00181050,VR_DS,"SpatialResolution",
	0x00181060,VR_DS,"TriggerTime",
	0x00181061,VR_LO,"TriggerSourceorType",
	0x00181062,VR_IS,"NominalInterval",
	0x00181063,VR_DS,"FrameTime",
	0x00181064,VR_LO,"FramingType",
	0x00181065,VR_DS,"FrameTimeVector",
	0x00181066,VR_DS,"FrameDelay",
	0x00181070,VR_LO,"RadiopharmaceuticalRoute",
	0x00181071,VR_DS,"RadiopharmaceuticalVolume",
	0x00181072,VR_TM,"RadiopharmaceuticalStartTime",
	0x00181073,VR_TM,"RadiopharmaceuticalStopTime",
	0x00181074,VR_DS,"RadionuclideTotalDose",
	0x00181075,VR_DS,"RadionuclideHalfLife",
	0x00181076,VR_DS,"RadionuclidePositronFraction",
	0x00181077,VR_DS,"RadiopharmaceuticalSpecificActivity",
	0x00181080,VR_CS,"BeatRejectionFlag",
	0x00181081,VR_IS,"LowR-RValue",
	0x00181082,VR_IS,"HighR-RValue",
	0x00181083,VR_IS,"IntervalsAcquired",
	0x00181084,VR_IS,"IntervalsRejected",
	0x00181085,VR_LO,"PVCRejection",
	0x00181086,VR_IS,"SkipBeats",
	0x00181088,VR_IS,"HeartRate",
	0x00181090,VR_IS,"CardiacNumberofImages",
	0x00181094,VR_IS,"TriggerWindow",
	0x00181100,VR_DS,"ReconstructionDiameter",
	0x00181110,VR_DS,"DistanceSourcetoDetector",
	0x00181111,VR_DS,"DistanceSourcetoPatient",
	0x00181114,VR_DS,"EstimatedRadiographicMagnificationFactor",
	0x00181120,VR_DS,"Gantry/DetectorTilt",
	0x00181121,VR_DS,"Gantry/DetectorSlew",
	0x00181130,VR_DS,"TableHeight",
	0x00181131,VR_DS,"TableTraverse",
	0x00181134,VR_CS,"TableMotion",
	0x00181135,VR_DS,"TableVerticalIncrement",
	0x00181136,VR_DS,"TableLateralIncrement",
	0x00181137,VR_DS,"TableLongitudinalIncrement",
	0x00181138,VR_DS,"TableAngle",
	0x0018113A,VR_CS,"TableType",
	0x00181140,VR_CS,"RotationDirection",
	0x00181141,VR_DS,"AngularPosition",
	0x00181142,VR_DS,"RadialPosition",
	0x00181143,VR_DS,"ScanArc",
	0x00181144,VR_DS,"AngularStep",
	0x00181145,VR_DS,"CenterofRotationOffset",
	0x00181147,VR_CS,"FieldofViewShape",
	0x00181149,VR_IS,"FieldofViewDimension(s)",
	0x00181150,VR_IS,"ExposureTime",
	0x00181151,VR_IS,"X-rayTubeCurrent",
	0x00181152,VR_IS,"Exposure",
	0x00181153,VR_IS,"ExposureinuAs",
	0x00181154,VR_DS,"AveragePulseWidth",
	0x00181155,VR_CS,"RadiationSetting",
	0x00181156,VR_CS,"RectificationType",
	0x0018115A,VR_CS,"RadiationMode",
	0x0018115E,VR_DS,"ImageAreaDoseProduct",
	0x00181160,VR_SH,"FilterType",
	0x00181161,VR_LO,"TypeofFilters",
	0x00181162,VR_DS,"IntensifierSize",
	0x00181164,VR_DS,"ImagerPixelSpacing",
	0x00181166,VR_CS,"Grid",
	0x00181170,VR_IS,"GeneratorPower",
	0x00181180,VR_SH,"Collimator/gridName",
	0x00181181,VR_CS,"CollimatorType",
	0x00181182,VR_IS,"FocalDistance",
	0x00181183,VR_DS,"XFocusCenter",
	0x00181184,VR_DS,"YFocusCenter",
	0x00181190,VR_DS,"FocalSpot(s)",
	0x00181191,VR_CS,"AnodeTargetMaterial",
	0x001811A0,VR_DS,"BodyPartThickness",
	0x001811A2,VR_DS,"CompressionForce",
	0x00181200,VR_DA,"DateofLastCalibration",
	0x00181201,VR_TM,"TimeofLastCalibration",
	0x00181210,VR_SH,"ConvolutionKernel",
	0x00181242,VR_IS,"ActualFrameDuration",
	0x00181243,VR_IS,"CountRate",
	0x00181244,VR_US,"PreferredPlaybackSequencing",
	0x00181250,VR_SH,"ReceivingCoil",
	0x00181251,VR_SH,"TransmittingCoil",
	0x00181260,VR_SH,"PlateType",
	0x00181261,VR_LO,"PhosphorType",
	0x00181300,VR_DS,"ScanVelocity",
	0x00181301,VR_CS,"WholeBodyTechnique",
	0x00181302,VR_IS,"ScanLength",
	0x00181310,VR_US,"AcquisitionMatrix",
	0x00181312,VR_CS,"PhaseEncodingDirection",
	0x00181314,VR_DS,"FlipAngle",
	0x00181315,VR_CS,"VariableFlipAngleFlag",
	0x00181316,VR_DS,"SAR",
	0x00181318,VR_DS,"dB/dt",
	0x00181400,VR_LO,"AcquisitionDeviceProcessingDescription",
	0x00181401,VR_LO,"AcquisitionDeviceProcessingCode",
	0x00181402,VR_CS,"CassetteOrientation",
	0x00181403,VR_CS,"CassetteSize",
	0x00181404,VR_US,"ExposuresonPlate",
	0x00181405,VR_IS,"RelativeX-rayExposure",
	0x00181450,VR_CS,"ColumnAngulation",
	0x00181460,VR_DS,"TomoLayerHeight",
	0x00181470,VR_DS,"TomoAngle",
	0x00181480,VR_DS,"TomoTime",
	0x00181490,VR_CS,"TomoType",
	0x00181491,VR_CS,"TomoClass",
	0x00181495,VR_IS,"NumberOfTomoSynthesisSourceImages",
	0x00181500,VR_CS,"PositionerMotion",
	0x00181508,VR_CS,"PositionerType",
	0x00181510,VR_DS,"PositionerPrimaryAngle",
	0x00181511,VR_DS,"PositionerSecondaryAngle",
	0x00181520,VR_DS,"PositionerPrimaryAngleIncrement",
	0x00181521,VR_DS,"PositionerSecondaryAngleIncrement",
	0x00181530,VR_DS,"DetectorPrimaryAngle",
	0x00181531,VR_DS,"DetectorSecondaryAngle",
	0x00181600,VR_CS,"ShutterShape",
	0x00181602,VR_IS,"ShutterLeftVerticalEdge",
	0x00181604,VR_IS,"ShutterRightVerticalEdge",
	0x00181606,VR_IS,"ShutterUpperHorizontalEdge",
	0x00181608,VR_IS,"ShutterLowerHorizontalEdge",
	0x00181610,VR_IS,"CenterofCircularShutter",
	0x00181612,VR_IS,"RadiusofCircularShutter",
	0x00181620,VR_IS,"VerticesofthePolygonalShutter",
	0x00181700,VR_CS,"CollimatorShape",
	0x00181702,VR_IS,"CollimatorLeftVerticalEdge",
	0x00181704,VR_IS,"CollimatorRightVerticalEdge",
	0x00181706,VR_IS,"CollimatorUpperHorizontalEdge",
	0x00181708,VR_IS,"CollimatorLowerHorizontalEdge",
	0x00181710,VR_IS,"CenterofCircularCollimator",
	0x00181712,VR_IS,"RadiusofCircularCollimator",
	0x00181720,VR_IS,"VerticesofthePolygonalCollimator",
	0x00185000,VR_SH,"OutputPower",
	0x00185010,VR_LO,"TransducerData",
	0x00185012,VR_DS,"FocusDepth",
	0x00185020,VR_LO,"ProcessingFunction",
	0x00185021,VR_LO,"PostprocessingFunction",
	0x00185022,VR_DS,"MechanicalIndex",
	0x00185024,VR_DS,"ThermalIndex",
	0x00185026,VR_DS,"CranialThermalIndex",
	0x00185027,VR_DS,"SoftTissueThermalIndex",
	0x00185028,VR_DS,"SoftTissue-focusThermalIndex",
	0x00185029,VR_DS,"SoftTissue-surfaceThermalIndex",
	0x00185050,VR_IS,"DepthofScanField",
	0x00185100,VR_CS,"PatientPosition",
	0x00185101,VR_CS,"ViewPosition",
	0x00185104,VR_SQ,"ProjectionEponymousNameCodeSequence",
	0x00185210,VR_DS,"ImageTransformationMatrix",
	0x00185212,VR_DS,"ImageTranslationVector",
	0x00186000,VR_DS,"Sensitivity",
	0x00186011,VR_SQ,"SequenceofUltrasoundRegions",
	0x00186012,VR_US,"RegionSpatialFormat",
	0x00186014,VR_US,"RegionDataType",
	0x00186016,VR_UL,"RegionFlags",
	0x00186018,VR_UL,"RegionLocationMinX0",
	0x0018601A,VR_UL,"RegionLocationMinY0",
	0x0018601C,VR_UL,"RegionLocationMaxX1",
	0x0018601E,VR_UL,"RegionLocationMaxY1",
	0x00186020,VR_SL,"ReferencePixelX0",
	0x00186022,VR_SL,"ReferencePixelY0",
	0x00186024,VR_US,"PhysicalUnitsXDirection",
	0x00186026,VR_US,"PhysicalUnitsYDirection",
	0x00186028,VR_FD,"ReferencePixelPhysicalValueX",
	0x0018602A,VR_FD,"ReferencePixelPhysicalValueY",
	0x0018602C,VR_FD,"PhysicalDeltaX",
	0x0018602E,VR_FD,"PhysicalDeltaY",
	0x00186030,VR_UL,"TransducerFrequency",
	0x00186031,VR_CS,"TransducerType",
	0x00186032,VR_UL,"PulseRepetitionFrequency",
	0x00186034,VR_FD,"DopplerCorrectionAngle",
	0x00186036,VR_FD,"SteeringAngle",
	0x00186038,VR_UL,"DopplerSampleVolumeXPosition",
	0x0018603A,VR_UL,"DopplerSampleVolumeYPosition",
	0x0018603C,VR_UL,"TM-LinePositionX0",
	0x0018603E,VR_UL,"TM-LinePositionY0",
	0x00186040,VR_UL,"TM-LinePositionX1",
	0x00186042,VR_UL,"TM-LinePositionY1",
	0x00186044,VR_US,"PixelComponentOrganization",
	0x00186046,VR_UL,"PixelComponentMask",
	0x00186048,VR_UL,"PixelComponentRangeStart",
	0x0018604A,VR_UL,"PixelComponentRangeStop",
	0x0018604C,VR_US,"PixelComponentPhysicalUnits",
	0x0018604E,VR_US,"PixelComponentDataType",
	0x00186050,VR_UL,"NumberofTableBreakPoints",
	0x00186052,VR_UL,"TableofXBreakPoints",
	0x00186054,VR_FD,"TableofYBreakPoints",
	0x00186056,VR_UL,"NumberofTableEntries",
	0x00186058,VR_UL,"TableofPixelValues",
	0x0018605A,VR_FL,"TableofParameterValues",
	0x00187000,VR_CS,"DetectorConditionsNominalFlag",
	0x00187001,VR_DS,"DetectorTemperature",
	0x00187004,VR_CS,"DetectorType",
	0x00187005,VR_CS,"DetectorConfiguration",
	0x00187006,VR_LT,"DetectorDescription",
	0x00187008,VR_LT,"DetectorMode",
	0x0018700A,VR_SH,"DetectorID",
	0x0018700C,VR_DA,"DateOfLastDetectorCalibration",
	0x0018700E,VR_TM,"TimeOfLastDetectorCalibration",
	0x00187010,VR_IS,"ExposuresOnDetectorSinceLastCalibration",
	0x00187011,VR_IS,"ExposuresOnDetectorSinceManufactured",
	0x00187012,VR_DS,"DetectorTimeSinceLastExposure",
	0x00187014,VR_DS,"DetectorActiveTime",
	0x00187016,VR_DS,"DetectorActivationOffsetFromExposure",
	0x0018701A,VR_DS,"DetectorBinning",
	0x00187020,VR_DS,"DetectorElementPhysicalSize",
	0x00187022,VR_DS,"DetectorElementSpacing",
	0x00187024,VR_CS,"DetectorActiveShape",
	0x00187026,VR_DS,"DetectorActiveDimension(s)",
	0x00187028,VR_DS,"DetectorActiveOringin",
	0x00187030,VR_DS,"FieldOfViewOrigin",
	0x00187032,VR_DS,"FieldOfViewRotation",
	0x00187034,VR_CS,"FieldOfViewHorizontalFlip",
	0x00187040,VR_LT,"GridAbsorbingMaterial",
	0x00187041,VR_LT,"GridSpacingMaterial",
	0x00187042,VR_DS,"GridThickness",
	0x00187044,VR_DS,"GridPitch",
	0x00187046,VR_IS,"GridAspectRatio",
	0x00187048,VR_DS,"GridPeriod",
	0x0018704C,VR_DS,"GridFocalDistance",
	0x00187050,VR_LT,"FilterMaterial",
	0x00187052,VR_DS,"FilterThicknessMinimum",
	0x00187054,VR_DS,"FilterThicknessMaximum",
	0x00187060,VR_CS,"ExposureControlMode",
	0x00187062,VR_LT,"ExposureControlModeDescription",
	0x00187064,VR_CS,"ExposureStatus",
	0x00187065,VR_DS,"PhototimerSetting",
	0x00200000,VR_UL,"GroupLength",
	0x0020000D,VR_UI,"StudyInstanceUID",
	0x0020000E,VR_UI,"SeriesInstanceUID",
	0x00200010,VR_SH,"StudyID",
	0x00200011,VR_IS,"SeriesNumber",
	0x00200012,VR_IS,"AcquisitionNumber",
	0x00200013,VR_IS,"InstanceNumber",
	0x00200020,VR_CS,"PatientOrientation",
	0x00200022,VR_IS,"OverlayNumber",
	0x00200024,VR_IS,"CurveNumber",
	0x00200026,VR_IS,"LookupTableNumber",
	0x00200032,VR_DS,"ImagePosition(Patient)",
	0x00200037,VR_DS,"ImageOrientation(Patient)",
	0x00200052,VR_UI,"FrameofReferenceUID",
	0x00200060,VR_CS,"Laterality",
	0x00200062,VR_CS,"ImageLaterality",
	0x00200100,VR_IS,"TemporalPositionIdentifier",
	0x00200105,VR_IS,"NumberofTemporalPositions",
	0x00200110,VR_DS,"TemporalResolution",
	0x00201000,VR_IS,"SeriesinStudy",
	0x00201002,VR_IS,"ImagesinAcquisition",
	0x00201004,VR_IS,"AcquisitionsinStudy",
	0x00201040,VR_LO,"PositionReferenceIndicator",
	0x00201041,VR_DS,"SliceLocation",
	0x00201070,VR_IS,"OtherStudyNumbers",
	0x00201200,VR_IS,"NumberofPatientRelatedStudies",
	0x00201202,VR_IS,"NumberofPatientRelatedSeries",
	0x00201204,VR_IS,"NumberofPatientRelatedImages",
	0x00201206,VR_IS,"NumberofStudyRelatedSeries",
	0x00201208,VR_IS,"NumberofStudyRelatedImages",
	0x00201209,VR_IS,"NumberofSeriesRelatedImages",
	0x00204000,VR_LT,"ImageComments",
	0x00280000,VR_UL,"GroupLength",
	0x00280002,VR_US,"SamplesperPixel",
	0x00280004,VR_CS,"PhotometricInterpretation",
	0x00280006,VR_US,"PlanarConfiguration",
	0x00280008,VR_IS,"NumberofFrames",
	0x00280009,VR_AT,"FrameIncrementPointer",
	0x00280010,VR_US,"Rows",
	0x00280011,VR_US,"Columns",
	0x00280012,VR_US,"Planes",
	0x00280014,VR_US,"UltrasoundColorDataPresent",
	0x00280030,VR_DS,"PixelSpacing",
	0x00280031,VR_DS,"ZoomFactor",
	0x00280032,VR_DS,"ZoomCenter",
	0x00280034,VR_IS,"PixelAspectRatio",
	0x00280051,VR_CS,"CorrectedImage",
	0x00280100,VR_US,"BitsAllocated",
	0x00280101,VR_US,"BitsStored",
	0x00280102,VR_US,"HighBit",
	0x00280103,VR_US,"PixelRepresentation",
	0x00280106,VR_US,"SmallestImagePixelValue",
	0x00280107,VR_US,"LargestImagePixelValue",
	0x00280108,VR_US,"SmallestPixelValueinSeries",
	0x00280109,VR_US,"LargestPixelValueinSeries",
	0x00280110,VR_US,"SmallestImagePixelValueinPlane",
	0x00280111,VR_US,"LargestImagePixelValueinPlane",
	0x00280120,VR_US,"PixelPaddingValue",
	0x00280300,VR_CS,"QualityControlImage",
	0x00280301,VR_CS,"BurnedInAnnotation",
	0x00281040,VR_CS,"PixelIntensityRelationship",
	0x00281041,VR_SS,"PixelIntensityRelationshipSign",
	0x00281050,VR_DS,"WindowCenter",
	0x00281051,VR_DS,"WindowWidth",
	0x00281052,VR_DS,"RescaleIntercept",
	0x00281053,VR_DS,"RescaleSlope",
	0x00281054,VR_LO,"RescaleType",
	0x00281055,VR_LO,"WindowCenter&WidthExplanation",
	0x00281090,VR_CS,"RecommendedViewingMode",
	0x00281101,VR_US,"RedPaletteColorLookupTableDescriptor",
	0x00281102,VR_US,"GreenPaletteColorLookupTableDescriptor",
	0x00281103,VR_US,"BluePaletteColorLookupTableDescriptor",
	0x00281199,VR_UI,"PaletteColorLookupTableUID",
	0x00281201,VR_OW,"RedPaletteColorLookupTableData",
	0x00281202,VR_OW,"GreenPaletteColorLookupTableData",
	0x00281203,VR_OW,"BluePaletteColorLookupTableData",
	0x00281221,VR_OW,"SegmentedRedPaletteColorLookupTableData",
	0x00281222,VR_OW,"SegmentedGreenPaletteColorLookupTableData",
	0x00281223,VR_OW,"SegmentedBluePaletteColorLookupTableData",
	0x00281300,VR_CS,"ImplantPresent",
	0x00282110,VR_CS,"LossyImageCompression",
	0x00282112,VR_DS,"LossyImageCompressionRatio",
	0x00283000,VR_SQ,"ModalityLUTSequence",
	0x00283002,VR_US,"LUTDescriptor",
	0x00283003,VR_LO,"LUTExplanation",
	0x00283004,VR_LO,"ModalityLUTType",
	0x00283006,VR_SS,"LUTDataUSor",
	0x00283010,VR_SQ,"VOILUTSequence",
	0x00285000,VR_SQ,"Bi-PlaneAcquisitionSequence",
	0x00286010,VR_US,"RepresentativeFrameNumber",
	0x00286020,VR_US,"FrameNumbersofInterest(FOI)",
	0x00286022,VR_LO,"Frame(s)ofInterestDescription",
	0x00286030,VR_US,"MaskPointer(s)",
	0x00286040,VR_US,"RWavePointer",
	0x00286100,VR_SQ,"MaskSubtractionSequence",
	0x00286101,VR_CS,"MaskOperation",
	0x00286102,VR_US,"ApplicableFrameRange",
	0x00286110,VR_US,"MaskFrameNumbers",
	0x00286112,VR_US,"ContrastFrameAveraging",
	0x00286114,VR_FL,"MaskSub-pixelShift",
	0x00286120,VR_SS,"TIDOffset",
	0x00286190,VR_ST,"MaskOperationExplanation",
	0x00320000,VR_UL,"GroupLength",
	0x0032000A,VR_CS,"StudyStatusID",
	0x0032000C,VR_CS,"StudyPriorityID",
	0x00320012,VR_LO,"StudyIDIssuer",
	0x00320032,VR_DA,"StudyVerifiedDate",
	0x00320033,VR_TM,"StudyVerifiedTime",
	0x00320034,VR_DA,"StudyReadDate",
	0x00320035,VR_TM,"StudyReadTime",
	0x00321000,VR_DA,"ScheduledStudyStartDate",
	0x00321001,VR_TM,"ScheduledStudyStartTime",
	0x00321010,VR_DA,"ScheduledStudyStopDate",
	0x00321011,VR_TM,"ScheduledStudyStopTime",
	0x00321020,VR_LO,"ScheduledStudyLocation",
	0x00321021,VR_AE,"ScheduledStudyLocationAETitle(s)",
	0x00321030,VR_LO,"ReasonforStudy",
	0x00321032,VR_PN,"RequestingPhysician",
	0x00321033,VR_LO,"RequestingService",
	0x00321040,VR_DA,"StudyArrivalDate",
	0x00321041,VR_TM,"StudyArrivalTime",
	0x00321050,VR_DA,"StudyCompletionDate",
	0x00321051,VR_TM,"StudyCompletionTime",
	0x00321055,VR_CS,"StudyComponentStatusID",
	0x00321060,VR_LO,"RequestedProcedureDescription",
	0x00321064,VR_SQ,"RequestedProcedureCodeSequence",
	0x00321070,VR_LO,"RequestedContrastAgent",
	0x00324000,VR_LT,"StudyComments",
	0x00380000,VR_UL,"GroupLength",
	0x00380004,VR_SQ,"ReferencedPatientAliasSequence",
	0x00380008,VR_CS,"VisitStatusID",
	0x00380010,VR_LO,"AdmissionID",
	0x00380011,VR_LO,"IssuerofAdmissionID",
	0x00380016,VR_LO,"RouteofAdmissions",
	0x0038001A,VR_DA,"ScheduledAdmissionDate",
	0x0038001B,VR_TM,"ScheduledAdmissionTime",
	0x0038001C,VR_DA,"ScheduledDischargeDate",
	0x0038001D,VR_TM,"ScheduledDischargeTime",
	0x0038001E,VR_LO,"ScheduledPatientInstitutionResidence",
	0x00380020,VR_DA,"AdmittingDate",
	0x00380021,VR_TM,"AdmittingTime",
	0x00380030,VR_DA,"DischargeDate",
	0x00380032,VR_TM,"DischargeTime",
	0x00380040,VR_LO,"DischargeDiagnosisDescription",
	0x00380044,VR_SQ,"DischargeDiagnosisCodeSequence",
	0x00380050,VR_LO,"SpecialNeeds",
	0x00380300,VR_LO,"CurrentPatientLocation",
	0x00380400,VR_LO,"Patient'sInstitutionResidence",
	0x00380500,VR_LO,"PatientState",
	0x00384000,VR_LT,"VisitComments",
	0x00400000,VR_UL,"GroupLength",
	0x00400001,VR_AE,"ScheduledStationAETitle",
	0x00400002,VR_DA,"ScheduledProcedureStepStartDate",
	0x00400003,VR_TM,"ScheduledProcedureStepStartTime",
	0x00400004,VR_DA,"ScheduledProcedureStepEndDate",
	0x00400005,VR_TM,"ScheduledProcedureStepEndTime",
	0x00400006,VR_PN,"ScheduledPerformingPhysician'sName",
	0x00400007,VR_LO,"ScheduledProcedureStepDescription",
	0x00400008,VR_SQ,"ScheduledActionItemCodeSequence",
	0x00400009,VR_SH,"ScheduledProcedureStepID",
	0x00400010,VR_SH,"ScheduledStationName",
	0x00400011,VR_SH,"ScheduledProcedureStepLocation",
	0x00400012,VR_LO,"Pre-Medication",
	0x00400020,VR_CS,"ScheduledProcedureStepStatus",
	0x00400100,VR_SQ,"ScheduledProcedureStepSequence",
	0x00400220,VR_SQ,"ReferencedStandaloneSOPInstanceSequence",
	0x00400241,VR_AE,"PerformedStationAETitle",
	0x00400242,VR_SH,"PerformedStationName",
	0x00400243,VR_SH,"PerformedLocation",
	0x00400244,VR_DA,"PerformedProcedureStepStartDate",
	0x00400245,VR_TM,"PerformedProcedureStepStartTime",
	0x00400250,VR_DA,"PerformedProcedureStepEndDate",
	0x00400251,VR_TM,"PerformedProcedureStepEndTime",
	0x00400252,VR_CS,"PerformedProcedureStepStatus",
	0x00400253,VR_CS,"PerformedProcedureStepID",
	0x00400254,VR_LO,"PerformedProcedureStepDescription",
	0x00400255,VR_LO,"PerformedProcedureTypeDescription",
	0x00400260,VR_SQ,"PerformedActionItemSequence",
	0x00400270,VR_SQ,"ScheduledStepAttributesSequence",
	0x00400275,VR_SQ,"RequestAttributesSequence",
	0x00400280,VR_ST,"CommentsonthePerformedProcedureSteps",
	0x00400293,VR_SQ,"QuantitySequence",
	0x00400294,VR_DS,"Quantity",
	0x00400295,VR_SQ,"MeasuringUnitsSequence",
	0x00400296,VR_SQ,"BillingItemSequence",
	0x00400300,VR_US,"TotalTimeofFluoroscopy",
	0x00400301,VR_US,"TotalNumberofExposures",
	0x00400302,VR_US,"EntranceDose",
	0x00400303,VR_US,"ExposedArea",
	0x00400306,VR_DS,"DistanceSourcetoEntrance",
	0x00400307,VR_DS,"DistanceSourcetoSupport",
	0x00400310,VR_ST,"CommentsonRadiationDose",
	0x00400312,VR_DS,"X-RayOutput",
	0x00400314,VR_DS,"HalfValueLayer",
	0x00400316,VR_DS,"OrganDose",
	0x00400318,VR_CS,"OrganExposed",
	0x00400320,VR_SQ,"BillingProcedureStepSequence",
	0x00400321,VR_SQ,"FilmConsumptionSequence",
	0x00400324,VR_SQ,"BillingSuppliesandDevicesSequence",
	0x00400330,VR_SQ,"ReferencedProcedureStepSequence",
	0x00400340,VR_SQ,"PerformedSeriesSequence",
	0x00400400,VR_LT,"CommentsontheScheduledProcedureStep",
	0x00400555,VR_SQ,"AcquisitionContextSequence",
	0x00400556,VR_ST,"AcquisitionContextDescription",
	0x004008EA,VR_SQ,"MeasurementUnitsCodeSequence",
	0x00401001,VR_SH,"RequestedProcedureID",
	0x00401002,VR_LO,"ReasonfortheRequestedProcedure",
	0x00401003,VR_SH,"RequestedProcedurePriority",
	0x00401004,VR_LO,"PatientTransportArrangements",
	0x00401005,VR_LO,"RequestedProcedureLocation",
	0x00401006,VR_SH,"PlacerOrderNumber/Procedure",
	0x00401007,VR_SH,"FillerOrderNumber/Procedure",
	0x00401008,VR_LO,"ConfidentialityCode",
	0x00401009,VR_SH,"ReportingPriority",
	0x00401010,VR_PN,"NamesofIntendedRecipientsofResults",
	0x00401060,VR_ST,"RequestedProcedureDescription",
	0x00401064,VR_SQ,"RequestedProcedureCodeSequence",
	0x00401400,VR_LT,"RequestedProcedureComments",
	0x00402001,VR_LO,"ReasonfortheImagingServiceRequest",
	0x00402004,VR_DA,"IssueDateofImagingServiceRequest",
	0x00402005,VR_TM,"IssueTimeofImagingServiceRequest",
	0x00402006,VR_SH,"PlacerOrderNumber/ImagingServiceRequest",
	0x00402007,VR_SH,"FillerOrderNumber/ImagingServiceRequest",
	0x00402008,VR_PN,"OrderEnteredBy",
	0x00402009,VR_SH,"OrderEnterersLocation",
	0x00402010,VR_SH,"OrderCallbackPhoneNumber",
	0x00402016,VR_LO,"PlacerOrderNumber/ImagingServiceRequest",
	0x00402017,VR_LO,"FillerOrderNumber/ImagingServiceRequest",
	0x00402400,VR_LT,"ImagingServiceRequestComments",
	0x00403001,VR_LO,"ConfidentialityConstraintonPatientDataDescription",
	0x0040A010,VR_CS,"RelationshipType",
	0x0040A027,VR_LO,"VerifyingOrganization",
	0x0040A030,VR_DT,"VerificationDateTime",
	0x0040A032,VR_DT,"ObservationDateTime",
	0x0040A040,VR_CS,"ValueType",
	0x0040A043,VR_SQ,"ConceptNameCodeSequence",
	0x0040A050,VR_CS,"ContinuityOfContent",
	0x0040A073,VR_SQ,"VerifyingObserverSequence",
	0x0040A075,VR_PN,"VerifyingObserverName",
	0x0040A088,VR_SQ,"VerifyingObserverIdentificationCodeSequence",
	0x0040A0B0,VR_US,"ReferencedWaveformChannels",
	0x0040A120,VR_DT,"DateTime",
	0x0040A121,VR_DA,"Date",
	0x0040A122,VR_TM,"Time",
	0x0040A123,VR_PN,"PersonName",
	0x0040A124,VR_UI,"UID",
	0x0040A130,VR_CS,"TemporalRangeType",
	0x0040A132,VR_US,"ReferencedSamplePositions",
	0x0040A136,VR_US,"ReferencedFrameNumbers",
	0x0040A138,VR_US,"ReferencedTimeOffsets",
	0x0040A13A,VR_DT,"ReferencedDateTime",
	0x0040A160,VR_UT,"TextValue",
	0x0040A168,VR_SQ,"ConceptCodeSequence",
	0x0040A300,VR_SQ,"MeasuredValueSequence",
	0x0040A30A,VR_DS,"NumericValue",
	0x0040A360,VR_SQ,"PredecessorDocumentsSequence",
	0x0040A370,VR_SQ,"ReferencedRequestSequence",
	0x0040A372,VR_SQ,"PerformProcedureCodeSequence",
	0x0040A375,VR_SQ,"CurrentRequestedProcedureEvidenceSequence",
	0x0040A385,VR_SQ,"PatientOtherEvidenceSequence",
	0x0040A491,VR_CS,"CompletionFlag",
	0x0040A492,VR_LO,"CompletionFlagDescription",
	0x0040A493,VR_CS,"VerificationFlag",
	0x0040A504,VR_SQ,"ContentTemplateSequence",
	0x0040A525,VR_SQ,"IdenticalDocumentsSequence",
	0x0040A730,VR_SQ,"ContentSequence",
	0x0040DB00,VR_CS,"TemplateIdentifier",
	0x0040DB06,VR_DT,"TemplateVersion",
	0x0040DB07,VR_DT,"TemplateLocalVersion",
	0x0040DB0B,VR_CS,"TemplateExtensionFlag",
	0x0040DB0C,VR_UI,"TemplateExtensionOrganizationUID",
	0x0040DB0D,VR_UI,"TemplateExtensionCreatorUID",
	0x0040DB73,VR_UL,"ReferencedContentItemIdentifier",
	0x00500000,VR_UL,"GroupLength",
	0x00500004,VR_CS,"CalibrationImage",
	0x00500010,VR_SQ,"DeviceSequence",
	0x00500014,VR_DS,"DeviceLength",
	0x00500016,VR_DS,"DeviceDiameter",
	0x00500017,VR_CS,"DeviceDiameterUnits",
	0x00500018,VR_DS,"DeviceVolume",
	0x00500019,VR_DS,"Inter-markerDistance",
	0x00500020,VR_LO,"DeviceDescription",
	0x00540000,VR_UL,"GroupLength",
	0x00540010,VR_US,"EnergyWindowVector",
	0x00540011,VR_US,"NumberofEnergyWindows",
	0x00540012,VR_SQ,"EnergyWindowInformationSequence",
	0x00540013,VR_SQ,"EnergyWindowRangeSequence",
	0x00540014,VR_DS,"EnergyWindowLowerLimit",
	0x00540015,VR_DS,"EnergyWindowUpperLimit",
	0x00540016,VR_SQ,"RadiopharmaceuticalInformationSequence",
	0x00540017,VR_IS,"ResidualSyringeCounts",
	0x00540018,VR_SH,"EnergyWindowName",
	0x00540020,VR_US,"DetectorVector",
	0x00540021,VR_US,"NumberofDetectors",
	0x00540022,VR_SQ,"DetectorInformationSequence",
	0x00540030,VR_US,"PhaseVector",
	0x00540031,VR_US,"NumberofPhases",
	0x00540032,VR_SQ,"PhaseInformationSequence",
	0x00540033,VR_US,"NumberofFramesinPhase",
	0x00540036,VR_IS,"PhaseDelay",
	0x00540038,VR_IS,"PauseBetweenFrames",
	0x00540050,VR_US,"RotationVector",
	0x00540051,VR_US,"NumberofRotations",
	0x00540052,VR_SQ,"RotationInformationSequence",
	0x00540053,VR_US,"NumberofFramesinRotation",
	0x00540060,VR_US,"R-RIntervalVector",
	0x00540061,VR_US,"NumberofR-RIntervals",
	0x00540062,VR_SQ,"GatedInformationSequence",
	0x00540063,VR_SQ,"DataInformationSequence",
	0x00540070,VR_US,"TimeSlotVector",
	0x00540071,VR_US,"NumberofTimeSlots",
	0x00540072,VR_SQ,"TimeSlotInformationSequence",
	0x00540073,VR_DS,"TimeSlotTime",
	0x00540080,VR_US,"SliceVector",
	0x00540081,VR_US,"NumberofSlices",
	0x00540090,VR_US,"AngularViewVector",
	0x00540100,VR_US,"TimeSliceVector",
	0x00540101,VR_US,"NumberofTimeSlices",
	0x00540200,VR_DS,"StartAngle",
	0x00540202,VR_CS,"TypeofDetectorMotion",
	0x00540210,VR_IS,"TriggerVector",
	0x00540211,VR_US,"NumberofTriggersinPhase",
	0x00540220,VR_SQ,"ViewCodeSequence",
	0x00540222,VR_SQ,"ViewModifierCodeSequence",
	0x00540300,VR_SQ,"RadionuclideCodeSequence",
	0x00540302,VR_SQ,"AdministrationRouteCodeSequence",
	0x00540304,VR_SQ,"RadiopharmaceuticalCodeSequence",
	0x00540306,VR_SQ,"CalibrationDataSequence",
	0x00540308,VR_US,"EnergyWindowNumber",
	0x00540400,VR_SH,"ImageID",
	0x00540410,VR_SQ,"PatientOrientationCodeSequence",
	0x00540412,VR_SQ,"PatientOrientationModifierCodeSequence",
	0x00540414,VR_SQ,"PatientGantryRelationshipCodeSequence",
	0x00541000,VR_CS,"SeriesType",
	0x00541001,VR_CS,"Units",
	0x00541002,VR_CS,"CountsSource",
	0x00541004,VR_CS,"ReprojectionMethod",
	0x00541100,VR_CS,"RandomsCorrectionMethod",
	0x00541101,VR_LO,"AttenuationCorrectionMethod",
	0x00541102,VR_CS,"DecayCorrection",
	0x00541103,VR_LO,"ReconstructionMethod",
	0x00541104,VR_LO,"DetectorLinesofResponseUsed",
	0x00541105,VR_LO,"ScatterCorrectionMethod",
	0x00541200,VR_DS,"AxialAcceptance",
	0x00541201,VR_IS,"AxialMash",
	0x00541202,VR_IS,"TransverseMash",
	0x00541203,VR_DS,"DetectorElementSize",
	0x00541210,VR_DS,"CoincidenceWindowWidth",
	0x00541220,VR_CS,"SecondaryCountsType",
	0x00541300,VR_DS,"FrameReferenceTime",
	0x00541310,VR_IS,"Primary(Prompts)CountsAccumulated",
	0x00541311,VR_IS,"SecondaryCountsAccumulated",
	0x00541320,VR_DS,"SliceSensitivityFactor",
	0x00541321,VR_DS,"DecayFactor",
	0x00541322,VR_DS,"DoseCalibrationFactor",
	0x00541323,VR_DS,"ScatterFractionFactor",
	0x00541324,VR_DS,"DeadTimeFactor",
	0x00541330,VR_US,"ImageIndex",
	0x00541400,VR_CS,"CountsIncluded",
	0x00541401,VR_CS,"DeadTimeCorrectionFlag",
	0x00603000,VR_SQ,"HistogramSequence",
	0x00603002,VR_US,"HistogramNumberOfBins",
	0x00603004,VR_US,"HistogramFirstBinValue",
	0x00603006,VR_US,"HistogramLastBinValue",
	0x00603008,VR_US,"HistogramBinWidth",
	0x00603010,VR_LO,"HistogramExplanation",
	0x00603020,VR_UL,"HistogramData",
	0x00700022,VR_FL,"GraphicData",
	0x00700023,VR_CS,"GraphicType",
	0x00880000,VR_UL,"GroupLength",
	0x00880130,VR_SH,"StorageMediaFile-setID",
	0x00880140,VR_UI,"StorageMediaFile-setUID",
	0x00880200,VR_SQ,"IconImageSequence",
	0x00880904,VR_LO,"TopicTitle",
	0x00880906,VR_ST,"TopicSubject",
	0x00880910,VR_LO,"TopicAuthor",
	0x00880912,VR_LO,"TopicKeyWords",
	0x20000000,VR_UL,"GroupLength",
	0x20000010,VR_IS,"NumberofCopies",
	0x20000020,VR_CS,"PrintPriority",
	0x20000030,VR_CS,"MediumType",
	0x20000040,VR_CS,"FilmDestination",
	0x20000050,VR_LO,"FilmSessionLabel",
	0x20000060,VR_IS,"MemoryAllocation",
	0x20000062,VR_CS,"ColorImagePrintingFlag",
	0x20000063,VR_CS,"CollationFlag",
	0x20000065,VR_CS,"AnnotationFlag",
	0x20000067,VR_CS,"ImageOverlayFlag",
	0x20000069,VR_CS,"PresentationLUTFlag",
	0x2000006A,VR_CS,"ImageBoxPresentationLUTFlag",
	0x20000500,VR_SQ,"ReferencedFilmBoxSequence",
	0x20000510,VR_SQ,"ReferencedStoredPrintSequence",
	0x20100000,VR_UL,"GroupLength",
	0x20100010,VR_ST,"ImageDisplayFormat",
	0x20100030,VR_CS,"AnnotationDisplayFormatID",
	0x20100040,VR_CS,"FilmOrientation",
	0x20100050,VR_CS,"FilmSizeID",
	0x20100060,VR_CS,"MagnificationType",
	0x20100080,VR_CS,"SmoothingType",
	0x20100100,VR_CS,"BorderDensity",
	0x20100110,VR_CS,"EmptyImageDensity",
	0x20100120,VR_US,"MinDensity",
	0x20100130,VR_US,"MaxDensity",
	0x20100140,VR_CS,"Trim",
	0x20100150,VR_ST,"ConfigurationInformation",
	0x2010015E,VR_US,"Illumination",
	0x20100160,VR_US,"ReflectedAmbientLight",
	0x20100500,VR_SQ,"ReferencedFilmSessionSequence",
	0x20100510,VR_SQ,"ReferencedImageBoxSequence",
	0x20100520,VR_SQ,"ReferencedBasicAnnotationBoxSequence",
	0x20200000,VR_UL,"GroupLength",
	0x20200010,VR_US,"ImagePosition",
	0x20200020,VR_CS,"Polarity",
	0x20200030,VR_DS,"RequestedImageSize",
	0x20200110,VR_SQ,"BasicGrayscaleImageSequence",
	0x20200111,VR_SQ,"BasicColorImageSequence",
	0x20200130,VR_SQ,"ReferencedImageOverlayBoxSequence",
	0x20200140,VR_SQ,"ReferencedVOILUTBoxSequence",
	0x20300000,VR_UL,"GroupLength",
	0x20300010,VR_US,"AnnotationPosition",
	0x20300020,VR_LO,"TextString",
	0x20400000,VR_UL,"GroupLength",
	0x20400010,VR_SQ,"ReferencedOverlayPlaneSequence",
	0x20400011,VR_US,"ReferencedOverlayPlaneGroups",
	0x20400060,VR_CS,"OverlayMagnificationType",
	0x20400070,VR_CS,"OverlaySmoothingType",
	0x20400080,VR_CS,"OverlayForegroundDensity",
	0x20400090,VR_CS,"OverlayMode",
	0x20400100,VR_CS,"ThresholdDensity",
	0x20500010,VR_SQ,"PresentationLUTSequence",
	0x20500020,VR_CS,"PresentationLUTShape",
	0x20500500,VR_SQ,"ReferencedPresentationLUTSequence",
	0x21000000,VR_UL,"GroupLength",
	0x21000010,VR_SH,"PrintJobID",
	0x21000020,VR_CS,"ExecutionStatus",
	0x21000030,VR_CS,"ExecutionStatusInfo",
	0x21000040,VR_DA,"CreationDate",
	0x21000050,VR_TM,"CreationTime",
	0x21000070,VR_AE,"Originator",
	0x21000140,VR_AE,"DestinationAE",
	0x21000160,VR_SH,"OwnerID",
	0x21000170,VR_IS,"NumberofFilms",
	0x21000500,VR_SQ,"ReferencedPrintJobSequence",
	0x21100000,VR_UL,"GroupLength",
	0x21100010,VR_CS,"PrinterStatus",
	0x21100020,VR_CS,"PrinterStatusInfo",
	0x21100030,VR_LO,"PrinterName",
	0x21100099,VR_SH,"PrintQueueID",
	0x21200010,VR_CS,"QueueStatus",
	0x21200050,VR_SQ,"PrintJobDescriptionSequence",
	0x21200070,VR_SQ,"ReferencedPrintJobSequence",
	0x21300010,VR_SQ,"PrintManagementCapabilitiesSequence",
	0x21300015,VR_SQ,"PrinterCharacteristicsSequence",
	0x21300030,VR_SQ,"FilmBoxContentSequence",
	0x21300040,VR_SQ,"ImageBoxContentSequence",
	0x21300050,VR_SQ,"AnnotationContentSequence",
	0x21300060,VR_SQ,"ImageOverlayBoxContentSequence",
	0x21300080,VR_SQ,"PresentationLUTContentSequence",
	0x213000A0,VR_SQ,"ProposedStudySequence",
	0x213000C0,VR_SQ,"OriginalImageSequence",
	0x30020002,VR_SH,"RTImageLabel",
	0x30020003,VR_LO,"RTImageName",
	0x30020004,VR_ST,"RTImageDescription",
	0x3002000A,VR_CS,"ReportedValuesOrigin",
	0x3002000C,VR_CS,"RTImagePlane",
	0x3002000E,VR_DS,"X-RayImageReceptorAngle",
	0x30020010,VR_DS,"RTImageOrientation",
	0x30020011,VR_DS,"ImagePlanePixelSpacing",
	0x30020012,VR_DS,"RTImagePosition",
	0x30020020,VR_SH,"RadiationMachineName",
	0x30020022,VR_DS,"RadiationMachineSAD",
	0x30020024,VR_DS,"RadiationMachineSSD",
	0x30020026,VR_DS,"RTImageSID",
	0x30020028,VR_DS,"SourcetoReferenceObjectDistance",
	0x30020029,VR_IS,"FractionNumber",
	0x30020030,VR_SQ,"ExposureSequence",
	0x30020032,VR_DS,"MetersetExposure",
	0x30040001,VR_CS,"DVHType",
	0x30040002,VR_CS,"DoseUnits",
	0x30040004,VR_CS,"DoseType",
	0x30040006,VR_LO,"DoseComment",
	0x30040008,VR_DS,"NormalizationPoint",
	0x3004000A,VR_CS,"DoseSummationType",
	0x3004000C,VR_DS,"GridFrameOffsetVector",
	0x3004000E,VR_DS,"DoseGridScaling",
	0x30040010,VR_SQ,"RTDoseROISequence",
	0x30040012,VR_DS,"DoseValue",
	0x30040040,VR_DS,"DVHNormalizationPoint",
	0x30040042,VR_DS,"DVHNormalizationDoseValue",
	0x30040050,VR_SQ,"DVHSequence",
	0x30040052,VR_DS,"DVHDoseScaling",
	0x30040054,VR_CS,"DVHVolumeUnits",
	0x30040056,VR_IS,"DVHNumberofBins",
	0x30040058,VR_DS,"DVHData",
	0x30040060,VR_SQ,"DVHReferencedROISequence",
	0x30040062,VR_CS,"DVHROIContributionType",
	0x30040070,VR_DS,"DVHMinimumDose",
	0x30040072,VR_DS,"DVHMaximumDose",
	0x30040074,VR_DS,"DVHMeanDose",
	0x30060002,VR_SH,"StructureSetLabel",
	0x30060004,VR_LO,"StructureSetName",
	0x30060006,VR_ST,"StructureSetDescription",
	0x30060008,VR_DA,"StructureSetDate",
	0x30060009,VR_TM,"StructureSetTime",
	0x30060010,VR_SQ,"ReferencedFrameofReferenceSequence",
	0x30060012,VR_SQ,"RTReferencedStudySequence",
	0x30060014,VR_SQ,"RTReferencedSeriesSequence",
	0x30060016,VR_SQ,"ContourImageSequence",
	0x30060020,VR_SQ,"StructureSetROISequence",
	0x30060022,VR_IS,"ROINumber",
	0x30060024,VR_UI,"ReferencedFrameofReferenceUID",
	0x30060026,VR_LO,"ROIName",
	0x30060028,VR_ST,"ROIDescription",
	0x3006002A,VR_IS,"ROIDisplayColor",
	0x3006002C,VR_DS,"ROIVolume",
	0x30060030,VR_SQ,"RTRelatedROISequence",
	0x30060033,VR_CS,"RTROIRelationship",
	0x30060036,VR_CS,"ROIGenerationAlgorithm",
	0x30060038,VR_LO,"ROIGenerationDescription",
	0x30060039,VR_SQ,"ROIContourSequence",
	0x30060040,VR_SQ,"ContourSequence",
	0x30060042,VR_CS,"ContourGeometricType",
	0x30060044,VR_DS,"ContourSlabThickness",
	0x30060045,VR_DS,"ContourOffsetVector",
	0x30060046,VR_IS,"NumberofContourPoints",
	0x30060050,VR_DS,"ContourData",
	0x30060080,VR_SQ,"RTROIObservationsSequence",
	0x30060082,VR_IS,"ObservationNumber",
	0x30060084,VR_IS,"ReferencedROINumber",
	0x30060085,VR_SH,"ROIObservationLabel",
	0x30060086,VR_SQ,"RTROIIdentificationCodeSequence",
	0x30060088,VR_ST,"ROIObservationDescription",
	0x300600A0,VR_SQ,"RelatedRTROIObservationsSequence",
	0x300600A4,VR_CS,"RTROIInterpretedType",
	0x300600A6,VR_PN,"ROIInterpreter",
	0x300600B0,VR_SQ,"ROIPhysicalPropertiesSequence",
	0x300600B2,VR_CS,"ROIPhysicalProperty",
	0x300600B4,VR_DS,"ROIPhysicalPropertyValue",
	0x300600C0,VR_SQ,"FrameofReferenceRelationshipSequence",
	0x300600C2,VR_UI,"RelatedFrameofReferenceUID",
	0x300600C4,VR_CS,"FrameofReferenceTransformationType",
	0x300600C6,VR_DS,"FrameofReferenceTransformationMatrix",
	0x300600C8,VR_LO,"FrameofReferenceTransformationComment",
	0x300A0002,VR_SH,"RTPlanLabel",
	0x300A0003,VR_LO,"RTPlanName",
	0x300A0004,VR_ST,"RTPlanDescription",
	0x300A0006,VR_DA,"RTPlanDate",
	0x300A0007,VR_TM,"RTPlanTime",
	0x300A0009,VR_LO,"TreatmentProtocols",
	0x300A000A,VR_CS,"TreatmentIntent",
	0x300A000B,VR_LO,"TreatmentSites",
	0x300A000C,VR_CS,"RTPlanGeometry",
	0x300A000E,VR_ST,"PrescriptionDescription",
	0x300A0010,VR_SQ,"DoseReferenceSequence",
	0x300A0012,VR_IS,"DoseReferenceNumber",
	0x300A0014,VR_CS,"DoseReferenceStructureType",
	0x300A0016,VR_LO,"DoseReferenceDescription",
	0x300A0018,VR_DS,"DoseReferencePointCoordinates",
	0x300A001A,VR_DS,"NominalPriorDose",
	0x300A0020,VR_CS,"DoseReferenceType",
	0x300A0021,VR_DS,"ConstraintWeight",
	0x300A0022,VR_DS,"DeliveryWarningDose",
	0x300A0023,VR_DS,"DeliveryMaximumDose",
	0x300A0025,VR_DS,"TargetMinimumDose",
	0x300A0026,VR_DS,"TargetPrescriptionDose",
	0x300A0027,VR_DS,"TargetMaximumDose",
	0x300A0028,VR_DS,"TargetUnderdoseVolumeFraction",
	0x300A002A,VR_DS,"OrganatRiskFull-volumeDose",
	0x300A002B,VR_DS,"OrganatRiskLimitDose",
	0x300A002C,VR_DS,"OrganatRiskMaximumDose",
	0x300A002D,VR_DS,"OrganatRiskOverdoseVolumeFraction",
	0x300A0040,VR_SQ,"ToleranceTableSequence",
	0x300A0042,VR_IS,"ToleranceTableNumber",
	0x300A0043,VR_SH,"ToleranceTableLabel",
	0x300A0044,VR_DS,"GantryAngleTolerance",
	0x300A0046,VR_DS,"BeamLimitingDeviceAngleTolerance",
	0x300A0048,VR_SQ,"BeamLimitingDeviceToleranceSequence",
	0x300A004A,VR_DS,"BeamLimitingDevicePositionTolerance",
	0x300A004C,VR_DS,"PatientSupportAngleTolerance",
	0x300A004E,VR_DS,"TableTopEccentricAngleTolerance",
	0x300A0051,VR_DS,"TableTopVerticalPositionTolerance",
	0x300A0052,VR_DS,"TableTopLongitudinalPositionTolerance",
	0x300A0053,VR_DS,"TableTopLateralPositionTolerance",
	0x300A0055,VR_CS,"RTPlanRelationship",
	0x300A0070,VR_SQ,"FractionGroupSequence",
	0x300A0071,VR_IS,"FractionGroupNumber",
	0x300A0078,VR_IS,"NumberofFractionsPlanned",
	0x300A0079,VR_IS,"NumberofFractionsPerDay",
	0x300A007A,VR_IS,"RepeatFractionCycleLength",
	0x300A007B,VR_LT,"FractionPattern",
	0x300A0080,VR_IS,"NumberofBeams",
	0x300A0082,VR_DS,"BeamDoseSpecificationPoint",
	0x300A0084,VR_DS,"BeamDose",
	0x300A0086,VR_DS,"BeamMeterset",
	0x300A00A0,VR_IS,"NumberofBrachyApplicationSetups",
	0x300A00A2,VR_DS,"BrachyApplicationSetupDoseSpecificationPoint",
	0x300A00A4,VR_DS,"BrachyApplicationSetupDose",
	0x300A00B0,VR_SQ,"BeamSequence",
	0x300A00B2,VR_SH,"TreatmentMachineName",
	0x300A00B3,VR_CS,"PrimaryDosimeterUnit",
	0x300A00B4,VR_DS,"Source-AxisDistance",
	0x300A00B6,VR_SQ,"BeamLimitingDeviceSequence",
	0x300A00B8,VR_CS,"RTBeamLimitingDeviceType",
	0x300A00BA,VR_DS,"SourcetoBeamLimitingDeviceDistance",
	0x300A00BC,VR_IS,"NumberofLeaf/JawPairs",
	0x300A00BE,VR_DS,"LeafPositionBoundaries",
	0x300A00C0,VR_IS,"BeamNumber",
	0x300A00C2,VR_LO,"BeamName",
	0x300A00C3,VR_ST,"BeamDescription",
	0x300A00C4,VR_CS,"BeamType",
	0x300A00C6,VR_CS,"RadiationType",
	0x300A00C8,VR_IS,"ReferenceImageNumber",
	0x300A00CA,VR_SQ,"PlannedVerificationImageSequence",
	0x300A00CC,VR_LO,"ImagingDevice-SpecificAcquisitionParameters",
	0x300A00CE,VR_CS,"TreatmentDeliveryType",
	0x300A00D0,VR_IS,"NumberofWedges",
	0x300A00D1,VR_SQ,"WedgeSequence",
	0x300A00D2,VR_IS,"WedgeNumber",
	0x300A00D3,VR_CS,"WedgeType",
	0x300A00D4,VR_SH,"WedgeID",
	0x300A00D5,VR_IS,"WedgeAngle",
	0x300A00D6,VR_DS,"WedgeFactor",
	0x300A00D8,VR_DS,"WedgeOrientation",
	0x300A00DA,VR_DS,"SourcetoWedgeTrayDistance",
	0x300A00E0,VR_IS,"NumberofCompensators",
	0x300A00E1,VR_SH,"MaterialID",
	0x300A00E2,VR_DS,"TotalCompensatorTrayFactor",
	0x300A00E3,VR_SQ,"CompensatorSequence",
	0x300A00E4,VR_IS,"CompensatorNumber",
	0x300A00E5,VR_SH,"CompensatorID",
	0x300A00E6,VR_DS,"SourcetoCompensatorTrayDistance",
	0x300A00E7,VR_IS,"CompensatorRows",
	0x300A00E8,VR_IS,"CompensatorColumns",
	0x300A00E9,VR_DS,"CompensatorPixelSpacing",
	0x300A00EA,VR_DS,"CompensatorPosition",
	0x300A00EB,VR_DS,"CompensatorTransmissionData",
	0x300A00EC,VR_DS,"CompensatorThicknessData",
	0x300A00ED,VR_IS,"NumberofBoli",
	0x300A00F0,VR_IS,"NumberofBlocks",
	0x300A00F2,VR_DS,"TotalBlockTrayFactor",
	0x300A00F4,VR_SQ,"BlockSequence",
	0x300A00F5,VR_SH,"BlockTrayID",
	0x300A00F6,VR_DS,"SourcetoBlockTrayDistance",
	0x300A00F8,VR_CS,"BlockType",
	0x300A00FA,VR_CS,"BlockDivergence",
	0x300A00FC,VR_IS,"BlockNumber",
	0x300A00FE,VR_LO,"BlockName",
	0x300A0100,VR_DS,"BlockThickness",
	0x300A0102,VR_DS,"BlockTransmission",
	0x300A0104,VR_IS,"BlockNumberofPoints",
	0x300A0106,VR_DS,"BlockData",
	0x300A0107,VR_SQ,"ApplicatorSequence",
	0x300A0108,VR_SH,"ApplicatorID",
	0x300A0109,VR_CS,"ApplicatorType",
	0x300A010A,VR_LO,"ApplicatorDescription",
	0x300A010C,VR_DS,"CumulativeDoseReferenceCoefficient",
	0x300A010E,VR_DS,"FinalCumulativeMetersetWeight",
	0x300A0110,VR_IS,"NumberofControlPoints",
	0x300A0111,VR_SQ,"ControlPointSequence",
	0x300A0112,VR_IS,"ControlPointIndex",
	0x300A0114,VR_DS,"NominalBeamEnergy",
	0x300A0115,VR_DS,"DoseRateSet",
	0x300A0116,VR_SQ,"WedgePositionSequence",
	0x300A0118,VR_CS,"WedgePosition",
	0x300A011A,VR_SQ,"BeamLimitingDevicePositionSequence",
	0x300A011C,VR_DS,"Leaf/JawPositions",
	0x300A011E,VR_DS,"GantryAngle",
	0x300A011F,VR_CS,"GantryRotationDirection",
	0x300A0120,VR_DS,"BeamLimitingDeviceAngle",
	0x300A0121,VR_CS,"BeamLimitingDeviceRotationDirection",
	0x300A0122,VR_DS,"PatientSupportAngle",
	0x300A0123,VR_CS,"PatientSupportRotationDirection",
	0x300A0124,VR_DS,"TableTopEccentricAxisDistance",
	0x300A0125,VR_DS,"TableTopEccentricAngle",
	0x300A0126,VR_CS,"TableTopEccentricRotationDirection",
	0x300A0128,VR_DS,"TableTopVerticalPosition",
	0x300A0129,VR_DS,"TableTopLongitudinalPosition",
	0x300A012A,VR_DS,"TableTopLateralPosition",
	0x300A012C,VR_DS,"IsocenterPosition",
	0x300A012E,VR_DS,"SurfaceEntryPoint",
	0x300A0130,VR_DS,"SourcetoSurfaceDistance",
	0x300A0134,VR_DS,"CumulativeMetersetWeight",
	0x300A0180,VR_SQ,"PatientSetupSequence",
	0x300A0182,VR_IS,"PatientSetupNumber",
	0x300A0184,VR_LO,"PatientAdditionalPosition",
	0x300A0190,VR_SQ,"FixationDeviceSequence",
	0x300A0192,VR_CS,"FixationDeviceType",
	0x300A0194,VR_SH,"FixationDeviceLabel",
	0x300A0196,VR_ST,"FixationDeviceDescription",
	0x300A0198,VR_SH,"FixationDevicePosition",
	0x300A01A0,VR_SQ,"ShieldingDeviceSequence",
	0x300A01A2,VR_CS,"ShieldingDeviceType",
	0x300A01A4,VR_SH,"ShieldingDeviceLabel",
	0x300A01A6,VR_ST,"ShieldingDeviceDescription",
	0x300A01A8,VR_SH,"ShieldingDevicePosition",
	0x300A01B0,VR_CS,"SetupTechnique",
	0x300A01B2,VR_ST,"SetupTechniqueDescription",
	0x300A01B4,VR_SQ,"SetupDeviceSequence",
	0x300A01B6,VR_CS,"SetupDeviceType",
	0x300A01B8,VR_SH,"SetupDeviceLabel",
	0x300A01BA,VR_ST,"SetupDeviceDescription",
	0x300A01BC,VR_DS,"SetupDeviceParameter",
	0x300A01D0,VR_ST,"SetupReferenceDescription",
	0x300A01D2,VR_DS,"TableTopVerticalSetupDisplacement",
	0x300A01D4,VR_DS,"TableTopLongitudinalSetupDisplacement",
	0x300A01D6,VR_DS,"TableTopLateralSetupDisplacement",
	0x300A0200,VR_CS,"BrachyTreatmentTechnique",
	0x300A0202,VR_CS,"BrachyTreatmentType",
	0x300A0206,VR_SQ,"TreatmentMachineSequence",
	0x300A0210,VR_SQ,"SourceSequence",
	0x300A0212,VR_IS,"SourceNumber",
	0x300A0214,VR_CS,"SourceType",
	0x300A0216,VR_LO,"SourceManufacturer",
	0x300A0218,VR_DS,"ActiveSourceDiameter",
	0x300A021A,VR_DS,"ActiveSourceLength",
	0x300A0222,VR_DS,"SourceEncapsulationNominalThickness",
	0x300A0224,VR_DS,"SourceEncapsulationNominalTransmission",
	0x300A0226,VR_LO,"SourceIsotopeName",
	0x300A0228,VR_DS,"SourceIsotopeHalfLife",
	0x300A022A,VR_DS,"ReferenceAirKermaRate",
	0x300A022C,VR_DA,"AirKermaRateReferenceDate",
	0x300A022E,VR_TM,"AirKermaRateReferenceTime",
	0x300A0230,VR_SQ,"ApplicationSetupSequence",
	0x300A0232,VR_CS,"ApplicationSetupType",
	0x300A0234,VR_IS,"ApplicationSetupNumber",
	0x300A0236,VR_LO,"ApplicationSetupName",
	0x300A0238,VR_LO,"ApplicationSetupManufacturer",
	0x300A0240,VR_IS,"TemplateNumber",
	0x300A0242,VR_SH,"TemplateType",
	0x300A0244,VR_LO,"TemplateName",
	0x300A0250,VR_DS,"TotalReferenceAirKerma",
	0x300A0260,VR_SQ,"BrachyAccessoryDeviceSequence",
	0x300A0262,VR_IS,"BrachyAccessoryDeviceNumber",
	0x300A0263,VR_SH,"BrachyAccessoryDeviceID",
	0x300A0264,VR_CS,"BrachyAccessoryDeviceType",
	0x300A0266,VR_LO,"BrachyAccessoryDeviceName",
	0x300A026A,VR_DS,"BrachyAccessoryDeviceNominalThickness",
	0x300A026C,VR_DS,"BrachyAccessoryDeviceNominalTransmission",
	0x300A0280,VR_SQ,"ChannelSequence",
	0x300A0282,VR_IS,"ChannelNumber",
	0x300A0284,VR_DS,"ChannelLength",
	0x300A0286,VR_DS,"ChannelTotalTime",
	0x300A0288,VR_CS,"SourceMovementType",
	0x300A028A,VR_IS,"NumberofPulses",
	0x300A028C,VR_DS,"PulseRepetitionInterval",
	0x300A0290,VR_IS,"SourceApplicatorNumber",
	0x300A0291,VR_SH,"SourceApplicatorID",
	0x300A0292,VR_CS,"SourceApplicatorType",
	0x300A0294,VR_LO,"SourceApplicatorName",
	0x300A0296,VR_DS,"SourceApplicatorLength",
	0x300A0298,VR_LO,"SourceApplicatorManufacturer",
	0x300A029C,VR_DS,"SourceApplicatorWallNominalThickness",
	0x300A029E,VR_DS,"SourceApplicatorWallNominalTransmission",
	0x300A02A0,VR_DS,"SourceApplicatorStepSize",
	0x300A02A2,VR_IS,"TransferTubeNumber",
	0x300A02A4,VR_DS,"TransferTubeLength",
	0x300A02B0,VR_SQ,"ChannelShieldSequence",
	0x300A02B2,VR_IS,"ChannelShieldNumber",
	0x300A02B3,VR_SH,"ChannelShieldID",
	0x300A02B4,VR_LO,"ChannelShieldName",
	0x300A02B8,VR_DS,"ChannelShieldNominalThickness",
	0x300A02BA,VR_DS,"ChannelShieldNominalTransmission",
	0x300A02C8,VR_DS,"FinalCumulativeTimeWeight",
	0x300A02D0,VR_SQ,"BrachyControlPointSequence",
	0x300A02D2,VR_DS,"ControlPointRelativePosition",
	0x300A02D4,VR_DS,"ControlPoint3DPosition",
	0x300A02D6,VR_DS,"CumulativeTimeWeight",
	0x300C0002,VR_SQ,"ReferencedRTPlanSequence",
	0x300C0004,VR_SQ,"ReferencedBeamSequence",
	0x300C0006,VR_IS,"ReferencedBeamNumber",
	0x300C0007,VR_IS,"ReferencedReferenceImageNumber",
	0x300C0008,VR_DS,"StartCumulativeMetersetWeight",
	0x300C0009,VR_DS,"EndCumulativeMetersetWeight",
	0x300C000A,VR_SQ,"ReferencedBrachyApplicationSetupSequence",
	0x300C000C,VR_IS,"ReferencedBrachyApplicationSetupNumber",
	0x300C000E,VR_IS,"ReferencedSourceNumber",
	0x300C0020,VR_SQ,"ReferencedFractionGroupSequence",
	0x300C0022,VR_IS,"ReferencedFractionGroupNumber",
	0x300C0040,VR_SQ,"ReferencedVerificationImageSequence",
	0x300C0042,VR_SQ,"ReferencedReferenceImageSequence",
	0x300C0050,VR_SQ,"ReferencedDoseReferenceSequence",
	0x300C0051,VR_IS,"ReferencedDoseReferenceNumber",
	0x300C0055,VR_SQ,"BrachyReferencedDoseReferenceSequence",
	0x300C0060,VR_SQ,"ReferencedStructureSetSequence",
	0x300C006A,VR_IS,"ReferencedPatientSetupNumber",
	0x300C0080,VR_SQ,"ReferencedDoseSequence",
	0x300C00A0,VR_IS,"ReferencedToleranceTableNumber",
	0x300C00B0,VR_SQ,"ReferencedBolusSequence",
	0x300C00C0,VR_IS,"ReferencedWedgeNumber",
	0x300C00D0,VR_IS,"ReferencedCompensatorNumber",
	0x300C00E0,VR_IS,"ReferencedBlockNumber",
	0x300C00F0,VR_IS,"ReferencedControlPointIndex",
	0x300E0002,VR_CS,"ApprovalStatus",
	0x300E0004,VR_DA,"ReviewDate",
	0x300E0005,VR_TM,"ReviewTime",
	0x300E0008,VR_PN,"ReviewerName",
	0x40080000,VR_UL,"GroupLength",
	0x40080040,VR_SH,"ResultsID",
	0x40080042,VR_LO,"ResultsIDIssuer",
	0x40080050,VR_SQ,"ReferencedInterpretationSequence",
	0x40080100,VR_DA,"InterpretationRecordedDate",
	0x40080101,VR_TM,"InterpretationRecordedTime",
	0x40080102,VR_PN,"InterpretationRecorder",
	0x40080103,VR_LO,"ReferencetoRecordedSound",
	0x40080108,VR_DA,"InterpretationTranscriptionDate",
	0x40080109,VR_TM,"InterpretationTranscriptionTime",
	0x4008010A,VR_PN,"InterpretationTranscriber",
	0x4008010B,VR_ST,"InterpretationText",
	0x4008010C,VR_PN,"InterpretationAuthor",
	0x40080111,VR_SQ,"InterpretationApproverSequence",
	0x40080112,VR_DA,"InterpretationApprovalDate",
	0x40080113,VR_TM,"InterpretationApprovalTime",
	0x40080114,VR_PN,"PhysicianApprovingInterpretation",
	0x40080115,VR_LT,"InterpretationDiagnosisDescription",
	0x40080117,VR_SQ,"InterpretationDiagnosisCodeSequence",
	0x40080118,VR_SQ,"ResultsDistributionListSequence",
	0x40080119,VR_PN,"DistributionName",
	0x4008011A,VR_LO,"DistributionAddress",
	0x40080200,VR_SH,"InterpretationID",
	0x40080202,VR_LO,"InterpretationIDIssuer",
	0x40080210,VR_CS,"InterpretationTypeID",
	0x40080212,VR_CS,"InterpretationStatusID",
	0x40080300,VR_ST,"Impressions",
	0x40084000,VR_ST,"ResultsComments",
	0x7FE00000,VR_UL,"GroupLength",
	0x7FE00010,VR_OB,"PixelDataOW",
	0xFFFCFFFC,VR_OB,"DataSetTrailingPadding",
};

static const DictEntry DICT_ENTRY50XX[] =
{
	0x50000000,VR_UL,"GroupLength",
	0x50000005,VR_US,"CurveDimensions",
	0x50000010,VR_US,"NumberofPoints",
	0x50000020,VR_CS,"TypeofData",
	0x50000022,VR_LO,"CurveDescription",
	0x50000030,VR_SH,"AxisUnits",
	0x50000040,VR_SH,"AxisLabels",
	0x50000103,VR_US,"DataValueRepresentation",
	0x50000104,VR_US,"MinimumCoordinateValue",
	0x50000105,VR_US,"MaximumCoordinateValue",
	0x50000106,VR_SH,"CurveRange",
	0x50000110,VR_US,"CurveDataDescriptor",
	0x50000112,VR_US,"CoordinateStartValue",
	0x50000114,VR_US,"CoordinateStepValue",
	0x50002000,VR_US,"AudioType",
	0x50002002,VR_US,"AudioSampleFormat",
	0x50002004,VR_US,"NumberofChannels",
	0x50002006,VR_UL,"NumberofSamples",
	0x50002008,VR_UL,"SampleRate",
	0x5000200A,VR_UL,"TotalTime",
	0x5000200C,VR_OW,"AudioSampleData",
	0x5000200E,VR_LT,"AudioComments",
	0x50002500,VR_LO,"CurveLabel",
	0x50002600,VR_SQ,"ReferencedOverlaySequence",
	0x50002610,VR_US,"ReferencedOverlayGroup",
	0x50003000,VR_OB,"CurveDataOW"
};

static const DictEntry DICT_ENTRY60XX[] =
{
	0x60000000,VR_UL,"GroupLength",
	0x60000010,VR_US,"OverlayRows",
	0x60000011,VR_US,"OverlayColumns",
	0x60000012,VR_US,"OverlayPlanes",
	0x60000015,VR_IS,"NumberofFramesinOverlay",
	0x60000022,VR_LO,"OverlayDescription",
	0x60000040,VR_CS,"OverlayType",
	0x60000045,VR_LO,"OverlaySubtype",
	0x60000050,VR_SS,"OverlayOrigin",
	0x60000051,VR_US,"ImageFrameOrigin",
	0x60000052,VR_US,"OverlayPlaneOrigin",
	0x60000100,VR_US,"OverlayBitsAllocated",
	0x60000102,VR_US,"OverlayBitPosition",
	0x60001301,VR_IS,"ROIArea",
	0x60001302,VR_DS,"ROIMean",
	0x60001303,VR_DS,"ROIStandardDeviation",
	0x60001500,VR_LO,"OverlayLabel",
	0x60003000,VR_OW,"OverlayData"
};

DataDict DataDict::m_instance;

#ifndef __VMS
struct MakeValueType
{
	typedef std::pair<VR, const char*> Item_T;
	typedef std::map<Tag, Item_T> Map_T;
	typedef Map_T::value_type Elem_T;
	Elem_T operator()(const DictEntry& item) {
		return Elem_T(Tag(item.tag), Item_T(item.vr, item.name)); }
};


DataDict::DataDict()
{
	using std::transform;
	using std::inserter;
	MakeValueType makeValueType;
	transform(DICT_ENTRY0, DICT_ENTRY0 + sizeof(DICT_ENTRY0)/sizeof(DICT_ENTRY0)[0],
		inserter(m_map0, m_map0.begin()), makeValueType);
	transform(DICT_ENTRY2, DICT_ENTRY2 + sizeof(DICT_ENTRY2)/sizeof(DICT_ENTRY2)[0],
		inserter(m_map2, m_map2.begin()), makeValueType);
	transform(DICT_ENTRY4, DICT_ENTRY4 + sizeof(DICT_ENTRY4)/sizeof(DICT_ENTRY4)[0],
		inserter(m_map4, m_map4.begin()), makeValueType);
	transform(DICT_ENTRY, DICT_ENTRY + sizeof(DICT_ENTRY)/sizeof(DICT_ENTRY)[0],
		inserter(m_map, m_map.begin()), makeValueType);
	transform(DICT_ENTRY50XX, DICT_ENTRY50XX + sizeof(DICT_ENTRY50XX)/sizeof(DICT_ENTRY50XX)[0],
		inserter(m_map50xx, m_map50xx.begin()), makeValueType);
	transform(DICT_ENTRY60XX, DICT_ENTRY60XX + sizeof(DICT_ENTRY60XX)/sizeof(DICT_ENTRY60XX)[0],
		inserter(m_map60xx, m_map60xx.begin()), makeValueType);
}
#else // __VMS
typedef std::pair<VR, const char*> Item_T;
typedef std::map<Tag, Item_T> Map_T;
typedef Map_T::value_type Elem_T;
static void fillMap(const DictEntry* first, const DictEntry* last, Map_T& m)
{
	Map_T::iterator iter = m.begin();
	while (first != last)
	{
		iter = m.insert(iter, Elem_T(Tag(first->tag), Item_T(first->vr, first->name)));
		first++;
	}
}

DataDict::DataDict()
{
	fillMap(DICT_ENTRY0, DICT_ENTRY0 + sizeof(DICT_ENTRY0)/sizeof(DICT_ENTRY0)[0],
		m_map0);
	fillMap(DICT_ENTRY2, DICT_ENTRY2 + sizeof(DICT_ENTRY2)/sizeof(DICT_ENTRY2)[0],
		m_map2);
	fillMap(DICT_ENTRY4, DICT_ENTRY4 + sizeof(DICT_ENTRY4)/sizeof(DICT_ENTRY4)[0],
		m_map4);
	fillMap(DICT_ENTRY, DICT_ENTRY + sizeof(DICT_ENTRY)/sizeof(DICT_ENTRY)[0],
		m_map);
	fillMap(DICT_ENTRY50XX, DICT_ENTRY50XX + sizeof(DICT_ENTRY50XX)/sizeof(DICT_ENTRY50XX)[0],
		m_map50xx);
	fillMap(DICT_ENTRY60XX, DICT_ENTRY60XX + sizeof(DICT_ENTRY60XX)/sizeof(DICT_ENTRY60XX)[0],
		m_map60xx);
}

#endif

bool DataDict::lookup(Tag tag, VR* vr_p, const char** name_p) const
{
	Iter it;
	switch (tag & 0xffff0000)
	{
	case 0x00000000:
		if (m_map0.end() == (it = m_map0.find(tag)))
			return false;
		break;
	case 0x00020000:
		if (m_map2.end() == (it = m_map2.find(tag)))
			return false;
		break;
	case 0x00040000:
		if (m_map4.end() == (it = m_map4.find(tag)))
			return false;
		break;
	default:
      switch (tag & 0xff000000)
      {
      case 0x50000000:
		if (m_map50xx.end() == (it = m_map50xx.find(Tag(tag & 0xff00ffff))))
			return false;
		break;
      case 0x60000000:
		if (m_map60xx.end() == (it = m_map60xx.find(Tag(tag & 0xff00ffff))))
			return false;
		break;
      default:
		if (m_map.end() == (it = m_map.find(tag)))
			return false;
		break;
      }
	}

	*vr_p = it->second.first;
	if (name_p)
		*name_p = it->second.second;

	return true;
}
