#ifndef __DATASET_H__
#define __DATASET_H__
// DataSet.h: definition of
//		class Value
//		typedef HValue
//		class DataSet
//		typedef HDataSet
//		class Sequence
//
//////////////////////////////////////////////////////////////////////
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "Handle.h"
#include "Tag.h"

#include <map>
class Sequence;
/*!
	This class is a form of VARIANT, that is, a class
	that can represent several different data types.
	Type information is stored in member m_vr, 
	and data is stored in m_length, m_data_p, or m_seq_p.

	The managment of pointers needs to be improved
*/
class Value  
{
	VR m_vr;
	UINT32 m_length;
	char* m_data_p;
	Sequence* m_seq_p;
private:
	Value(const Value&);			// not implemented
	Value& operator=(const Value&);	// not implemented
public:
	// Constructors
	Value(VR vr = VR_UN);
	Value(VR vr, UINT32 length, void* data_p);
	Value(VR vr, const std::string& str);
	Value(Sequence* seq_p);
	Value(UINT16 val);
	Value(INT16 val);
	Value(UINT32 val);
	Value(INT32 val);
	Value(float val);
	Value(double val);
	
	//Destructor
	~Value();

	// MANIPULATORS
	void setVR(VR vr) { m_vr = vr; }
	void setData(VR vr, UINT32 length, char* data_p);
	void setSQ(Sequence* seq_p);

	// ACCESSORS
	VR vr() const { return m_vr; }
	UINT32 length() const { return m_length; }
	const void* data() const { return m_data_p; }
	const Sequence* seq() const { return m_seq_p; }
	std::string str(VR vr) const;
	std::string str(VR vr, int index) const;
	UINT16 getUS(int index = 0) const;
	INT16 getSS(int index = 0) const;
	UINT32 getUL(int index = 0) const;
	INT32 getSL(int index = 0) const;
	float getFL(int index = 0) const;
	double getFD(int index = 0) const;
	bool getSafeUS(UINT16* val_p, int index = 0) const;
	bool getSafeSS(INT16* val_p, int index = 0) const;
	bool getSafeUL(UINT32* val_p, int index = 0) const;
	bool getSafeSL(INT32* val_p, int index = 0) const;
	bool getSafeFL(float* val_p, int index = 0) const;
	bool getSafeFD(double* val_p, int index = 0) const;
};

typedef Handle<Value> HValue;

class DataSet : public std::map<Tag, HValue>
{
private:
	//DataSet(const DataSet&);			// not implemented
	//DataSet& operator=(const DataSet&);	// not implemented

	const_iterator throwIfNotFind(Tag tag) const;	// helper
public:
	// CREATORS
	DataSet();


	// MANIPULATORS
	std::pair<iterator, bool>  insertElem(Tag tag, HValue val_h);
	std::pair<iterator, bool>  insertElem(Tag tag, VR vr);
	std::pair<iterator, bool>  insertElem(Tag tag, VR vr, UINT32 length, void* data_p);
	std::pair<iterator, bool>  insertElem(Tag tag, Sequence* seq_p);
	std::pair<iterator, bool>  insertElem(Tag tag, VR vr, const std::string& val);
	std::pair<iterator, bool>  insertElem(Tag tag, UINT16 val);
	std::pair<iterator, bool>  insertElem(Tag tag, INT16 val);
	std::pair<iterator, bool>  insertElem(Tag tag, UINT32 val);
	std::pair<iterator, bool>  insertElem(Tag tag, INT32 val);
	std::pair<iterator, bool>  insertElem(Tag tag, float val);
	std::pair<iterator, bool>  insertElem(Tag tag, double val);

	void insertElemFrom(Tag* tags, const DataSet& src);

	// ACCESSORS
	int getTotElem(bool nested) const;
	bool exist(Tag tag) const;
	HValue getHValue(Tag tag) const;
	const void* getData(Tag tag) const;
	const Sequence* getSQ(Tag tag) const;
	std::string getStr(Tag tag, VR vr) const;
	std::string getStr(Tag tag, VR vr, int index) const;
	UINT16 getUS(Tag tag, int index = 0) const;
	INT16 getSS(Tag tag, int index = 0) const;
	UINT32 getUL(Tag tag, int index = 0) const;
	INT32 getSL(Tag tag, int index = 0) const;
	float getFL(Tag tag, int index = 0) const;
	double getFD(Tag tag, int index = 0) const;
	bool getSafeHValue(HValue& val_h, Tag tag) const;
	bool getSafeData(const void** val_p, Tag tag) const;
	bool getSafeSQ(const Sequence** val_p, Tag tag) const;
	bool getSafeStr(std::string* val_p, Tag tag, VR vr) const;
	bool getSafeStr(std::string* val_p, Tag tag, VR vr, int index) const;
	bool getSafeUS(UINT16* val_p, Tag tag, int index = 0) const;
	bool getSafeSS(INT16* val_p, Tag tag, int index = 0) const;
	bool getSafeUL(UINT32* val_p, Tag tag, int index = 0) const;
	bool getSafeSL(INT32* val_p, Tag tag, int index = 0) const;
	bool getSafeFL(float* val_p, Tag tag, int index = 0) const;
	bool getSafeFD(double* val_p, Tag tag, int index = 0) const;
};

typedef Handle<DataSet> HDataSet;

class Sequence : public std::vector<HDataSet>
{
private:
	/*
		could use boost::noncopyable.
	*/
	Sequence(const Sequence&);			// not implemented
	Sequence& operator=(const Sequence&);	// not implemented
public:
	// Constructor
	Sequence() {}
};

#endif //__DATASET_H__