// DcmFile.cpp: implementation of the DcmFile class.
//
//////////////////////////////////////////////////////////////////////

#include "base.hpp"

#include "DcmFile.h"
#include "Decoder2.h"
#include "Encoder2.h"

using std::string;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
/*
	This constructor really shouldn't be allowed, because
	by failing to initialise m_fmi it leaves the object in 
	an unknown/invalid state.
*/
DcmFile::DcmFile()
{
}

DcmFile::DcmFile(const string& classUID, const string& instUID, TS ts)
: m_fmi(classUID, instUID, ts)
{
}

bool DcmFile::read(DataSet& ds)
{
  clear();
  TS ts(TS::IMPL_VR_LE);
  if (m_fmi.readFrom(*this))
    ts = m_fmi.getTS();
  return good() && readRawData(ds, ts);
}

bool DcmFile::readACR(DataSet& ds)
{
  clear();
  TS ts(TS::EXPL_VR_LE);
  if (m_fmi.readACRFrom(*this))
    ts = m_fmi.getTS();
  return good() && readRawData(ds, ts);
}

bool DcmFile::readRawData(DataSet& ds, TS ts)
{
  Decoder2 decoder(*this, ts);
  decoder.read(ds);
  if (rdstate() == (eofbit|failbit))
    clear();
  return good();
}

bool DcmFile::write(const DataSet& ds)
{
  return m_fmi.writeTo(*this) && writeRawData(ds, m_fmi.getTS());
}

bool DcmFile::writeRawData(const DataSet& ds, TS ts)
{
  Encoder2 encoder(*this, ts);
  encoder.write(ds);
  return good();
}
