#ifndef __DCMFILESET_H__
#define __DCMFILESET_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "FileMetaInfo.h"

class Decoder2;

struct CreateDirRec  
{
   enum Type {
      PATIENT,
      STUDY,
      SERIES,
      IMAGE,
      OVERLAY,
      MODALITY_LUT,
      VOI_LUT,
      CURVE,
      TOPIC,
      VISIT,
      RESULTS,
      INTERPRETATION,
      STUDY_COMPONENT,
      PRINT_QUEUE,
      FILM_SESSION,
      FILM_BOX,
      IMAGE_BOX,
      PRIVATE,
      MRDR
   };
   static void make(DataSet* pOut, Type type);
   static void makeFrom(DataSet* pOut, Type type, const DataSet& src);
   static void addRefFile(DataSet* pOut, const std::string& fileID, const FileMetaInfo& fmi);
   virtual void makePatKeyFrom(DataSet* pOut, const DataSet& src) const;
   virtual void makeStudyKeyFrom(DataSet* pOut, const DataSet& src) const;
   virtual void makeSeriesKeyFrom(DataSet* pOut, const DataSet& src) const;
   virtual void makeImageKeyFrom(DataSet* pOut, const DataSet& src) const;
};

class DcmFileSet : public std::fstream    
{
   FileMetaInfo m_fmi;
   std::string m_id;
   std::string m_descFile;
   std::string m_charSet;
   UINT32 m_firstRecPos;
   UINT32 m_lastRecPos;
   UINT16 m_consistency;
   UINT32 m_sqLen;
   UINT32 m_newRecPos;

   UINT32 m_firstRecPosPos;
   UINT32 m_lastRecPosPos;
   UINT32 m_consistencyPos;
   UINT32 m_sqLenPos;
public:
	// CREATORS
	DcmFileSet();
   DcmFileSet(const std::string& id, const std::string& instUID, TS ts);
	~DcmFileSet();

   // I/O
   bool readFileSetInfo();
   bool readItem(DataSet* pRec, UINT32 pos);
   bool readFirstItem(DataSet* pRec) { return readItem(pRec, m_firstRecPos); }

   bool writeFileSetInfo();
   UINT32 writeItem(const DataSet& ds);

   // MANIPULATORS
   FileMetaInfo& metaInfo() { return m_fmi; }
   void setDescFile(const std::string& str) { m_descFile = str; }
   void setCharSet(const std::string& str) { m_charSet = str; }
   void setConsistency(bool flag);
   void setNext(UINT32 pos, UINT32 val);
   void setLower(UINT32 pos, UINT32 val);
   void setInUse(UINT32 pos);
   void setInActive(UINT32 pos);
   UINT32 regImageFile(const std::string& path, const FileMetaInfo& fmi, const DataSet& ds,
      const CreateDirRec& cdr = CreateDirRec());

   // ACCESSORS
   const FileMetaInfo& metaInfo() const { return m_fmi; }
   const std::string& id() const { return m_id; }
   const std::string& descFile() const { return m_descFile; }
   const std::string& charSet() const { return m_charSet; }
   bool consistency() const { return m_consistency == 0; }
   UINT32 firstRecPos() const {  return m_firstRecPos; }
   UINT32 lastRecPos() const {  return m_lastRecPos; }
   UINT32 next(UINT32 pos);
   UINT32 lower(UINT32 pos);
   bool inUse(UINT32 pos);
   UINT32 findPatKey(DataSet* pRec, UINT32 pos, const std::string& id);
   UINT32 findStudyKey(DataSet* pRec, UINT32 pos, const std::string& id);
   UINT32 findSeriesKey(DataSet* pRec, UINT32 pos, const std::string& id);
   UINT32 findImageKey(DataSet* pRec, UINT32 pos, const std::string& id);
private:
   typedef std::vector<UINT32> PosVec;
   void init();
   bool readFileSetInfo1(Decoder2& decoder);
   UINT32 read32(UINT32 pos);
   UINT16 read16(UINT32 pos);
	void write16(UINT32 pos, UINT16 val);
	void write32(UINT32 pos, UINT32 val);
   void getChildRecPos(PosVec& posVec, UINT32 pos, UINT16 deep = 0xffff);
   UINT32 findKey(DataSet* pRec, UINT32 pos, const std::string& id, 
                                CreateDirRec::Type type, Tag tag, VR vr);
};


#endif //__DCMFILESET_H__