/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#ifndef __DECODER_H__
#define __DECODER_H__

#include "DataSet.h"
#include "TS.h"

class LinkedBuffer;

class Decoder 
{
	enum ErrorCode {
		NO_ERR, ERR_LEN0, ERR_ITEMTAG, ERR_NOITEMTAG, ERR_SEQ_DELTAG,
		ERR_LEN_INVALID, ERR_READ };
public:
	// CREATORS
	Decoder(LinkedBuffer& buffer, TS ts)
		: m_buffer(buffer), m_ts(ts), m_err(NO_ERR) {}

	// MANIPULATORS
	UINT32 read(DataSet::value_type& rhs);
	UINT32 read(DataSet& rhs, UINT32 len = 0xffffffff);
	UINT32 read(Sequence* pSeq, UINT32 len);

	// ACCESSORS
	ErrorCode error() const { return m_err; }
	bool good() const { return m_err == NO_ERR; }
private:
	LinkedBuffer& m_buffer;
	TS m_ts;
	ErrorCode m_err;
	enum { MAX_VALID_LENGTH = 80000000 };
};

#endif //__DECODER_H__