/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"

#include "Decoder2.h"
#include "DataDict.h"

#include "Endian.h"

using std::istream;

UINT32 Decoder2::read32()
{
  return m_ts.isBigEndian() ?
  (
    UINT32(m_is.get())<<24 |
    UINT32(m_is.get())<<16 | 
    UINT32(m_is.get())<<8 |
    UINT32(m_is.get())
  ):
  (
    UINT32(m_is.get()) |
    UINT32(m_is.get())<<8 | 
    UINT32(m_is.get())<<16 |
    UINT32(m_is.get())<<24
  );
}

UINT16 Decoder2::read16()
{
  return m_ts.isBigEndian() ?
  (
    UINT16(m_is.get())<<8 |
    UINT16(m_is.get())
  ):
  (
    UINT16(m_is.get()) |
    UINT16(m_is.get())<<8
  );
  return 2;
}

Tag Decoder2::readTag()
{
  UINT16 gggg = read16();
  UINT16 eeee = read16();
  return makeTag(gggg,eeee);
}

UINT32 Decoder2::read(DataSet::value_type& elem)
{
  Tag& tag = (Tag&)(elem.first);
  tag = readTag();
  if (!m_is)     // eof seen?
    return 0;

  UINT32 len;
  elem.second.bind(0);

  switch (tag)
  {
  case TAG_ITEM_DELIM_ITEM:
    if (read32() != 0)
      m_is.clear(m_is.rdstate()|istream::failbit);
    return 8;
  case TAG_ITEM:
  case TAG_SEQ_DELIM_ITEM:
    m_is.clear(m_is.rdstate()|istream::failbit);
    return 4;
  }

  VR vr = VR_UN;
  UINT32 readLen = sizeof(UINT16) + sizeof(UINT16);
  if (m_ts.isExplicitVR())
  {
    vr = VR(m_is.get() | (UINT(m_is.get()) << 8));
    len = read16();
    readLen += sizeof(UINT16) + sizeof(UINT16);
  }

  if (vr == VR_UN || vr == VR_SQ || vr == VR_OW || vr == VR_OB)
  {
    len = read32();
    readLen += sizeof(UINT32);
  }

  if (len != 0xffffffff && len > MAX_VALID_LENGTH)
  {
    m_is.clear(m_is.rdstate()|istream::failbit);
    return readLen;
  }

  if (getGrTag(tag) != 0x0000 && !m_ts.isExplicitVR())
  {
    if (tag == TAG_PIXEL_DATA ||
      tag == TAG_OVERLAY_DATA ||
      tag == TAG_RED_PAL_LUT ||
      tag == TAG_GREEN_PAL_LUT ||
      tag == TAG_BLUE_PAL_LUT ||
      tag == TAG_SEG_RED_PAL_LUT ||
      tag == TAG_SEG_GREEN_PAL_LUT ||
      tag == TAG_SEG_BLUE_PAL_LUT)
      vr = VR_OW;
    else if (tag == TAG_CURVE_DATA)
      vr = VR_OB;
    else
    {
      DataDict::instance().lookup(tag, &vr);
    }
  }

  Value* pValue = new Value(vr);
  elem.second.bind(pValue);

  if (len == 0)
	  return readLen;

  if (len == 0xffffffff || vr == VR_SQ)
  {
	  Sequence* pSeq = new Sequence;
	  pValue->setSQ(pSeq);
	  return readLen + read(pSeq, len);
  }
  char* data_p = new char[len];
  pValue->setData(vr, len, data_p);
  switch(m_ts.isBigEndian() ? vr : VR_OB)
  {
  case VR_US:
  case VR_SS:
  case VR_AT:
  case VR_OW:
  {
    LE_UINT16* pData = (LE_UINT16*)data_p;
    BE_UINT16 temp;
    for (int count = len / sizeof(UINT16); count--;)
    {
      m_is.read((char*)&temp, sizeof(UINT16));
      *pData++ = temp;
      readLen += sizeof(UINT16);
    }
  }
  break;
  case VR_UL:
  case VR_SL:
  case VR_FL:
  {
    LE_UINT32* pData = (LE_UINT32*)data_p;
    BE_UINT32 temp;
    for (int count = len / sizeof(UINT32); count--;)
    {
      m_is.read((char*)&temp, sizeof(UINT32));
      *pData++ = temp;
      readLen += sizeof(UINT32);
    }
  }
  break;
  case VR_FD:
  {
    LE_double* pData = (LE_double*)data_p;
    BE_double temp;
    for (int count = len / sizeof(double); count--;)
    {
      m_is.read((char*)&temp, sizeof(double));
      *pData++ = temp;
      readLen += sizeof(double);
    }
  }
  break;
  default:
  {
    m_is.read((char*)data_p, len);
    readLen += len;
  }
  }
  return readLen;
}

UINT32 Decoder2::read(DataSet& ds, UINT32 len)
{
  ds.clear();
  UINT32 readLen = 0;
  while (readLen < len)
  {
    DataSet::value_type elem;
    readLen += read(elem);
    if (!m_is)
      return readLen;

    switch(elem.first)
    {
    case TAG_ITEM_DELIM_ITEM:
    case TAG_DATA_SET_PADDING:
      return readLen;
    }
    if (getElTag(elem.first) != 0x0000)
      ds.insert(elem);
  }
  return readLen;
}

UINT32 Decoder2::readItem(DataSet& ds, Tag& tag)
{
  tag = readTag();
  UINT32 len = read32();
  switch(tag)
  {
  case TAG_ITEM:
    return read(ds, len) + 8;
  case TAG_SEQ_DELIM_ITEM:
  if (len == 0)
    return 8;
  }
  m_is.clear(m_is.rdstate()|istream::failbit);
  return 8;
}

UINT32 Decoder2::read(Sequence* pSeq, UINT32 len)
{
  UINT32 readLen = 0;
  while (readLen < len)
  {
  Tag tag;
  HDataSet hDS(new DataSet);
    readLen += readItem(*hDS, tag);
  if (!m_is || tag != TAG_ITEM)
    break;
    pSeq->push_back(hDS);
  }
  return readLen;
}
