/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"
#include "Dispatcher.h"
using std::string;

bool Dispatcher::addCStoreHandler(const string& classUID, CHandler* handler)
{
	return m_mapCStore.insert(CHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::addCFindHandler(const string& classUID, CHandler* handler)
{
	return m_mapCFind.insert(CHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::addCGetHandler(const string& classUID, CHandler* handler)
{
	return m_mapCGet.insert(CHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::addCMoveHandler(const string& classUID, CHandler* handler)
{
	return m_mapCMove.insert(CHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::addNCreateHandler(const string& classUID, NHandler* handler)
{
	return m_mapNCreate.insert(NHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::addNSetHandler(const string& classUID, NHandler* handler)
{
	return m_mapNSet.insert(NHandlerMap::value_type(classUID, handler)).second;
}

bool Dispatcher::dispatch(PDU_Service& pdu, const DataSet& rq) const
{
	UINT16 cmd;
	string classUID;
	if (!rq.getSafeUS(&cmd,TAG_CMD_FIELD) ||
		(!rq.getSafeStr(&classUID,TAG_AFF_SOP_CLASS_UID, VR_UI) &&
		!rq.getSafeStr(&classUID,TAG_REQ_SOP_CLASS_UID, VR_UI)))
		return false;	// TODO correct Error Response
	switch(cmd)
	{
		case C_ECHO_RQ:
			return CEchoSCP().handle(pdu, rq, classUID);
		case C_STORE_RQ:
			return CStoreSCP(getCHandler(m_mapCStore, classUID)).handle(pdu, rq, classUID);
		case C_GET_RQ:
			return CGetSCP(getCHandler(m_mapCGet, classUID)).handle(pdu, rq, classUID);
		case C_FIND_RQ:
			return CFindSCP(getCHandler(m_mapCFind, classUID)).handle(pdu, rq, classUID);
		case C_MOVE_RQ:
			return CMoveSCP(getCHandler(m_mapCMove, classUID)).handle(pdu, rq, classUID);
		case C_CANCEL_RQ:
			return true;	// ignored by this version
		case N_CREATE_RQ:
			return NCreateSCP(getNHandler(m_mapNCreate, classUID)).handle(pdu, rq, classUID);
		case N_SET_RQ:
			return NSetSCP(getNHandler(m_mapNSet, classUID)).handle(pdu, rq, classUID);
	}
	return false;	// TODO correct Error Response
}

Dispatcher::NoCHandler Dispatcher::s_noCHandler;
Dispatcher::NoNHandler Dispatcher::s_noNHandler;

CHandler* Dispatcher::getCHandler(const CHandlerMap& _map, const string& classUID)
{
	CHandlerMap::const_iterator iter = _map.find(classUID);
	return iter != _map.end() ? iter->second : &s_noCHandler;
}

UINT16 Dispatcher::NoCHandler::handle(int sock, const DataSet& rq, const DataSet& data, Sequence* sq)
{
	return ST_UNKN_SOP_CLASS;
}

NHandler* Dispatcher::getNHandler(const NHandlerMap& _map, const string& classUID)
{
	NHandlerMap::const_iterator iter = _map.find(classUID);
	return iter != _map.end() ? iter->second : &s_noNHandler;
}

UINT16 Dispatcher::NoNHandler::handle(int sock, DataSet* rspData_p, string& instUID,
			const DataSet& rqCmd, const DataSet& rqData)
{
	return ST_UNKN_SOP_CLASS;
}