#ifndef __DISPATCHER_H__
#define __DISPATCHER_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "Cdimse.h"
#include "Ndimse.h"

class Dispatcher  
{
public:
	bool addCStoreHandler(const std::string& classUID, CHandler* handler = 0);
	bool addCFindHandler(const std::string& classUID, CHandler* handler = 0);
	bool addCGetHandler(const std::string& classUID, CHandler* handler = 0);
	bool addCMoveHandler(const std::string& classUID, CHandler* handler = 0);
	bool addNCreateHandler(const std::string& classUID, NHandler* handler = 0);
	bool addNSetHandler(const std::string& classUID, NHandler* handler = 0);

	bool dispatch(PDU_Service& pdu, const DataSet& rq) const;
private:
	typedef std::map<std::string,CHandler*> CHandlerMap;
	typedef std::map<std::string,NHandler*> NHandlerMap;
	static CHandler* getCHandler(const CHandlerMap& _map, const std::string& classUID);
	static NHandler* getNHandler(const NHandlerMap& _map, const std::string& classUID);
private:
	CHandlerMap m_mapCStore;
	CHandlerMap m_mapCFind;
	CHandlerMap m_mapCGet;
	CHandlerMap m_mapCMove;
	NHandlerMap m_mapNCreate;
	NHandlerMap m_mapNSet;
	class NoCHandler : public CHandler
	{
	public:
		virtual UINT16 handle(int sock, const DataSet& rq, const DataSet& data, Sequence* sq);
	};
	class NoNHandler : public NHandler
	{
	public:
		virtual UINT16 handle(int sock, DataSet* rspData_p, std::string& instUID,
			const DataSet& rqCmd, const DataSet& rqData);
	};
	static NoCHandler s_noCHandler;
	static NoNHandler s_noNHandler;
};

#endif //__DISPATCHER_H__