/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"

#include "Encoder2.h"
#include "DataDict.h"

#include "Endian.h"

UINT32 Encoder2::write32(UINT32 val)
{
   if (m_ts.isBigEndian())
   {
      m_os.put(val>>24);
      m_os.put(val>>16);
      m_os.put(val>>8);
      m_os.put(val);
   } else
   {
      m_os.put(val);
      m_os.put(val>>8);
      m_os.put(val>>16);
      m_os.put(val>>24);
   }
   return 4;
}

UINT32 Encoder2::write16(UINT16 val)
{
   if (m_ts.isBigEndian())
   {
      m_os.put(val>>8);
      m_os.put(val);
   } else
   {
      m_os.put(val);
      m_os.put(val>>8);
   }
   return 2;
}

UINT32 Encoder2::writeTag(const Tag& tag)
{
   return write16(getGrTag(tag)) + write16(getElTag(tag));
}

UINT32 Encoder2::write(const DataSet::value_type& elem)
{
   writeTag(elem.first);

	UINT32 writeLen = sizeof(UINT16) + sizeof(UINT16) + sizeof(UINT32);
	UINT32 len = elem.second->length();
	if (m_ts.isExplicitVR())
	{
		VR vr = elem.second->vr();
		if (vr == VR_UN)
			DataDict::instance().lookup(elem.first, &vr);

		m_os << char(vr) << char(vr >> 8);
		switch(vr)
		{
		case VR_OB:
		case VR_OW:
		case VR_SQ:
		case VR_UN:
         write16(0);
         write32(len);
			writeLen += sizeof(UINT32);
		break;
		default :
         write16(len);
		break;
	  }
	} else
      write32(len);

	if (len > 0)
		writeLen += write(*elem.second);
	return writeLen;
}

UINT32 Encoder2::write(const Value& val)
{
	const Sequence* seq_p = val.seq();
	if (seq_p != 0)
		return write(seq_p);

	VR vr = val.vr();
	int len = val.length();
	ASSERT((len&1) == 0);
	switch(m_ts.isBigEndian() ? vr : VR_OB)
	{
	case VR_US:
	case VR_SS:
	case VR_AT:
	case VR_OW:
		{
			const LE_UINT16* pData = (LE_UINT16*)val.data();
			BE_UINT16 temp;
			for (int count = len / sizeof(UINT16); count--;)
			{
				temp = *pData++;
				m_os.write((char*)&temp, sizeof(UINT16));
			}
		}
		break;
	case VR_UL:
	case VR_SL:
	case VR_FL:
		{
			const LE_UINT32* pData = (LE_UINT32*)val.data();
			BE_UINT32 temp;
			for (int count = len / sizeof(UINT32); count--;)
			{
				temp = *pData++;
				m_os.write((char*)&temp, sizeof(UINT32));
			}
		}
		break;
	case VR_FD:
		{
			const LE_double* pData = (LE_double*)val.data();
			BE_double temp;
			for (int count = len / sizeof(double); count--;)
			{
				temp = *pData++;
				m_os.write((char*)&temp, sizeof(double));
			}
		}
		break;
	default:
		{
			const char* pData = (char*)val.data();
			m_os.write(pData, len);
		}
	}
	return len;
}

UINT32 Encoder2::write(const DataSet& ds)
{
//	ASSERT(!ds.empty());

	UINT32 writeLen = 0;
	DataSet::const_iterator iter = ds.begin();
	UINT16 g0 = getGrTag(iter->first);
	if (g0 == 0x0000 || g0 == 0x0002)
	{
		DataSet::value_type elem(makeTag(g0,0x0000), new Value(getGrLen(ds)));
		writeLen += write(elem);
	}
	DataSet::const_iterator last = ds.end();
	for (; iter != last; ++iter)
		writeLen += write(*iter);
	return writeLen;
}

UINT32 Encoder2::writeItem(const DataSet& ds)
{
   writeTag(TAG_ITEM);
   write32(UINT32(0xffffffff));
	UINT32 writeLen = write(ds) + 16;
	writeTag(TAG_ITEM_DELIM_ITEM);
   write32(UINT32(0x00000000));
   return writeLen;
}

UINT32 Encoder2::write(const Sequence* seq_p)
{
//	ASSERT(!seq_p->empty()); //Empty SQ does happens - Sam Shen 

	UINT32 writeLen = 0;
	Sequence::const_iterator iter = seq_p->begin();
	Sequence::const_iterator last = seq_p->end();
	for (; iter != last; ++iter)
	{
      writeLen += writeItem(**iter);
	}
	writeTag(TAG_SEQ_DELIM_ITEM);
	write32(UINT32(0x00000000));
	writeLen += 8;
	return writeLen;
}

UINT32 Encoder2::getGrLen(const DataSet &ds)
{
	UINT32 grLen = 0;
	DataSet::const_iterator iter = ds.begin();
	UINT16 grTag0 = getGrTag(iter->first);

	ASSERT(m_ts.isExplicitVR() ? grTag0 == 0x0002 : grTag0 == 0x0000); 

	DataSet::const_iterator last = ds.end();
	for (; iter != last; ++iter)
	{
		ASSERT(getGrTag(iter->first) == grTag0);
		const Value& v = *iter->second;
		VR vr = v.vr();
		ASSERT(vr != VR_OW && vr != VR_SQ && vr != VR_UN);
		ASSERT(vr != VR_OB || grTag0 == 0x0002);

		grLen += (vr != VR_OB) ? 8 : 12;
		grLen += v.length();
	}
	return grLen;
}