#ifndef ENDIAN_H_INCLUDE_GUARD
#define ENDIAN_H_INCLUDE_GUARD

/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

// Architectually Neutral Endian types.
//
template <class T>
class SwitchEndian
{
	T m_data;
	static T switchBytes(T x)
	{
		T y;
		const char* x_p = (const char*)&x;
		char* y_p = (char*)&y + sizeof(T);
		int i = sizeof(T);
		while (i--)
			*--y_p = *x_p++;
		return y;
	}
public:
	SwitchEndian() { m_data = 0; }
	SwitchEndian(const SwitchEndian& rhs) { m_data = rhs.m_data; }
	SwitchEndian(T x) { m_data = switchBytes(x); }
	SwitchEndian& operator=(const SwitchEndian& rhs) {
		m_data = rhs.m_data; return *this; }
	SwitchEndian& operator=(T x) {
		m_data = switchBytes(x); return *this; }
	operator T() const { return switchBytes(m_data); }
	SwitchEndian* operator &() { return (SwitchEndian*)&m_data; }
};

#if	NATIVE_ENDIAN	==	LITTLE_ENDIAN

typedef	UINT16 LE_UINT16;
typedef	UINT32 LE_UINT32;
typedef	INT16 LE_INT16;
typedef	INT32 LE_INT32;
typedef	float LE_float;
typedef	double LE_double;
typedef	SwitchEndian<UINT16> BE_UINT16;
typedef	SwitchEndian<UINT32> BE_UINT32;
typedef	SwitchEndian<INT16> BE_INT16;
typedef	SwitchEndian<INT32> BE_INT32;
typedef	SwitchEndian<float> BE_float;
typedef	SwitchEndian<double> BE_double;

#else

typedef	UINT16 BE_UINT16;
typedef	UINT32 BE_UINT32;
typedef	INT16 BE_INT16;
typedef	INT32 BE_INT32;
typedef	float BE_float;
typedef	double BE_double;
typedef	SwitchEndian<UINT16> LE_UINT16;
typedef	SwitchEndian<UINT32> LE_UINT32;
typedef	SwitchEndian<INT16> LE_INT16;
typedef	SwitchEndian<INT32> LE_INT32;
typedef	SwitchEndian<float> LE_float;
typedef	SwitchEndian<double> LE_double;

#endif




#endif //ENDIAN_H_INCLUDE_GUARD