/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

// Handle Class from Stroustrup's The C++ Programming Language, 3. Edition, p.783

#ifndef __HANDLE_H__
#define __HANDLE_H__

template<class X> class Handle {
	X* rep;
	int* pcount;
public:
	X* operator->() { return rep; }
	const X* operator->() const { return rep; }
	X& operator*() { return *rep; }
	const X& operator*() const { return *rep; }

	Handle(X* pp = 0) : rep(pp), pcount(new int(1)) {}
	Handle(const Handle& r) : rep(r.rep), pcount(r.pcount) { (*pcount)++; }

	Handle& operator=(const Handle& r)
	{
		if (rep == r.rep) return *this;
		if (--(*pcount) == 0) {
			delete rep;
			delete pcount;
		}
		rep = r.rep;
		pcount = r.pcount;
		(*pcount)++;
		return *this;
	}
	
	~Handle() { if (--(*pcount) == 0) { delete rep; delete pcount; }}

	X* get_rep() { return rep; }
	const X* get_rep() const { return rep; }

	void bind(X* pp)
	{
		if (pp != rep) {
			if (--(*pcount) == 0) {
				delete rep;
				*pcount = 1;			// recycle pcount
			}
			else
				pcount = new int(1);	// new pcount
			rep = pp;
		}
	}
};

#endif //__HANDLE_H__