/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"
#include "Impl.h"
#include <time.h>
#include <sstream>
#include <string>
#include <exception>
const char* makeUID(char* buffer)
{
  sprintf(buffer, "%s.999.%i.%i", IMPL_CLASS_UID, time(0), clock());
  return buffer;
}
/*
	I'd like to make this a bit more robust.  Trevor Morgan.
*/
/*
proposed alternate implementation:

std::string makeUID(std::string Prefix, bool ApplyImplementationClassUID=true)
{
	std::stringstream stream;
	stream << Prefix ;
	if(ApplyImplementationClassUID)
		stream <<"." << IMPL_CLASS_UID;
	stream << "."	<< time(0)<< "." << clock();
	stream << "."	<< SomeRandomNumber;
	return stream.str();
}
also should take into account threading, checking against immediate
duplicates.
Is there a standard or implmentation specific limit on the length of UIDs ?
It seems that STRADA has an implementation limit of 64 characters on a UID.
IMPL_CLASS_UID + UID_SUNNYBROOK_YAFFEGRP_PREFIX = 38 characters, gives us 26 left to 
play with.
time(0) = 10 characters.
clock = 4 characters(<10 seconds) -> 8 characters (24 hours).  Say 10 to be on safe side.
38+10+10=58 + 4 * '.' = 62.
No room for any more.
There again, I don't really see the benefit in including IMPL_CLASS_UID in 
every generated UID.  Especially if this library is used more or less universally
in the group, that would be pretty redundant.
*/
/*!
	This is not GUARANTEED to be unique.
*/
std::string makeUID(const std::string &Prefix)
{
	std::stringstream stream;
	stream	<< Prefix
			<< "."		<<	time(0)
			<< "."		<<	clock()
			<< "."		<<	rand();
	std::string UID=stream.str();
	if(UID.size()>64)
		throw std::exception("Generated UID is larger than 64 characters.");
	return UID;

}