/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"

#include "MsgLogger.h"
#include "Dumper.h"

enum ENUM_PDU { AA_RQ, AA_AC, AA_RJ, AR_RQ, AR_RP, AAB_RQ, PD_TF, NO_PDU };
using std::endl;

static const char* PDU_NAMES [] =
{
  "A-ASSOCIATE-RQ",
  "A-ASSOCIATE-AC",
  "A-ASSOCIATE-RJ",
  "A-RELEASE-RQ",
  "A-RELEASE-RP",
  "A-ABORT-RQ",
  "P-DATA-TF"
};


MsgLogger* MsgLogger::s_inst_p = 0;

DefMsgLogger::DefMsgLogger(std::ostream& os)
:	m_os(os)
{
}

void DefMsgLogger::logListenSock(const std::string& port)
{
  m_os << "Start listen port - " << port << endl;
}

void DefMsgLogger::logAcceptSock(int sock)
{
  m_os << "Accept connection socket - #" << sock << endl;
}

void DefMsgLogger::logAcceptAssoc(int sock)
{
  m_os << '#' << sock << ": Accept association" << endl;
}

void DefMsgLogger::logRejectAssoc(int sock, int result, int source, int reason)
{
  m_os << '#' << sock << ": Reject association - Result:" << result
	  << ", Source:" << source << ", Reason:" << reason << endl;
}

void DefMsgLogger::logReleaseAssoc(int sock)
{
  m_os << '#' << sock << ": Release association" << endl;
}

void DefMsgLogger::logStartThread(int sock)
{
  m_os << '#' << sock << ": Start thread" << endl;
}

void DefMsgLogger::logExitThread(int sock)
{
  m_os << '#' << sock << ": Exit thread" << endl;
}

void DefMsgLogger::logReceivedData(int sock, const DataSet& data)
{
  m_os << '#' << sock << ": Received command/data" << endl;
  Dumper dumper(m_os);
  dumper.write(data);
  m_os.flush();
}

void DefMsgLogger::logTransmittedData(int sock, const DataSet& data)
{
  m_os << '#' << sock << ": Transmitted command/data" << endl;
  Dumper dumper(m_os);
  dumper.write(data);
  m_os.flush();
}

void DefMsgLogger::logReceivedPdu(int sock, int id)
{
  m_os << '#' << sock << ": Receive " << PDU_NAMES[id] << endl;
}

void DefMsgLogger::logAcceptRemoteApTitle(int sock, const std::string& aet, bool accept)
{
  m_os << '#' << sock << (accept ? ": Accept calling AET - " : ": Reject calling AET - ") << aet << endl;
}

void DefMsgLogger::logAcceptLocalApTitle(int sock, const std::string& aet, bool accept)
{
  m_os << '#' << sock << (accept ? ": Accept called AET - " : ": Reject called AET - ") << aet << endl;
}

void DefMsgLogger::logAcceptApplicationContext(int sock, const std::string& uid, bool accept)
{
  m_os << '#' << sock << (accept ? ": Accept Application Context - " : ": Reject Application Context - ") << uid << endl;
}

void DefMsgLogger::logAcceptAbstractSyntax(int sock, const std::string& uid, bool accept)
{
  m_os << '#' << sock << (accept ? ": Accept Abstract Syntax - " : ": Reject Abstract Syntax - ") << uid << endl;
}

ErrLogger* ErrLogger::s_inst_p = 0;

DefErrLogger::DefErrLogger(std::ostream& os)
:	m_os(os)
{
}

void DefErrLogger::logListenSockFailed(const std::string& port)
{
  m_os << "Listen port " << port << "failed!" << endl;
}

void DefErrLogger::logAcceptSockFailed(const std::string& port)
{
  m_os << "Accept connection on port " << port << "failed!" << endl;
}

void DefErrLogger::logStartThreadFailed(int sock)
{
  m_os << '#' << sock << ": Start thread failed!" << endl;
}

void DefErrLogger::logDispatchFailed(int sock)
{
  m_os << '#' << sock << ": Dispatch command failed!" << endl;
}

void DefErrLogger::logReadDataFailed(int sock)
{
  m_os << '#' << sock << ": Reading Command/Data failed!" << endl;
}

void DefErrLogger::logWriteDataFailed(int sock)
{
  m_os << '#' << sock << ": Writing Command/Data failed!" << endl;
}

void DefErrLogger::logReceivedPdu(int sock, int id)
{
  m_os << '#' << sock << ": Receive unexpected " << PDU_NAMES[id] << endl;
}

void DefErrLogger::logTimeOutError(int sock)
{
  m_os << '#' << sock << ": Run into Time Out!" << endl;
}

void DefErrLogger::logTcpCloseError(int sock)
{
  m_os << '#' << sock << ": TCP connection has been closed!" << endl;
}

void DefErrLogger::logProtocolError(int sock)
{
  m_os << '#' << sock << ": Protocol Error!" << endl;
}

