#ifndef __MSGLOGGER_H__
#define __MSGLOGGER_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

class DataSet;

class MsgLogger  
{
  static MsgLogger* s_inst_p;
public:
  static void set(MsgLogger* inst_p) { s_inst_p = inst_p; }
  static MsgLogger* get() { return s_inst_p; }

  virtual ~MsgLogger() {}
  virtual void logListenSock(const std::string& port) = 0;
  virtual void logAcceptSock(int sock) = 0;
  virtual void logAcceptAssoc(int sock) = 0;
  virtual void logRejectAssoc(int sock, int result, int source, int reason) = 0;
  virtual void logReleaseAssoc(int sock) = 0;
  virtual void logStartThread(int sock) = 0;
  virtual void logExitThread(int sock) = 0;
  virtual void logReceivedData(int sock, const DataSet& cmd) = 0;
  virtual void logTransmittedData(int sock, const DataSet& cmd) = 0;
  virtual void logReceivedPdu(int sock, int id) = 0;
  virtual void logAcceptRemoteApTitle(int sock, const std::string& aet, bool accept) = 0;
  virtual void logAcceptLocalApTitle(int sock, const std::string& aet, bool accept) = 0;
  virtual void logAcceptApplicationContext(int sock, const std::string& uid, bool accept) = 0;
  virtual void logAcceptAbstractSyntax(int sock, const std::string& uid, bool accept) = 0;
};

class DefMsgLogger : public MsgLogger
{
  std::ostream& m_os;
public:
  DefMsgLogger(std::ostream& os);
  virtual void logListenSock(const std::string& port);
  virtual void logAcceptSock(int sock);
  virtual void logAcceptAssoc(int sock);
  virtual void logRejectAssoc(int sock, int result, int source, int reason);
  virtual void logReleaseAssoc(int sock);
  virtual void logStartThread(int sock);
  virtual void logExitThread(int sock);
  virtual void logReceivedData(int sock, const DataSet& data);
  virtual void logTransmittedData(int sock, const DataSet& cmd);
  virtual void logReceivedPdu(int sock, int id);
  virtual void logAcceptRemoteApTitle(int sock, const std::string& aet, bool accept);
  virtual void logAcceptLocalApTitle(int sock, const std::string& aet, bool accept);
  virtual void logAcceptApplicationContext(int sock, const std::string& uid, bool accept);
  virtual void logAcceptAbstractSyntax(int sock, const std::string& uid, bool accept);
  std::ostream& os() { return m_os; }
};

class ErrLogger  
{
  static ErrLogger* s_inst_p;
public:
  static void set(ErrLogger* inst_p) { s_inst_p = inst_p; }
  static ErrLogger* get() { return s_inst_p; }

  virtual ~ErrLogger() {}
  virtual void logListenSockFailed(const std::string& port) = 0;
  virtual void logAcceptSockFailed(const std::string& port) = 0;
  virtual void logStartThreadFailed(int sock) = 0;
  virtual void logDispatchFailed(int sock) = 0;
  virtual void logReadDataFailed(int sock) = 0;
  virtual void logWriteDataFailed(int sock) = 0;
  virtual void logReceivedPdu(int sock, int id) = 0;
  virtual void logProtocolError(int sock) = 0;
  virtual void logTimeOutError(int sock) = 0;
  virtual void logTcpCloseError(int sock) = 0;
};

class DefErrLogger : public ErrLogger
{
	std::ostream& m_os;
public:
  DefErrLogger(std::ostream& os);
  virtual void logListenSockFailed(const std::string& port);
  virtual void logAcceptSockFailed(const std::string& port);
  virtual void logStartThreadFailed(int sock);
  virtual void logWriteDataFailed(int sock);
  virtual void logReadDataFailed(int sock);
  virtual void logDispatchFailed(int sock);
  virtual void logReceivedPdu(int sock, int id);
  virtual void logProtocolError(int sock);
  virtual void logTimeOutError(int sock);
  virtual void logTcpCloseError(int sock);
  std::ostream& os() { return m_os; }
};

#endif //__MSGLOGGER_H__