/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"
#include "Ndimse.h"

#include "Pdu.h"
#include "Impl.h"

NEventReportRQ::NEventReportRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 eventTypID, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_EVENT_REPORT_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_AFF_SOP_INST_UID, VR_UI, instUID);
  insertElem(TAG_EVENT_TYPE_ID, eventTypID);
}

NEventReportRSP::NEventReportRSP(UINT16 msgID, const std::string& classUID, UINT16 stat,
								 UINT16 eventTypID, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_EVENT_REPORT_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_STATUS, stat);
  if (stat == ST_SUCCESS)
    insertElem(TAG_EVENT_TYPE_ID, eventTypID);
}

NGetRQ::NGetRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		const TagVector& attrList)
{
  insertElem(TAG_REQ_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_GET_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
  insertElem(TAG_REQ_SOP_INST_UID, VR_UI, instUID);
//  insertElem(
//    TAG_ATTR_ID_LIST, attrList);
}

NGetRSP::NGetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_GET_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_STATUS, stat);
}

NSetRQ::NSetRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID)
{
  insertElem(TAG_REQ_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_SET_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, YES_DATA_SET);
  insertElem(TAG_REQ_SOP_INST_UID, VR_UI, instUID);
}

NSetRSP::NSetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_SET_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_STATUS, stat);
}

NActionRQ::NActionRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 actionTypID, UINT16 dsType)
{
  insertElem(TAG_REQ_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_ACTION_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_REQ_SOP_INST_UID, VR_UI, instUID);
  insertElem(TAG_ACTION_TYPE_ID, actionTypID);
}

NActionRSP::NActionRSP(UINT16 msgID, const std::string& classUID, UINT16 stat,
					   UINT16 actionTypID, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_ACTION_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_STATUS, stat);
  if (stat == ST_SUCCESS)
    insertElem(TAG_ACTION_TYPE_ID, actionTypID);
}

NCreateRQ::NCreateRQ(UINT16 msgID, const std::string& classUID, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_CREATE_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
}

NCreateRQ::NCreateRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
					 UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_CREATE_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_AFF_SOP_INST_UID, VR_UI, instUID);
}

NCreateRSP::NCreateRSP(UINT16 msgID, const std::string& classUID, const std::string& instUID,
					   UINT16 stat, UINT16 dsType)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_CREATE_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, dsType);
  insertElem(TAG_STATUS, stat);
  if (stat == ST_SUCCESS)
    insertElem(TAG_AFF_SOP_INST_UID, VR_UI, instUID);
}

NDeleteRQ::NDeleteRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID)
{
  insertElem(TAG_REQ_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_DELETE_RQ);
  insertElem(TAG_MSG_ID, msgID);
  insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
  insertElem(TAG_REQ_SOP_INST_UID, VR_UI, instUID);
}

NDeleteRSP::NDeleteRSP(UINT16 msgID, const std::string& classUID, UINT16 stat)
{
  insertElem(TAG_AFF_SOP_CLASS_UID, VR_UI, classUID);
  insertElem(TAG_CMD_FIELD, N_DELETE_RSP);
  insertElem(TAG_MSG_ID_RSP, msgID);
  insertElem(TAG_DATA_SET_TYPE, NO_DATA_SET);
  insertElem(TAG_STATUS, stat);
}

bool NCreateSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
  UINT16 msgID = 0;
  if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
    return false;

  UINT16 dstype = 0;
  if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
    return false;

  std::string instUID;
  rqCmd.getSafeStr(&instUID, TAG_AFF_SOP_INST_UID, VR_UI);
  if (instUID.empty())
  {
    char buffer[64];
    instUID = makeUID(buffer);
  }

  DataSet rqData;
  if (dstype != NO_DATA_SET)
  {
    if (pdu.Read(rqData) == false)
      return false;
  }

  UINT16 stat = ST_SUCCESS;
  DataSet rspData;
  if (m_pHandler)
    stat = m_pHandler->handle(pdu.GetSocketfd(), &rspData, instUID, rqCmd, rqData);

  NCreateRSP rspCmd(msgID, classUID, instUID, stat,
    rspData.empty() ? NO_DATA_SET : YES_DATA_SET);
  return 
    pdu.WriteCommand(rspCmd, classUID) != false &&
    (rspData.empty() || pdu.WriteDataSet(rspData, classUID) != false);
}

bool NSetSCP::handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID)
{
  UINT16 msgID = 0;
  if (!rqCmd.getSafeUS(&msgID, TAG_MSG_ID))
    return false;

  UINT16 dstype = 0;
  if (!rqCmd.getSafeUS(&dstype, TAG_DATA_SET_TYPE))
    return false;

  std::string instUID;
  if (!rqCmd.getSafeStr(&instUID, TAG_REQ_SOP_INST_UID, VR_UI))
    return false;

  if (dstype == NO_DATA_SET)
    return false;

  DataSet rqData;
  if (pdu.Read(rqData) == false)
    return false;

  UINT16 stat = ST_SUCCESS;
  DataSet rspData;
  if (m_pHandler)
    stat = m_pHandler->handle(pdu.GetSocketfd(), &rspData, instUID, rqCmd, rqData);

  NSetRSP rspCmd(msgID, classUID, stat,
    rspData.empty() ? NO_DATA_SET : YES_DATA_SET);
  return 
    pdu.WriteCommand(rspCmd, classUID) != false &&
    (rspData.empty() || pdu.WriteDataSet(rspData, classUID) != false);
}

NCreateSCU::NCreateSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool NCreateSCU::writeRQ(PDU_Service& pdu, const std::string& instUID, const DataSet& data)
{
  NCreateRQ rqCmd(uniq16odd(), m_classUID, instUID, YES_DATA_SET);
  return pdu.WriteCommand(rqCmd, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool NCreateSCU::writeRQ(PDU_Service& pdu, const DataSet& data)
{
  NCreateRQ rqCmd(uniq16odd(), m_classUID, YES_DATA_SET);
  return pdu.WriteCommand(rqCmd, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool NCreateSCU::writeRQ(PDU_Service& pdu, const std::string& instUID)
{
  NCreateRQ rqCmd(uniq16odd(), m_classUID, instUID, NO_DATA_SET);
  return pdu.WriteCommand(rqCmd, m_classUID) != false;
}

bool NCreateSCU::writeRQ(PDU_Service& pdu)
{
  NCreateRQ rqCmd(uniq16odd(), m_classUID, NO_DATA_SET);
  return pdu.WriteCommand(rqCmd, m_classUID) != false;
}

bool NCreateSCU::readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu)
{
  DataSet rsp;
  return readRSP(stat_p, &rsp, data_p, pdu);
}

bool NCreateSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu)
{
  UINT16 dstype = 0;
  return pdu.Read(*rsp_p) != false &&
    rsp_p->getSafeUS(&dstype, TAG_DATA_SET_TYPE) &&
    rsp_p->getSafeUS(stat_p, TAG_STATUS) &&
    (dstype != NO_DATA_SET ? pdu.Read(*data_p) != false : true);
}

NSetSCU::NSetSCU(const std::string& classUID)
: m_classUID(classUID)
{
}

bool NSetSCU::writeRQ(PDU_Service& pdu, const std::string& instUID, const DataSet& data)
{
  NSetRQ rqCmd(uniq16odd(), m_classUID, instUID);
  return pdu.WriteCommand(rqCmd, m_classUID) && pdu.WriteDataSet(data, m_classUID);
}

bool NSetSCU::readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu)
{
  DataSet rsp;
  return readRSP(stat_p, &rsp, data_p, pdu);
}

bool NSetSCU::readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu)
{
  UINT16 dstype = 0;
  return pdu.Read(*rsp_p) != false &&
    rsp_p->getSafeUS(&dstype, TAG_DATA_SET_TYPE) &&
    rsp_p->getSafeUS(stat_p, TAG_STATUS) &&
    (dstype != NO_DATA_SET ? pdu.Read(*data_p) != false : true);
}
