#ifndef __NDIMSE_H__
#define __NDIMSE_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "DataSet.h"

class PDU_Service;

class NEventReportRQ : public DataSet  
{
public:
	NEventReportRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 eventTypID, UINT16 dsType);
};

class NEventReportRSP : public DataSet
{
public:
	NEventReportRSP(UINT16 msgID, const std::string& classUID, UINT16 stat,
		UINT16 eventTypID, UINT16 dsType);
};

class NGetRQ : public DataSet 
{
public:
	NGetRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		const TagVector& attrList);
};

class NGetRSP : public DataSet  
{
public:
	NGetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType);
};

class NSetRQ : public DataSet  
{
public:
	NSetRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID);
};

class NSetRSP : public DataSet  
{
public:
	NSetRSP(UINT16 msgID, const std::string& classUID, UINT16 stat, UINT16 dsType);
};

class NActionRQ : public DataSet  
{
public:
	NActionRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 actionTypID, UINT16 dsType);
};

class NActionRSP : public DataSet  
{
public:
	NActionRSP(UINT16 msgID, const std::string& classUID, UINT16 stat,
		UINT16 actionTypID, UINT16 dsType);
};

class NCreateRQ : public DataSet  
{
public:
	NCreateRQ(UINT16 msgID, const std::string& classUID, UINT16 dsType);
	NCreateRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 dsType);
};

class NCreateRSP : public DataSet  
{
public:
	NCreateRSP(UINT16 msgID, const std::string& classUID, const std::string& instUID,
		UINT16 stat, UINT16 dsType);
};

class NDeleteRQ : public DataSet  
{
public:
	NDeleteRQ(UINT16 msgID, const std::string& classUID, const std::string& instUID);
};

class NDeleteRSP : public DataSet  
{
public:
	NDeleteRSP(UINT16 msgID, const std::string& classUID, UINT16 stat);
};

class NHandler
{
public:
	virtual UINT16 handle(int sock, DataSet* rspData_p, std::string& instUID,
		const DataSet& rqCmd, const DataSet& rqData) = 0;
};

class NCreateSCP
{
	NHandler* m_pHandler;
public:
	NCreateSCP(NHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class NSetSCP
{
	NHandler* m_pHandler;
public:
	NSetSCP(NHandler* pHandler = 0) : m_pHandler(pHandler) {}
	bool handle(PDU_Service& pdu, const DataSet& rqCmd, const std::string& classUID);
};

class NCreateSCU  
{
	const std::string m_classUID;
public:
	NCreateSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu, const std::string& instUID, const DataSet& data);
	bool writeRQ(PDU_Service& pdu, const std::string& instUID);
	bool writeRQ(PDU_Service& pdu, const DataSet& data);
	bool writeRQ(PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu);
};

class NSetSCU  
{
	const std::string m_classUID;
public:
	NSetSCU(const std::string& classUID);
	bool writeRQ(PDU_Service& pdu, const std::string& instUID, const DataSet& data);
	bool readRSP(UINT16* stat_p, DataSet* data_p, PDU_Service& pdu);
	bool readRSP(UINT16* stat_p, DataSet* rsp_p, DataSet* data_p, PDU_Service& pdu);
};

#endif //__NDIMSE_H__