#ifndef PARSER_H_INCLUDE_GUARD
#define PARSER_H_INCLUDE_GUARD


/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "DataSet.h"

class Parser
{
public:
	struct Scheme
	{
		UINT32 truncChar;		// default: 64
		UINT32 truncByte;		// default: 32
		UINT32 truncWord;		// default: 16
		const char* fOB;		// default: "%hx %n"
		const char* fOW;		// default: "%hx %n"
		const char* fUS;		// default: "%hx %n"
		const char* fUL;		// default: "%x %n"
		const char* fSS;		// default: "%hd %n"
		const char* fSL;		// default: "%d %n"
		const char* fFL;		// default: "%g %n"
		const char* fFD;		// default: "%lg %n"
		const char* fTag;		// default: "(%hx,%hx)%n"
		const char* fLen;		// default: ",%d%n", 0 -> no given len
		const char* fVR;		// default: ",%c%c%n", 0 -> no given VR
		const char* prefixSQ;	// default: "<"
		const char* prefixVal;	// default: "\t"
//		const char* endOfElem;	// default: "\n"
	};
	static const Scheme defaultScheme;
	// CREATORS
	Parser(std::istream& is, const Scheme& scheme = defaultScheme);

	// MANIPULATORS

	// READERS
	bool read(DataSet::value_type& elem);
	UINT read(DataSet& ds, UINT nMax = UINT(-1));
private:
	char m_buffer[1024];
	bool read(const char*& pBuffer, Tag& tag);

	std::istream& m_is;
	const Scheme& m_scheme;
	std::vector<Sequence*> m_seqStack;
};
#endif //PARSER_H_INCLUDE_GUARD