/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/
/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 * Modifications signed with [gz]
 ***************************************************************************/

// main part taken from pdu.cxx of UCDMC source [gz]

#include "base.hpp"

#include "Pdu.h"

#include "Encoder.h"
#include "Decoder.h"
#include "Impl.h"
#include "MsgLogger.h"

PDU_Service	::	PDU_Service()
{
	m_lastPdu = NO_PDU;
	m_pAcceptRemoteApTitles = 0;
	m_pAcceptLocalApTitles = 0;
	m_pAcceptApplicationContexts = 0;
	m_pAcceptAbstractSyntaxs = 0;
	// PDU Level services are always big endian architecture
	Buffer :: SetIncomingEndian(BIG_ENDIAN);
	Buffer :: SetOutgoingEndian(BIG_ENDIAN);
}

PDU_Service	::	~PDU_Service()
{
	if(Link.Connected)
		Close();
	ClearAbstractSyntaxs();
	while ( AcceptedPresentationContexts.GetSize())
		AcceptedPresentationContexts.RemoveAt ( 0 );
}

static std::string TrimRight(const char* pData)
{
	int len = strlen(pData);
	while (len > 0 && pData[len-1] == ' ')
	{
		--len;
	}
	return std::string(pData, len);
}

bool	PDU_Service	::	InterogateAAssociateRQ()
	{
	UINT	Index, IndexTrn,ValidPresContexts;
	bool	Good;
	TransferSyntax				TrnSyntax;
	PresentationContext			PresContext;
	PresentationContextAccept	PresContextAccept;
	UserInformation				UserInfo;
	MaximumSubLength			MaxSubLength;
	Array<PresentationContext>	PCArray ( false );


	if(!ShouldIAcceptRemoteApTitle(
		TrimRight((const char*)AAssociateRQ :: CallingApTitle)))
		{
		AAssociateRJ :: Reason = 3;	// Calling party not rec.
		AAssociateRJ :: Source = 1;	// PDU
		AAssociateRJ :: Result = 1;	// Permanent
		AAssociateRJ :: Write ( *this );
		return ( false );
		}
	if(!ShouldIAcceptLocalApTitle(
		TrimRight((const char*)AAssociateRQ :: CalledApTitle)))
		{
		AAssociateRJ :: Reason = 7;	// Called Party Not Rec.
		AAssociateRJ :: Source = 1;	// PDU
		AAssociateRJ :: Result = 1;	// Permanent
		AAssociateRJ :: Write ( *this );
		return ( false );
		}
	if(!ShouldIAcceptApplicationContext(
		AAssociateRQ :: AppContext.ApplicationContextName))
		{
		AAssociateRJ :: Reason = 2;	// AppContext Not-Supported
		AAssociateRJ :: Source = 1;	// PDU
		AAssociateRJ :: Result = 1;	// Permanent
		AAssociateRJ :: Write ( *this );
		return ( false );
		}
	// Transfer the information over to the A-ASSOCIATE-AC Class
	AAssociateAC :: SetCalledApTitle(AAssociateRQ :: CalledApTitle);
	AAssociateAC :: SetCallingApTitle(AAssociateRQ :: CallingApTitle);
	AAssociateAC :: AppContext = AAssociateRQ :: AppContext;
	AAssociateAC :: UserInfo = AAssociateRQ :: UserInfo;

	// Ok, now, we need to find an acceptable presentation context
	Index = 0;
	ValidPresContexts = 0;
	PCArray = (AAssociateRQ :: PresContexts);

	while ( AcceptedPresentationContexts.GetSize())
		AcceptedPresentationContexts.RemoveAt ( 0 );
	while ( Index < PCArray.GetSize())
		{
		PresContext = PCArray.Get ( Index );
		PresContext.TrnSyntax.ClearType = false;
		IndexTrn = 0;
		Good = false;
		PresContextAccept.PresentationContextID =
			PresContext.PresentationContextID;
		while ( IndexTrn < PresContext.TrnSyntax.GetSize())
			{
			TrnSyntax = PresContext.TrnSyntax.Get(IndexTrn);
			if(CanYouHandleTransferSyntax(TrnSyntax))
				{
				PresContextAccept.TrnSyntax.TransferSyntaxName =
					TrnSyntax.TransferSyntaxName;
				PresContextAccept.Result = 0;
				AcceptedPresentationContexts.Add ( PresContextAccept );
				Good = true;
				++ValidPresContexts;
				break;
				}
			++IndexTrn;
			}
		if(!ShouldIAcceptAbstractSyntax(
			PresContext.AbsSyntax.AbstractSyntaxName))
			PresContextAccept.Result = 3;
		AAssociateAC :: AddPresentationContextAccept(PresContextAccept);
		++Index;
		}
	if(!ValidPresContexts)
		{
		AAssociateAC :: Write ( *this );
		return ( false );
		}
	MaxSubLength.Set(16384);	// we can do all DICOM can handle
	GetImplementationClass(UserInfo.ImpClass);
	GetImplementationVersion(UserInfo.ImpVersion);
	UserInfo.SetMax(MaxSubLength);

	AAssociateAC :: SetUserInformation ( UserInfo );
	AAssociateAC :: Write ( *this );

	return ( true );	// yes, the communication should work out
	}


bool	PDU_Service	::	InterogateAAssociateAC()
	{
	UINT	Index, ValidPresContexts;
	PresentationContextAccept	PresContextAccept;
	Array<PresentationContextAccept>	PCArray ( false );

	Index = 0;
	ValidPresContexts = 0;
	PCArray = (AAssociateAC :: PresContextAccepts);

	while ( AcceptedPresentationContexts.GetSize())
		AcceptedPresentationContexts.RemoveAt ( 0 );

	while ( Index < PCArray.GetSize())
		{
		PresContextAccept = PCArray.Get ( Index );
		if(!PresContextAccept.Result)
			{
			AcceptedPresentationContexts.Add ( PresContextAccept );
			}
		++Index;
		}
	if(!AcceptedPresentationContexts.GetSize())
		return ( false );	// no, this association should be terminated
	return ( true );	// yes, the communication should work out
	}


// Checks the Known Class list for the UID and further verifies that
// it's a big/little Endian.  If it is, then it sets the Transfer type
// in the TransferSyntax for further reference
bool	PDU_Service	::	CanYouHandleTransferSyntax(TransferSyntax &TrnSyntax)
	{
	/*
	KnownClass	KC;

	if(KnownUID(TrnSyntax.TransferSyntaxName, KC))
		return ( false );
	
	if(KC.Type != "transfer")
		return ( false );

	if(KC.Comment != "LittleEndian")
		{
		if(KC.Command != "BigEndian")
			return ( false );
		else
			TrnSyntax.SetType ( BIG_ENDIAN );
		}
	else
			TrnSyntax.SetType ( LITTLE_ENDIAN );
	*/
	if(!strcmp((char*) TrnSyntax.TransferSyntaxName.GetBuffer(1), "1.2.840.10008.1.2"))
		{
		TrnSyntax.SetType ( LITTLE_ENDIAN );
		return ( true );
		}
	return ( false );
	}

bool	PDU_Service	::	Listen(const std::string& port)
	{
	PDataTF :: Length = 0;
	
	if (Link.Connected)
		Link.Close();

	if(!Link.Listen((char *) port.c_str()))
		return ( false );

	if(!Link.Accept())
		return ( false );

//	fprintf(stderr, "Waiting for AAssociateRQ\n");
	AAssociateRQ :: Read( *this );
	m_lastPdu = AA_RQ;
//	fprintf(stderr, "Interogating AAssociateRQ\n");
	if(!InterogateAAssociateRQ())
		{
//		fprintf(stderr, "failed to connect\n");
		Link.Close();
		return ( false );
		}
//	fprintf(stderr, "connected, but not disconnecting (missing P-DATA service)\n");
//	AAbortRQ :: Write ( *this );

//	return ( false );
	return ( true );
	}

int	PDU_Service	::	Multiplex(const std::string&	port)
	{
	int	TempSocket;
	
	PDataTF :: Length = 0;

	if(!Link.Listen((char*)port.c_str()))
		return(-1);

	TempSocket = Link.ListenSocketfd;
	Link.ListenSocketfd = 0;
	Link.Listened = false;
	return (Listen(TempSocket));
	}

int	PDU_Service	::	Listen(int	sock)
	{
	int	TempSocket;

	if(Link.Connected)
		Link.Close();
	Link.ListenSocketfd = sock;
	Link.Listened =  true;

	while (!Link.Connected)
		{	
		if(!Link.Accept())
			return ( false );
	
		AAssociateRQ :: Read( *this );
		m_lastPdu = AA_RQ;
		if(!InterogateAAssociateRQ())
			{
			TempSocket = Link.ListenSocketfd;
			Link.Listened = false;
			Link.Close();
			Link.Listened = true;
			Link.ListenSocketfd = TempSocket;
			}
		}
	TempSocket = Link.ListenSocketfd;
	Link.ListenSocketfd = 0;
	Link.Listened = 0;
	return ( TempSocket );	
	}

bool	PDU_Service	::	Multiplex(int	Socketfd)
	{
	if(Link.Connected)
		Link.Close();
	Link.Connected = true;
	Link.Socketfd = Socketfd;
	AAssociateRQ :: Read( *this );
	m_lastPdu = AA_RQ;
	if(!InterogateAAssociateRQ())
		{
		Link.Close();
		return(false);
		}
	return (true);
	}	

bool	PDU_Service	::	Close()
	{
	PDataTF :: Length = 0;
	if ( Link.Connected )
		{
		AReleaseRQ :: Write ( *this );
		AReleaseRP :: Read ( *this );
		m_lastPdu = AR_RP;
		}
	Link.Close();
	return ( true );
	}

// Support routine to get Presentation Context ID from UID

BYTE	PDU_Service	::	GetAcceptedPCIDForTransferSyntax(const UID& uid)
	{
	UINT	Index;
	PresentationContextAccept	PCA;

	Index = 0;
	while (Index < AcceptedPresentationContexts.GetSize() )
		{
		PCA = AcceptedPresentationContexts.Get ( Index );
		if(PCA.TrnSyntax.TransferSyntaxName.str() == uid)
			return ( PCA.PresentationContextID );	
		++Index;
		}
	return ( 0 );
	}

BYTE	PDU_Service	::	GetPresentationContextID(const UID& uid)
	{
	UINT	Index;
	Array <PresentationContext>	PCArray ( false );
	PresentationContext		PresContext;

	PCArray = (AAssociateRQ :: PresContexts);

	Index = 0;
	while ( Index < PCArray.GetSize())
		{
		PresContext = PCArray.Get ( Index );
		PresContext.TrnSyntax.ClearType = false;
		if(PresContext.AbsSyntax.AbstractSyntaxName.str() == uid)
			return ( PresContext.PresentationContextID );
		++Index;
		}

	return ( 0 );
}

// Get the PCID for a given AbsUID and TrnUID
BYTE	PDU_Service	::	GetPresentationContextID(const std::string& AbsUID, const std::string& TrnUID)
{
	UINT	Index;
	Array <PresentationContext>	PCArray ( false );
	PresentationContext			PresContext;

	PCArray = (AAssociateRQ :: PresContexts);

	Index = 0;
	
	while ( Index < PCArray.GetSize())
	{
		PresContext = PCArray.Get ( Index );
		PresContext.TrnSyntax.ClearType = false;
		if(PresContext.AbsSyntax.AbstractSyntaxName.str() == AbsUID)
		{
			UINT	Index;
			PresentationContextAccept	PCA;
			Index = 0;
			while (Index < AcceptedPresentationContexts.GetSize() )
			{
				PCA = AcceptedPresentationContexts.Get ( Index );
				if(PCA.TrnSyntax.TransferSyntaxName.str() == TrnUID &&
					PCA.PresentationContextID == PresContext.PresentationContextID)
				{
					return ( PCA.PresentationContextID );
				}
				++Index;
			}
			return ( 0 );
		}
		++Index;
	}

	return ( 0 );
}

bool	PDU_Service	::	GetTransferSyntaxUID(BYTE	PCID, std::string& uid)
{
	UINT	Index;
	PresentationContextAccept	PCA;

	Index = 0;
	while (Index < AcceptedPresentationContexts.GetSize() )
	{
		PCA = AcceptedPresentationContexts.Get ( Index );
		if(PCA.PresentationContextID == PCID)
			{
			uid = PCA.TrnSyntax.TransferSyntaxName;
			return ( true );
			}
		++Index;
	}
	return ( false );
}


bool	PDU_Service::ClearAbstractSyntaxs ()
{
	while (ProposedAbstractSyntaxs.GetSize())
		ProposedAbstractSyntaxs.RemoveAt ( 0 );
	return ( true );
}

bool	PDU_Service	::	AddAbstractSyntax(const UID& uid)
{
	UINT			Index;
	AbstractSyntax	AbsSyntax;

	Index = 0;
	while ( Index < ProposedAbstractSyntaxs.GetSize() )
	{
		if ( uid == ProposedAbstractSyntaxs.Get ( Index ).
			AbstractSyntaxName.str())
			return ( true );	// already in array
		++Index;
	}
	AbsSyntax.Set(uid.c_str());
	ProposedAbstractSyntaxs.Add ( AbsSyntax );
	return ( true );
}

bool	PDU_Service	::	AddAbstractSyntaxAlias (
	const UID& uidSource,
	const UID& uidAlias)
{
	UINT	Index;
	Array <PresentationContext>	PCArray ( false );
	PresentationContext			PresContext;

	// First check to see if it's already aliased
	if ( GetPresentationContextID ( uidAlias ) )
		return ( true );

	PCArray = (AAssociateRQ :: PresContexts);

	Index = 0;
	while ( Index < PCArray.GetSize())
	{
		PresContext = PCArray.Get ( Index );
		PresContext.TrnSyntax.ClearType = false;
		if(PresContext.AbsSyntax.AbstractSyntaxName.str() == uidSource)
		{
			PresentationContext vPresContext = PresContext;
		
			vPresContext.TrnSyntax.ClearType = false;
				
			PresContext.TrnSyntax.Clear();
			UINT	SubIndex = 0;
			while (SubIndex < vPresContext.TrnSyntax.GetSize() )
			{
				PresContext.TrnSyntax.Add(vPresContext.TrnSyntax.Get(SubIndex));
				++SubIndex;
			}
			PresContext.TrnSyntax.ClearType = true;
			PresContext.AbsSyntax.AbstractSyntaxName = UID_(uidAlias.c_str());
			AAssociateRQ :: AddPresentationContext(PresContext);
			PresContext.TrnSyntax.ClearType = false;
			return ( true );
		}
		++Index;
	}

	// The source was not proposed

	return ( false );
}

bool	PDU_Service	::	SetApplicationContext(const UID& uid)
{
	AAssociateRQ :: SetApplicationContext(UID_(uid.c_str()));
	return ( true );
}

bool	PDU_Service	::	SetLocalAddress(const std::string& address)
{
	AAssociateRQ :: SetCallingApTitle((BYTE*)address.c_str());
	return ( true );
}

bool	PDU_Service	::	SetRemoteAddress(const std::string& address)
{
	AAssociateRQ :: SetCalledApTitle((BYTE*)address.c_str());
	return ( true );
}

bool	PDU_Service	::	AddTransferSyntaxs(PresentationContext &PresContext)
{
	UID_	uid;
	TransferSyntax	TrnSyntax;

	uid.Set(UID_IMPL_VR_LE_TRANSFER_SYNTAX);
	TrnSyntax.Set(uid);
	PresContext.AddTransferSyntax(TrnSyntax);
	return ( true );
}

bool	PDU_Service	::	GetImplementationClass(ImplementationClass &ImpClass)
{
	ImpClass.Set((BYTE*) IMPL_CLASS_UID);
	return(true);
}

bool	PDU_Service	::	GetImplementationVersion(ImplementationVersion &ImpVersion)
{
	ImpVersion.Set((BYTE*) IMPL_VERSION_NAME);
	return ( true );
}

bool	PDU_Service	::	Connect (const std::string& ip, const std::string& port )
{
	UINT					Index;
	PresentationContext		PresContext;
	UserInformation			UserInfo;
	MaximumSubLength		MaxSubLength;
	BYTE					ItemType;

	PDataTF :: Length = 0;

	if (!ProposedAbstractSyntaxs.GetSize () )
		return ( false );		// must proposed atleast one SOP class
	AAssociateRQ :: ClearPresentationContexts();
	Index = 0;
	//AddTransferSyntaxs(PresContext);
	while ( Index < ProposedAbstractSyntaxs.GetSize())
		{
		PresContext.PresentationContextID = uniq8odd();	/**GZ**/
		PresContext.TrnSyntax.Clear();
		AddTransferSyntaxs(PresContext);
		PresContext.SetAbstractSyntax ( ProposedAbstractSyntaxs.Get ( Index ) );
		AAssociateRQ :: AddPresentationContext ( PresContext );
		++Index;
		}
	PresContext.TrnSyntax.ClearType = false;
	MaxSubLength.Set(16384);	// we can do all DICOM can handle
	GetImplementationClass(UserInfo.ImpClass);
	GetImplementationVersion(UserInfo.ImpVersion);
	UserInfo.SetMax(MaxSubLength);

	AAssociateRQ :: SetUserInformation ( UserInfo );

	// ok, every thing proposed, now try and contact the remote end
	if ( ! Link.Open ( (char*) ip.c_str(), (char*)port.c_str() ) )
		return ( false );	// Socket level Error
	AAssociateRQ :: Size();
	AAssociateRQ :: Write ( *this );
	
	(*this) >> ItemType;
//	fprintf(stderr, "Got Response Type: %d\n", ItemType);
	switch ( ItemType )
		{
		case	0x02:
			AAssociateAC :: ReadDynamic ( * this );
			m_lastPdu = AA_AC;
			if(!InterogateAAssociateAC())
				{
				//printf("Failed to InterogateAAssociateAC\n");
				Link.Close();
				return ( false );
				}
			return ( true );	// this is what we want to happen
		case	0x03:
			AAssociateRJ :: ReadDynamic ( * this );
			m_lastPdu = AA_RJ;
			Link.Close();
			return ( false );
		default:
			// corrupt transmission
			m_lastPdu = NO_PDU;
			Link.Close();
			return ( false );
		}
	
	return ( false );
	}

	
bool PDU_Service::Read(DataSet& ds)
{
	if (!Link.Connected)
		return ( false );
	PDataTF :: MsgStatus = 0;
	if((PDataTF :: Length)!=0)	// for bad GCC v2.6.0
	{
		PDataTF :: ReadDynamic( *this );
		m_lastPdu = PD_TF;

		if (PDataTF :: MsgStatus > 0)
			return ParseRawVRIntoDataSet(ds);
	}
	
	while ( true )
	{
		BYTE		ItemType;
		(*this) >> ItemType;
		switch ( ItemType )
		{
		case	0x01:	// A-ASSOCIATE-RQ Should not get this here
			AAssociateRQ :: Read(*this);
			m_lastPdu = AA_RQ;
			AAbortRQ :: Write ( *this );
			Link.Close();
			return ( false );
		case	0x02:	// A-ASSOCIATE-AC Should not get this here
			AAssociateAC :: Read(*this);
			m_lastPdu = AA_AC;
			AAbortRQ :: Write ( *this );
			Link.Close();
			return ( false );
		case	0x03:	// A-ASSOCIATE-RJ Should not get this here
			m_lastPdu = AA_RJ;
			AAbortRQ :: Write ( *this );
			Link.Close();
			return ( false );
		case	0x04:	// P-DATA-TF
			PDataTF :: ReadDynamic( *this );
			m_lastPdu = PD_TF;

			if (PDataTF :: MsgStatus > 0)
				return ParseRawVRIntoDataSet(ds);
			break;
		case	0x05:	// A-RELEASE-RQ
			AReleaseRQ :: ReadDynamic(*this);
			m_lastPdu = AR_RQ;
			// also drop
			AReleaseRP :: Write(*this);
			Link.Close(); // inserted by [gz] 99/4/23
			return ( false );
		case	0x06:	// A-RELEASE-RP
			m_lastPdu = AR_RP;
			Link.Close();
			return ( false );
		case	0x07:	// A-ABORT-RQ
			m_lastPdu = AAB_RQ;
			Link.Close();
			return ( false );
		default:
			// Protocol Error
      switch (Link.GetLastError())
      {
      case Socket::ERR_TIMEOUT:
        m_lastPdu = TIMEOUT;
        break;
      case Socket::ERR_TCPCLOSE:
        m_lastPdu = TCPCLOSE;
        break;
      default:
        m_lastPdu = NO_PDU;
      }
			AAbortRQ :: Write ( *this );
			Link.Close();
			return ( false );
		}
	}
	return ( false );
}

bool PDU_Service::Write(MsgHeaderEnum msgHead, const DataSet& ds,
				const std::string& as, TS ts)
{
  UID_ absUID((BYTE*)as.c_str());
  UID_ tsUID((BYTE*)ts.getUID());
  PDataTF :: MsgHeader = msgHead;
  PDataTF :: PresentationContextID = GetPresentationContextID(absUID);
  bool ret = true;
  if(!PDataTF :: PresentationContextID)
    ret = false;
  else
  if (!GetPresentationContextID(absUID, tsUID))
    ret = false;
  else
  {
    PDataTF::VRBuffer.SetOutgoingEndian(ts.isBigEndian() ? BIG_ENDIAN : LITTLE_ENDIAN);
    Encoder(PDataTF::VRBuffer, ts).write(ds);

    ret = PDataTF :: Write (*this);
  }
  if (ret)
  {
    MsgLogger* msgLog_p = MsgLogger::get();
    if (msgLog_p)
        msgLog_p->logTransmittedData(Link.Socketfd, ds);
  } else
  {
    ErrLogger* errLog_p = ErrLogger::get();
    if (errLog_p)
          errLog_p->logWriteDataFailed(Link.Socketfd);
  }

  return ret;
}
/*
bool PDU_Service::WriteCommand(const DataSet& ds)
{
	DataSet::const_iterator iter = ds.find(TAG_AFF_SOP_CLASS_UID);
	return iter != ds.end() ? WriteCommand(ds, iter->second->str(VR_UI)) : false;
}
*/
bool PDU_Service::ParseRawVRIntoDataSet(DataSet& ds)
{
	if (!DoParseRawVRIntoDataSet(ds))
	{
		ErrLogger* errLog_p = ErrLogger::get();
		if (errLog_p)
			errLog_p->logReadDataFailed(Link.Socketfd);
		return false;
	}
	MsgLogger* msgLog_p = MsgLogger::get();
	if (msgLog_p)
		msgLog_p->logReceivedData(Link.Socketfd, ds);
	return true;
}

bool PDU_Service::DoParseRawVRIntoDataSet(DataSet& ds)
{
	SizeCap = 0;
	std::string uid;
	if(!GetTransferSyntaxUID(PDataTF :: PresentationContextID,uid))
		return ( false );	// Error no UID associated with PCID

	TS ts(uid);
        if (ts == TS::NOT_SUPPORTED)
		return ( false );	// Error Transfersyntax not supported

	PDataTF::VRBuffer.SetIncomingEndian(ts.isBigEndian() ? BIG_ENDIAN : LITTLE_ENDIAN);
	Decoder decoder(PDataTF::VRBuffer, ts);
	decoder.read(ds);
	return decoder.good();
}

bool PDU_Service::ShouldIAcceptRemoteApTitle(const std::string& aet)
{
	bool ret = ShouldIAccept(m_pAcceptRemoteApTitles, aet);
	MsgLogger* msgLog_p = MsgLogger::get();
	if (msgLog_p)
		msgLog_p->logAcceptRemoteApTitle(Link.Socketfd, aet, ret != 0);
  return ret;
}

bool PDU_Service::ShouldIAcceptLocalApTitle(const std::string& aet)
{
	bool ret = ShouldIAccept(m_pAcceptLocalApTitles, aet);
	MsgLogger* msgLog_p = MsgLogger::get();
	if (msgLog_p)
		msgLog_p->logAcceptLocalApTitle(Link.Socketfd, aet, ret != 0);
  return ret;
}

bool PDU_Service::ShouldIAcceptApplicationContext(const std::string& uid)
{
	bool ret = ShouldIAccept(m_pAcceptApplicationContexts, uid);
	MsgLogger* msgLog_p = MsgLogger::get();
	if (msgLog_p)
		msgLog_p->logAcceptApplicationContext(Link.Socketfd, uid, ret != 0);
  return ret;
}

bool PDU_Service::ShouldIAcceptAbstractSyntax(const std::string& uid)
{
	bool ret = ShouldIAccept(m_pAcceptAbstractSyntaxs, uid);
	MsgLogger* msgLog_p = MsgLogger::get();
	if (msgLog_p)
		msgLog_p->logAcceptAbstractSyntax(Link.Socketfd, uid, ret != 0);
  return ret;
}
