#ifndef __PDU_H__
#define __PDU_H__
/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/
/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 * Modifications signed with [gz]
 ***************************************************************************/

// main part from pdu.hpp of UCDMC source [gz]

#include "socket.hpp"	// Socket Transport Layer
#include "aarq.hpp"		// A-ASSOCIATE-RQ PDU
#include "aaac.hpp"		// A-ASSOCIATE-AC PDU
#include "aarj.hpp"		// A-ASSOCIATE-RJ/A-RELEASE-RQ/A-RELEASE-RP/A-ABORT-RQ
#include "pdata.hpp"	// P-DATA-TF
#include "TS.h"

class DataSet;

class	PDU_Service	:
	public	Buffer,			// Transport Buffer
	public	AAssociateRQ,	// Associate Request
	public	AAssociateAC,	// Associate Accept
	public	AAssociateRJ,	// Associate Reject
	public	AReleaseRQ,		// Associate Release
	public	AReleaseRP,		// Associate Release Response
	public	AAbortRQ,		// Associate Abort Request
	public	PDataTF			// P-DATA Service
	{
	protected:
		Array<AbstractSyntax>				ProposedAbstractSyntaxs;
		Array<PresentationContextAccept>	AcceptedPresentationContexts;
		UINT32								SizeCap;
		int m_lastPdu;
		const std::set<std::string>* m_pAcceptRemoteApTitles;
		const std::set<std::string>* m_pAcceptLocalApTitles;
		const std::set<std::string>* m_pAcceptApplicationContexts;
		const std::set<std::string>* m_pAcceptAbstractSyntaxs;
	protected:
		bool	InterogateAAssociateRQ();
		bool	InterogateAAssociateAC();
		bool	AddTransferSyntaxs(PresentationContext &);
		bool	GetImplementationClass(ImplementationClass &);
		bool	GetImplementationVersion(ImplementationVersion &);

		// Support Routines for Transfer Syntax Encoding/Decoding
	public:	// The following routine is usefull to check for a valid
		// Accepted Presentation Context UID
			BYTE	GetAcceptedPCIDForTransferSyntax(const std::string& TrnUID);
			bool	GetTransferSyntaxUID(BYTE, std::string& TrnUID);
			BYTE	GetPresentationContextID(const std::string& AbsUID);
			BYTE	GetPresentationContextID(const std::string& AbsUID,
											const std::string& TrnUID);
	public:
		void SetAcceptRemoteApTitles(const std::set<std::string>* pSet) {
			m_pAcceptRemoteApTitles = pSet; }
		void SetAcceptLocalApTitles(const std::set<std::string>* pSet) {
			m_pAcceptLocalApTitles = pSet; }
		void SetAcceptApplicationContexts(const std::set<std::string>* pSet) {
			m_pAcceptApplicationContexts = pSet; }
		void SetAcceptAbstractSyntaxs(const std::set<std::string>* pSet) {
			m_pAcceptAbstractSyntaxs = pSet; }
		enum ENUM_PDU { AA_RQ, AA_AC, AA_RJ, AR_RQ, AR_RP, AAB_RQ, PD_TF, NO_PDU, TIMEOUT, TCPCLOSE };
		int lastPdu() const { return m_lastPdu; }
		Socket	Link;
                int GetSocketfd() const { return Link.Socketfd; }

		// Wrap data over from the Associate/PData Classes over to the buffer
				bool	Read(BYTE	*Data, UINT	Count)
			{	return ( Buffer :: Read ( Data, Count ) ); };
				bool	Write(BYTE	*Data, UINT	Count)
			{	return ( Buffer :: Write ( Data, Count ) ); };
				bool	Kill(UINT	Count)
			{	return ( Buffer :: Kill ( Count ) ); };
				void	Flush()
			{	 Buffer :: Flush () ; };
		// Wrap data over from the buffer over to the socket
				INT	ReadBinary(BYTE *Data, UINT Count)
			{	return ( Link.ReadBinary(Data, Count) ); };
				bool	SendBinary(BYTE *Data, UINT Count)
			{	return ( Link.SendBinary(Data, Count) ); };

		bool	ClearAbstractSyntaxs();
		bool	AddAbstractSyntax(const UID& str);	
		bool	AddAbstractSyntaxAlias(const UID& uidSource, const UID& uidAlias);
		bool	SetApplicationContext(const UID& uid);
		bool	SetLocalAddress(const std::string& aet);	
		bool	SetRemoteAddress(const std::string& aet);	
		//////////////////////////////////////////////////
		bool	Connect(const std::string& ip, const std::string& port);	
		
		bool	Close();

		bool	Listen(const std::string& port);					
		bool	Multiplex(int);

		int		Multiplex(const std::string&);
		int		Listen(int);
		bool	SetTimeOut(int TimeOut)
			{ return ( Link.SetTimeOut(TimeOut) ); };
				PDU_Service();
		virtual	~PDU_Service();

		bool Read(DataSet& ds);
//		bool WriteCommand(const DataSet& ds);
		bool WriteCommand(const DataSet& ds, const std::string& as) {
			return Write(COMMAND, ds, as, TS::IMPL_VR_LE); } 

		bool WriteDataSet(const DataSet& ds, const std::string& as, TS ts = TS::IMPL_VR_LE) {
			return Write(DATASET, ds, as, ts); }
		bool IsOpen() const { return Link.Connected; }
	protected:
		enum MsgHeaderEnum { DATASET = 0x00, COMMAND = 0x01 };
		bool ParseRawVRIntoDataSet(DataSet& ds);
		bool DoParseRawVRIntoDataSet(DataSet& ds);
		bool Write(MsgHeaderEnum msgHead, const DataSet& ds,
				const std::string& as, TS ts);

		bool	CanYouHandleTransferSyntax(TransferSyntax &);

		/*
			this is slightly bizarre - an empty set means accept all AE titles.
			This should be documented.
		*/
		static bool ShouldIAccept(const std::set<std::string>* pSet, const std::string& str) {
			return pSet == 0 || pSet->find(str) != pSet->end(); }
		bool ShouldIAcceptRemoteApTitle(const std::string& aet);
    bool ShouldIAcceptLocalApTitle(const std::string& aet);
		bool ShouldIAcceptApplicationContext(const std::string& uid);
		bool ShouldIAcceptAbstractSyntax(const std::string& uid);
};


#endif //__PDU_H__