#ifndef __SERVER_H__
#define __SERVER_H__
/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "Dispatcher.h"
#include <set>
class Server
{
	Dispatcher m_dispatcher;
	int m_timeOut;
	/*
		I really don't think that the following members should be pointers - it
		means that the user of the library will always be responsible for resource
		management of these objects, which should be the libraries job.  morgan@sten.sunnybrook.utoronto.ca
	*/
	const std::set<std::string>* m_pAcceptRemoteApTitles;
	const std::set<std::string>* m_pAcceptLocalApTitles;
	const std::set<std::string>* m_pAcceptApplicationContexts;
	std::set<std::string> m_acceptAbstractSyntaxs;
public:
	bool bStopping;
	Server();
	void setAcceptRemoteApTitles(const std::set<std::string>* pSet) {
		m_pAcceptRemoteApTitles = pSet; }
	void setAcceptLocalApTitles(const std::set<std::string>* pSet) {
		m_pAcceptLocalApTitles = pSet; }
	void setAcceptApplicationContexts(const std::set<std::string>* pSet) {
		m_pAcceptApplicationContexts = pSet; }
	bool addCStoreHandler(const std::string& classUID, CHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addCStoreHandler(classUID, handler); }
	bool addCFindHandler(const std::string& classUID, CHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addCFindHandler(classUID, handler); }
	bool addCGetHandler(const std::string& classUID, CHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addCGetHandler(classUID, handler); }
	bool addCMoveHandler(const std::string& classUID, CHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addCMoveHandler(classUID, handler); }
	bool addNCreateHandler(const std::string& classUID, NHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addNCreateHandler(classUID, handler); }
	bool addNSetHandler(const std::string& classUID, NHandler* handler = 0) {
		m_acceptAbstractSyntaxs.insert(classUID);
		return m_dispatcher.addNSetHandler(classUID, handler); }

	enum ErrorCode { ERR_LISTEN = -1, ERR_ACCEPT = -2, ERR_THREAD = -3, ERR_NO = 0 };
	ErrorCode run(const std::string& port);
	ErrorCode runMultiThread(const std::string& port, bool abortIfErrThread = true);
	void setTimeOut(int n) { m_timeOut = n; }
private:
        virtual void onCloseAssoc(int sock, int lastPdu);
	void handle(int sock);
#ifdef WIN32
	friend void __cdecl start_func(void* p);
#else
	friend void* start_func(void* p);
#endif
};

#endif //__SERVER_H__