/****************************************************************************
          Copyright (C) 1998, Karl Franzens University, Graz

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE KARL FRANZENS
          UNIVERSITY DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the Karl Franzens University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 *
 ***************************************************************************/

#include "base.hpp"

#include "WinDumper.h"
#include "DataDict.h"
using std::string;

const WinDumper::Scheme WinDumper::defaultScheme =
{
	'#',		// noPrintCh;		 
	64,			// truncChar;	 
	32,			// truncByte;	 
	16,			// truncWord;	 
	"..",		// truncSign;	 
	"%.2x ",	// fOB;		 
	"%.4x ",	// fOW;		 
	"%.4d ",	// fUS;		 
	"%.8d ",	// fUL;		 
	"%+d ",		// fSS;		 
	"%+d ",		// fSL;		 
	"%g ",		// fFL;		 
	"%g ",		// fFD;		 
	"(%.4x,%.4x)",	// fTag;		 
	",%d",		// fLen;		 
	",%c%c",	// fVR;		 
	",%s",		// fName;		 
	"?????????????",	// unknown;	 
	"<",		// prefixSQ;	 
	"\t",		// prefixVal;	 
	"\r\n",		// endOfElem;	 
	"(null)",	// valSQelem;	 
	VR_OB		// fVR_UNas;		 
};

WinDumper::WinDumper(HWND hEdit, const Scheme& scheme)
:	m_hEdit(hEdit), m_scheme(scheme), m_SQlevel(0)
{
}

void WinDumper::write(const char* s)
{
	int end = ::GetWindowTextLength(m_hEdit);
	::SendMessage(m_hEdit, EM_SETSEL, (WPARAM)end, (LPARAM)end);
	::SendMessage(m_hEdit, EM_REPLACESEL, 0, (LPARAM)s);
}

void WinDumper::write(const DataSet::value_type& elem)
{
	if (!m_scheme.prefixSQ && elem.second->seq())
	{
		write(elem.second->seq());
		return;
	}

	char* ch_p = m_buffer;
	for (int i = m_SQlevel; i--;)
		ch_p += sprintf(ch_p, m_scheme.prefixSQ);
	ch_p += sprintf(ch_p, m_scheme.fTag, getGrTag(elem.first), getElTag(elem.first));
	const Value& val = *elem.second;
	VR vr = VR_UN;
	const char* name = m_scheme.unknown;
	DataDict::instance().lookup(elem.first, &vr, &name);
	if (val.vr() != VR_UN)
		vr = val.vr();

	if (m_scheme.fVR != 0)
	{
		ch_p += sprintf(ch_p, m_scheme.fVR, getCh1(vr), getCh2(vr));
	}
	UINT32 len = val.length();
	if (m_scheme.fLen != 0)
	{
		ch_p += sprintf(ch_p, m_scheme.fLen, len);
	}
	if (m_scheme.fName != 0)
	{
		ch_p += sprintf(ch_p, m_scheme.fName, name);
	}

	ch_p += sprintf(ch_p, m_scheme.prefixVal);
	const Sequence* pSQ = val.seq();
	if (pSQ != 0)
	{
		++m_SQlevel;
		ch_p += sprintf(ch_p, m_scheme.valSQelem);
		ch_p += sprintf(ch_p, m_scheme.endOfElem);
		write(m_buffer);
		write(pSQ);
		--m_SQlevel;
		return;
	}

	bool trunc = false;
	switch (vr != VR_UN ? vr : m_scheme.fVR_UNas)
	{
	case VR_AT:
		{
			const UINT32* pData = (UINT32*)val.data();
			trunc = len/8 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/8; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fTag,
					(UINT16)((UINT16*)pData)[0],
					(UINT16)((UINT16*)pData)[1]);
			}
		}
		break;
	case VR_FL:
		{
			const float* pData = (float*)val.data();
			trunc = len/4 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/4; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fFL, (float)*pData);
			}
		}
		break;
	case VR_FD:
		{
			const double* pData = (double*)val.data();
			trunc = len/8 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/8; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fFD, (double)*pData);
			}
		}
		break;
	case VR_OB:
		{
			const BYTE* pData = (BYTE*)val.data();
			trunc = len > m_scheme.truncByte;
			for(int k = trunc ? m_scheme.truncByte : len; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fOB, *pData);
			}
		}
		break;
	case VR_OW:
		{
			const UINT16* pData = (UINT16*)val.data();
			trunc = len/2 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/2; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fOW, (UINT16)*pData);
			}
		}
		break;
	case VR_SL:
		{
			const INT32* pData = (INT32*)val.data();
			trunc = len/4 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/4; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fSL, (INT32)*pData);
			}
		}
		break;
	case VR_SQ:
		ch_p += sprintf(ch_p, m_scheme.valSQelem);
		break;
	case VR_SS:
		{
			const INT16* pData = (INT16*)val.data();
			trunc = len/2 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/2; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fSS, (INT16)*pData);
			}
		}
		break;
	case VR_UL:
		{
			const UINT32* pData = (UINT32*)val.data();
			trunc = len/4 > m_scheme.truncWord;
			for(int k = trunc ? m_scheme.truncWord : len/4; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fUL, (UINT32)*pData);
			}
		}
		break;
	case VR_US:
		{
			const UINT16* pData = (UINT16*)val.data();
			trunc = len/2 > m_scheme.truncWord;
			for(UINT32 k = trunc ? m_scheme.truncWord : len/2; k != 0; --k, ++pData)
			{
				ch_p += sprintf(ch_p, m_scheme.fUS, (UINT16)*pData);
			}
		}
		break;
	default:
		{
			string temp((const char*)val.data(), min(val.length(), m_scheme.truncChar));
			*ch_p++ = '\"';
			for (const char* pData = temp.c_str(); *pData != '\0'; ++pData)
				*ch_p++ = (isprint(*pData) != 0 ? *pData : m_scheme.noPrintCh);
			if (len > m_scheme.truncChar)
				ch_p += sprintf(ch_p, m_scheme.truncSign);
			*ch_p++ = '\"';
		}
	}
	if (trunc)
		ch_p += sprintf(ch_p, m_scheme.truncSign);
	ch_p += sprintf(ch_p, m_scheme.endOfElem);
	write(m_buffer);
}

void WinDumper::write(const DataSet& ds)
{
	DataSet::const_iterator iter = ds.begin();
	DataSet::const_iterator last = ds.end();
	for (; iter != last; ++iter)
		write(*iter);
}

void WinDumper::write(const Sequence* pSQ)
{
	Sequence::const_iterator iter = pSQ->begin();
	Sequence::const_iterator last = pSQ->end();
	for (; iter != last; ++iter)
		write(**iter);
}
