#ifndef __AARQ_HPP__
#define __AARQ_HPP__
/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/
/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 * Modifications signed with [gz]
 ***************************************************************************/

/***********************************************************************
 * PDU Service Classes:
 *	A-ASSOCIATE-RQ Class.  
 *
 * Base Classes:
 *	UID_
 *	ApplicationContext
 *	AbstractSyntax
 *	TransferSyntax
 *	PresentationContext
 *	MaximumSubLength
 *	ImplementationClass
 *	ImplementationVersion
 *	UserInformation
 *
 *
 *
 *********************************************************************/

class	UID_
	{
	UINT	Length;
	BYTE	uid[65];
	
	public:
			UID_()	{ ZeroMem(uid, 64); Length = 0; };
			UID_(const BYTE *s) { Set ( s ); };				// add const [gz]
			UID_(const char *s) { Set ( (const BYTE*)s ); };	// add const [gz]
			void	Set(const BYTE *s)						// add const [gz]
				{
				if(!s)
					return;
				ZeroMem(uid, 64);
				strcpy((char *) uid, (char *) s);
				Length = strlen((char*) uid);
				};
			void	Set(const UID_	&u) { (*this) = u; };	// add const [gz]
			void	Set(const char *s) { this->Set((BYTE *) s); };	// add const [gz]
			BYTE	*GetBuffer(UINT Min) { return(&uid[0]); };
			void	SetLength(UINT	L)	{ Length = L; while (L < 65) uid[L++] = '\0'; };
			UINT	GetSize() const	{ return ( Length ); };	// add const [gz]
			bool	operator	==	(const UID_ &ud) const	// add const [gz]
				{ 	//if(GetSize()!=ud.GetSize()) return(false);
					if(!strcmp((char*) uid, (char*) ud.uid))	// added by [gz], comment out follow
//					if(!strcmp((char*) GetBuffer(1), (char*) ud.GetBuffer(1)/*, (int) GetSize()*/))
						return(true);
					return(false); };
			bool	operator	!=	(const UID_ &ud) const { return (!((*this)==ud));	// add const [gz]
};
			bool	operator	=	(const UID_ &ud)			// add const [gz]
				{	ByteCopy(uid, ud.uid, 64);				// added by [gz]
//					ByteCopy(uid, ud.GetBuffer(1), 64);		// comment out by [gz]
					SetLength(ud.GetSize());
					
					return(true);
				}
			std::string str() const { return std::string((const char*)uid, Length); }	// added by [gz]
			operator std::string() const { return str(); }	// added by [gz]
	};

class	ApplicationContext
	{
	private:
		BYTE		ItemType;		// 0x10
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			ApplicationContextName;

					ApplicationContext();
					ApplicationContext(const UID_	&);	// add const [gz]
					ApplicationContext(const BYTE *);	// add const [gz]
					~ApplicationContext();
		void		Set(const UID_	&);					// add const [gz]
		void		Set(const BYTE *);					// add const [gz]
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();
	};

class	AbstractSyntax
	{
	private:
		BYTE		ItemType;	// 0x30
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			AbstractSyntaxName;
					AbstractSyntax();
					AbstractSyntax(const BYTE *);	// add const [gz]
					AbstractSyntax(const UID_ &);	// add const [gz]
					~AbstractSyntax();
		void		Set(const UID_	&);	// add const [gz]
		void		Set(const BYTE *);	// add const [gz]
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();

	};

class	TransferSyntax
	{
	private:
		BYTE		ItemType;	// 0x40
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			TransferSyntaxName;
		UINT		EndianType;
					TransferSyntax();
					TransferSyntax(const BYTE *);	// add const [gz]
					TransferSyntax(const UID_ &);	// add const [gz]
					~TransferSyntax();
		void		Set(const UID_	&);	// add const [gz]
		void		Set(const BYTE *);	// add const [gz]
		void		SetType(UINT	T) { EndianType = T; };
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();

	};

class	ImplementationClass
	{
	private:
		BYTE		ItemType;	// 0x52
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			ImplementationName;
		UINT		EndianType;
					ImplementationClass();
					ImplementationClass(const BYTE *);	// add const [gz]
					ImplementationClass(const UID_ &);	// add const [gz]
					~ImplementationClass();
		void		Set(const UID_	&);	// add const [gz]
		void		Set(const BYTE *);	// add const [gz]
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();

	};


class	ImplementationVersion
	{
	private:
		BYTE		ItemType;	// 0x55
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			Version;
		UINT		EndianType;
					ImplementationVersion();
					ImplementationVersion(const BYTE *);	// add const [gz]
					ImplementationVersion(const UID_ &);	// add const [gz]
					~ImplementationVersion();
		void		Set(const UID_	&);	// add const [gz]
		void		Set(const BYTE *);	// add const [gz]
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();

	};


class	SCPSCURoleSelect
	{
	private:
		BYTE		ItemType;	// 0x54
		BYTE		Reserved1;
		UINT16		Length;
	public:
		UID_			uid;
		BYTE		SCURole;
		BYTE		SCPRole;
					SCPSCURoleSelect();
					~SCPSCURoleSelect();
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32		Size();

	};


		
class	PresentationContext
	{
	private:
		BYTE							ItemType;	// 0x20
		BYTE							Reserved1;
		UINT16							Length;
	public:
		BYTE							PresentationContextID;
	private:
		BYTE							Reserved2;
		BYTE							Reserved3;
		BYTE							Reserved4;
	public:
		AbstractSyntax				AbsSyntax;
		Array<TransferSyntax>		TrnSyntax;
					PresentationContext();
					PresentationContext(AbstractSyntax &, TransferSyntax &);
					~PresentationContext();
		void		SetAbstractSyntax(AbstractSyntax	&);
		void		AddTransferSyntax(TransferSyntax	&);
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();
		
	};

class	MaximumSubLength
	{
	private:
		BYTE						ItemType;	// 0x51
		BYTE						Reserved1;
		UINT16						Length;
		UINT32						MaximumLength;
	public:
					MaximumSubLength();
					MaximumSubLength(UINT32);
					~MaximumSubLength();
		void		Set(UINT32);
		UINT32		Get();
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32	Size();
	};

class	UserInformation
	{
	private:
		BYTE						ItemType;	// 0x50
		BYTE						Reserved1;
		UINT16						Length;
	public:
		UINT32						UserInfoBaggage;
		MaximumSubLength			MaxSubLength;
		ImplementationClass			ImpClass;
		ImplementationVersion		ImpVersion;
		SCPSCURoleSelect			SCPSCURole;
	public:
					UserInformation();
					~UserInformation();
		void		SetMax(MaximumSubLength	&);
		UINT32		GetMax();
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32		Size();
	};

class	AAssociateRQ
	{
	private:
		BYTE		ItemType;		// 0x01
		BYTE		Reserved1;
		UINT32	Length;
		UINT16	ProtocolVersion;	// 0x01
		UINT16	Reserved2;
	public:
		BYTE		CalledApTitle[17];	// 16 bytes transfered
		BYTE		CallingApTitle[17];	// 16 bytes transfered
		BYTE		Reserved3[32];

		ApplicationContext			AppContext;
		Array<PresentationContext>	PresContexts;
		UserInformation				UserInfo;
	public:
					AAssociateRQ();
					AAssociateRQ(const BYTE	*, const BYTE	*);	// add const [gz]
		virtual		~AAssociateRQ();
		void		SetCalledApTitle(const BYTE	*);			// add const [gz]
		void		SetCallingApTitle(const BYTE	*);		// add const [gz]
		void		SetApplicationContext(ApplicationContext &);
		void		SetApplicationContext(const UID_ &);	// add const [gz]
		void		AddPresentationContext(PresentationContext &);
		void		ClearPresentationContexts();
		void		SetUserInformation(UserInformation &);
		bool		Write(Buffer &);
		bool		Read(Buffer &);
		bool		ReadDynamic(Buffer	&);
		UINT32		Size();
	};			



#endif //__AARQ_HPP__