/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/
/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 * Modifications signed with [gz]
 ***************************************************************************/

/************************************************************
 * Buffer Class
 *
 ************************************************************/

#	include	"base.hpp"

#	define	DEFAULT_BREAK_SIZE	32600

BufferSpace	::	BufferSpace(UINT	Alloc)
	{
	if ( Alloc )
		Data = new BYTE [ Alloc ];
	else
		Data = NULL;
	Index = 0;
	BufferSize = Alloc;
	isTemp = false;
	}

BufferSpace	::	BufferSpace()
	{
	Data = NULL;
	Index = 0;
	BufferSize = 0;
	isTemp = true;
	}

BufferSpace	::	~BufferSpace()
	{
	if ( !isTemp )
		if ( Data )
			delete Data;
	}

Buffer	::	Buffer()
	{
	BreakSize = DEFAULT_BREAK_SIZE;
	InEndian = NATIVE_ENDIAN;
	OutEndian = NATIVE_ENDIAN;
	InSize = 0;
	OutSize = 0;
	}

Buffer	::	~Buffer()
{
	Clear();							//	added by [gz]
}	

void Buffer	::	Clear()					//	added by [gz]
{
	while ( Incoming.GetSize() )
		{
//		if ( Incoming[0].Data )
//			delete Incoming[0].Data;
//		Incoming[0].Data = NULL;
		
		delete Incoming.Get(0);
		Incoming.RemoveAt( 0 );
		}
	while ( Outgoing.GetSize() )
		{
//		if ( Outgoing[0].Data )
//			delete Outgoing[0].Data;
//		Outgoing[0].Data = NULL;
		
		delete Outgoing.Get(0);
		Outgoing.RemoveAt ( 0 );
		}
	Incoming.ClearType = 0;
	Outgoing.ClearType = 0;
//	Incoming.dt.Data = NULL;
//	Outgoing.dt.Data = NULL;
	BreakSize = DEFAULT_BREAK_SIZE;
	InEndian = NATIVE_ENDIAN;		//	added by [gz]
	OutEndian = NATIVE_ENDIAN;		//	added by [gz]
	InSize = 0;						//	added by [gz]
	OutSize = 0;					//	added by [gz]
}

void	Buffer	::	SetIncomingEndian(UINT	Endian)
{
	InEndian = Endian;
}

void Buffer	::	SetOutgoingEndian(UINT	Endian)
{
	OutEndian = Endian;
}

void Buffer	::	SetBreakSize(UINT	ToSize)
	{
	BreakSize = ToSize;
	
	}

Buffer	&	Buffer	::	operator	>>	(BYTE	&x)
	{
	BYTE	s[2];

	Read(s, 1);
	x = s[0];
//	fprintf(stderr, "%2.2x ", x);
	return (*this);
	}

/*************************************************
	BufferSpace	BS;

	if (InSize <= 0)
		{
		ReadBlock();
		}
	if (InSize <= 0)
		{
		x = 0;
		return ( * this );
		}
	BS = Incoming.Get ( 0 );
	if ( ! BS.Data )
		{
		InSize -= BS.Size;
		Incoming.RemoveAt ( 0 );
		x = 0;
		return ( * this );
		}
	x = BS.Data[BS.Index];
	++BS.Index;
	--InSize;
	if(BS.Index == BS.Size)
		Incoming.RemoveAt ( 0 );
	else
		Incoming.Get ( 0 ) = BS;
	BS.isTemp = true;
	return ( * this );
	}
***************************************************/

Buffer	&	Buffer	::	operator	>>	(UINT16	&x)
	{
	BYTE	b1, b2;
	BYTE	*op;

	(*this) >> b1;
	(*this) >> b2;

	if(InEndian == NATIVE_ENDIAN)
		{
		op = (BYTE*) &x;
		(*op) = b1;++op;
		(*op) = b2;	
		}
	else
		{
		op = (BYTE*) &x;
		(*op) = b2;++op;
		(*op) = b1;	
		}

	return ( * this );
	}
/*
static	void	Dump(void	*vp, int	size)
	{
	unsigned	char	*cvp;

	cvp = (unsigned char *) vp;
	while(size)
		{
		fprintf(stderr, "%2.2x ", *cvp);
		++cvp;
		--size;
		}
	}
*/

Buffer	&	Buffer	::	operator	>>	(UINT32	&x)
	{
	UINT16	w1, w2;
	BYTE	*op, *ip;

	(*this) >> w1;
	(*this) >> w2;

	if(InEndian == NATIVE_ENDIAN)
		{
		op = (BYTE *) &x;
		ip = (BYTE *) &w1;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		ip = (BYTE *) &w2;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);
		}
	else
		{
		op = (BYTE *) &x;
		ip = (BYTE *) &w2;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		ip = (BYTE *) &w1;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);
		}
	
	return ( * this );
	}


Buffer	&	Buffer	::	operator	>>	(double	&x)	// added by [gz]
	{
	UINT32	dw1, dw2;
	BYTE	*op, *ip;

	(*this) >> dw1;
	(*this) >> dw2;

	if(InEndian == NATIVE_ENDIAN)
		{
		op = (BYTE *) &x;
		ip = (BYTE *) &dw1;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		ip = (BYTE *) &dw2;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);
		}
	else
		{
		op = (BYTE *) &x;
		ip = (BYTE *) &dw2;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		ip = (BYTE *) &dw1;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);++op;++ip;
		(*op) = (*ip);
		}
	
	return ( * this );
	}

Buffer	&	Buffer	::	operator	<<	(const BYTE	&x)	// add const [gz]
	{
	BYTE	s[2];
	
//	fprintf(stderr, "Buffer :: << Byte\n");fflush(stderr);
	s[0] = x;
	Write(s, 1);
//	fprintf(stderr, "returning\n");fflush(stderr);
	return (*this);
	}

/************************************************
	BufferSpace	BS;

	if ( ! Outgoing.GetSize() )
		{
		BufferSpace	ABS( BreakSize );
		Outgoing.Add ( ABS );
		ABS.Data = NULL;
		}
	BS = Outgoing.Get ( Outgoing.GetSize() - 1 );
	if(BS.Index == BS.Size)
		{
		BufferSpace	ABS( BreakSize );
		Outgoing.Add ( ABS );
		ABS.Data = NULL;
		BS = Outgoing.Get ( Outgoing.GetSize() - 1);
		}
	BS.Data[BS.Index] = x;
	++BS.Index;
	Outgoing.Get ( Outgoing.GetSize() - 1 ) = BS;
	BS.isTemp = true;
	++OutSize;
	return (*this );
	}
*************************************************/	
Buffer	&	Buffer	::	operator	<<	(const UINT16	&x)	// add const [gz]
	{
	BYTE	b1, b2;
	BYTE	*ip;

	ip = (BYTE*)&x;
	b1 = (*ip);++ip;
	b2 = (*ip);
	if(OutEndian == NATIVE_ENDIAN)
		{
		(*this) << b1;
		(*this) << b2;
		}
	else
		{
		(*this) << b2;
		(*this) << b1;
		}
	return ( * this );
	}

Buffer	&	Buffer	::	operator	<<	(const UINT32	&x)	// add const [gz]
	{
	UINT16	w1, w2;
	UINT16	*ip;

	ip = (UINT16*)&x;
	w1 = (*ip);++ip;
	w2 = (*ip);
	if(OutEndian == NATIVE_ENDIAN)
		{
		(*this) << w1;
		(*this) << w2;
		}
	else
		{
		(*this) << w2;
		(*this) << w1;
		}
	return ( * this );
	}

Buffer	&	Buffer	::	operator	<<	(const double	&x)	// added by [gz]
	{
	UINT32	dw1, dw2;
	UINT32	*ip;

	ip = (UINT32*)&x;
	dw1 = (*ip);++ip;
	dw2 = (*ip);
	if(OutEndian == NATIVE_ENDIAN)
		{
		(*this) << dw1;
		(*this) << dw2;
		}
	else
		{
		(*this) << dw2;
		(*this) << dw1;
		}
	return ( * this );
	}

void	Buffer	::	Flush()
	{
	BufferSpace	*BS;

	
	//fprintf(stderr, "Buffer :: Flush() %d buffers\n", Outgoing.GetSize());fflush(stderr);
	while ( Outgoing.GetSize() )
		{
		BS = Outgoing.Get ( 0 );
		if(BS->Index)
			SendBinary(BS->Data, BS->Index);
		
		delete Outgoing.Get(0);
		Outgoing.RemoveAt ( 0 );
		}
	OutSize = 0;
//	fprintf(stderr, "returning true\n");fflush(stderr);
	
	}

bool	Buffer	::	Flush(UINT	Bytes)
	{
	BufferSpace	*BS;

	//fprintf(stderr, "FLUSH: OutSize=%d, Bytes=%d\n", OutSize,Bytes);

	if(Bytes > OutSize)	
	{
		Flush();
		return true;
	}

	
	OutSize -= Bytes;

	while(Bytes)
		{
		if(!Outgoing.GetSize())
			{
			//fprintf(stderr, "Attempting to flush more bytes than input, OutSize=%d Bytes=%d\n", OutSize,Bytes);
			OutSize = 0;
			return ( false );
			}

		BS = Outgoing.Get( 0 );
		if(BS->Index > Bytes)
			{
			SendBinary(BS->Data, Bytes);
			ByteCopy(BS->Data, &BS->Data[Bytes], BS->Index - Bytes);
			BS->Index = BS->Index - Bytes;
			Bytes = 0;
			return ( true );
			}
		else
			{
			if(BS->Index)
				{
				SendBinary(BS->Data, BS->Index);
				Bytes -= BS->Index;
				delete Outgoing.Get(0);
				Outgoing.RemoveAt ( 0 );
				}
			else
				{
				delete Outgoing.Get(0);
				Outgoing.RemoveAt ( 0 );
				}
			}
		}
	
	return ( true );
	}

				
bool	Buffer	::	Kill(UINT	Bytes)
	{
	bool	Ret;
	BYTE	*Data = new BYTE[Bytes+1];

	Ret = Read(Data, Bytes);
	delete Data;
	/*
		shouldn't previous line be:
		delete[] Data;
		? (morgan)
	*/
	return ( Ret );
	}
	
/*
	BufferSpace	*BS;
	InSize -= Bytes;
	while ( Incoming.GetSize() )
		{
		BS = Incoming.Get ( 0 );
		if ( (BS->Index + Bytes) > BS->BufferSize)
			Bytes -= (BS->BufferSize - BS->Index);
		else
			{
			// NOTE, Used to be other way around...
			BS->Index += Bytes;
			Bytes = 0;
			return ( true );
			}
		delete Incoming.Get(0);
		Incoming.RemoveAt ( 0 );
		}
	
	return ( false );	// not got that many bytes yet, but go negative
	}
*/

bool	Buffer	::	Fill(UINT	Bytes)
	{
	UINT	ECount;

	SetBreakSize(Bytes);
	
	ECount = InSize + Bytes;
	while(InSize < ECount)
		{
		//fprintf(stderr, "InSize: %d\n", InSize);
		//printf("Fill %d, %d\n", ECount, Bytes);
		SetBreakSize(ECount - InSize);

		if(!ReadBlock())
			return ( false );
		//fprintf(stderr, "InSize: %d\n", InSize);
		}
	SetBreakSize(DEFAULT_BREAK_SIZE);
	return ( true );
	}

		
bool	Buffer	::	Read(BYTE	*Data, UINT	Bytes)
	{
	BufferSpace	*BS;

	//SetBreakSize(Bytes);
	while(InSize < Bytes)
		if(!ReadBlock())
			return ( false );

	while ( Bytes )
		{
		BS = Incoming.Get ( 0 );
		if(BS->Index == BS->BufferSize)
			{
			delete BS;
			Incoming.RemoveAt( 0 );
			}
		else
			{
			if((BS->Index + Bytes) <= BS->BufferSize)
				{
				ByteCopy(Data, &BS->Data[BS->Index], Bytes);
				BS->Index += Bytes;
				InSize -= Bytes;
				return ( true );
				}
			else
				{
				ByteCopy(Data, &BS->Data[BS->Index], BS->BufferSize - BS->Index);
				Bytes -= (BS->BufferSize - BS->Index);
				Data += (BS->BufferSize - BS->Index);
				InSize -= (BS->BufferSize - BS->Index);
				delete BS;
				Incoming.RemoveAt ( 0 );
				}
			}
		}
 	
	return ( false );	// should not get here
	}

bool	Buffer	::	Write(const BYTE	*Data, UINT	Bytes)	// add const [gz]
	{
	BufferSpace	*BS;
	UINT		Count;

	
	if ( !Outgoing.GetSize() )
		{
		BufferSpace	*ABS	=	new BufferSpace(BreakSize);
		Outgoing.Add ( ABS );
		}
	while(Bytes)
		{
		BS = Outgoing.Get ( Outgoing.GetSize() - 1);
		Count = BS->BufferSize - BS->Index;
		if ( Count )
			{
			if (Count <= Bytes )
				{
				ByteCopy(&BS->Data[BS->Index], Data, Count);
				Bytes -= Count;
				Data += Count;
				BS->Index += Count;
				
				
				OutSize += Count;
				BufferSpace	*ABS	=	new BufferSpace(BreakSize);
				Outgoing.Add ( ABS );
				
				}
			else
				{
				ByteCopy(&BS->Data[BS->Index], Data, Bytes);
				BS->Index += Bytes;
				OutSize += Bytes;
				Bytes = 0;
				return ( true );
				}
			}
		else
			{
			//BufferSpace	ABS(BreakSize);
			BufferSpace	*ABS	=	new BufferSpace(DEFAULT_BREAK_SIZE);
			Outgoing.Add( ABS );
			
			}
		}
	return ( true );
	}

bool	Buffer	::	ReadBlock()
	{
	int		Length, ILength;
	BYTE	s[1024];

	if(BreakSize<1024)
		ILength = BreakSize;
	else
		ILength = 1024;
	
	//printf("ReadBlock(%d)\n", Length);
	
	while(!(Length = ReadBinary((BYTE*)s, ILength)))
		;
	
	//fprintf(stderr, "Block Size = %d\n", ABS.Size);
	if(Length == (-1))
		{
		//fprintf(stderr, "Error from ReadBinary()\n");
		//printf("ReadBinary:-1\n");
		return (false);	// Error from ReadBinary()
		}
	
	BufferSpace	*ABS	=	new BufferSpace(Length);
	//fprintf(stdout, "Length=%d\n", Length);
	memcpy((void*)ABS->Data, (void*)s, Length);

	InSize += ABS->BufferSize;
	Incoming.Add(ABS);
	
	return ( true );
	}

	/*
		pointless empty function
	*/
void	Buffer	::	Poll()
	{
	/*
	BufferSpace	ABS(BreakSize);

	ABS.BufferSize = ReadBinary(ABS.Data, BreakSize);
	if(!ABS.BufferSize)
		return ( true );	// no data available
	if(ABS.BufferSize == (-1))
		return (false);	// Error from ReadBinary()
	InSize += ABS.BufferSize;
	Incoming.Add(ABS);
	ABS.Data = NULL;
	*/
	
	}
/*
class	TestBuffer	:	public	Buffer
	{

	bool	SendBinary(BYTE	*Data, UINT	Count)
		{
		UINT	Index;
		fprintf(stderr, "Sending: %d Bytes\n", Count);
		Index = 0;
		while ( Index < Count )
			{
			fprintf(stderr, "%2.2x ", Data[Index]);
			++Index;
			if(!(Index%16))
				fprintf(stderr, "\n");
			}
		fprintf(stderr, "\n");
		fflush(stderr);
		return ( true );
		}
		
	INT	ReadBinary(BYTE	*Data, UINT	Count)
		{
		fgets((char *) Data, Count, stdin);
		return ( strlen((char*) Data));
		}
	};

*/
