#ifndef BUFFER_HPP_INCLUDE_GUARD
#define BUFFER_HPP_INCLUDE_GUARD


/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/
/***************************************************************************
 *
 * Karl Franzens University, Graz
 * UCDMC99 DICOM Network Transport Libraries 
 * Version 0.1 Beta
 *
 * Technical Contact: gunter.zeilinger@kfunigraz.ac.at
 * Modifications signed with [gz]
 ***************************************************************************/

class	BufferSpace
	{
	public:
		INT			BufferSize;
		UINT		Index;
		bool		isTemp;
		BYTE		*Data;
	
				BufferSpace(UINT);
				BufferSpace();
				~BufferSpace();
	};

class	Buffer
	{
	protected:

	UINT					BreakSize;
	UINT					InEndian;
	UINT					OutEndian;

	//INT						InSize;
	UINT					InSize;

	//INT						OutSize;
	UINT					OutSize;

	Array<BufferSpace*>		Incoming;
	Array<BufferSpace*>		Outgoing;

	public:
		bool		ReadBlock();
		void		Poll();
	public:
		void		SetBreakSize(UINT);
		void		SetIncomingEndian(UINT);
		void		SetOutgoingEndian(UINT);
		inline	UINT		GetIncomingEndian()	{ return ( InEndian ); };
		inline	UINT		GetOutgoingEndian()	{ return ( OutEndian ); };
					Buffer();
		virtual		~Buffer();		
		Buffer	&	operator			>>	(BYTE	&);
		Buffer	&	operator			>>	(UINT16	&);
		Buffer	&	operator			>>	(UINT32	&);
		Buffer	&	operator			>>	(double	&);		// added by [gz]
		inline	Buffer	&	operator	>>	(char	&x)
			{ return ( (*this)>>(BYTE &) x); };
		inline	Buffer	&	operator	>>	(INT16	&x)
			{ return ( (*this)>>(UINT16 &) x); };
		inline	Buffer	&	operator	>>	(INT32	&x)
			{ return ( (*this)>>(UINT32 &) x); };

		Buffer	&	operator			<<	(const BYTE	&);		// add const [gz]
		Buffer	&	operator			<<	(const UINT16	&);	// add const [gz]
		Buffer	&	operator			<<	(const UINT32	&);	// add const [gz]
		Buffer	&	operator			<<	(const double	&);	// added by [gz]
		inline	Buffer	&	operator	<<	(const char	&x)		// add const [gz]
			{ return ( (*this)<<(const BYTE &) x); };
		inline	Buffer	&	operator	<<	(const INT16	&x)	// add const [gz]
			{ return ( (*this)<<(const UINT16 &) x); };
		inline	Buffer	&	operator	<<	(const INT32	&x)	// add const [gz]
			{ return ( (*this)<<(const UINT32 &) x); };

		inline	UINT		GetSize()	{ return ( InSize ); };
		void		Flush();
		bool		Flush(UINT	Bytes);
		bool		Kill(UINT);
		bool		Read(BYTE *, UINT);
		bool		Write(const BYTE *, UINT);					// add const [gz]
		bool		Fill(UINT);
		virtual	INT			ReadBinary(BYTE	*, UINT)	=	0;
		virtual	bool		SendBinary(BYTE	*, UINT)	=	0;
		void		Clear();									// added by [gz]

	};

#endif //BUFFER_HPP_INCLUDE_GUARD